@@ -1,5 +1,5 @@
 /* Command-line parsing definitions for Xconq.
-   Copyright (C) 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1993, 1994, 1995, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,15 +19,14 @@ enum parsestage {
 extern int checkpointinterval;
 extern int allbedesigners;
 extern int option_popup_new_game_dialog;
-extern int warnings_suppressed;
 extern char *option_game_to_host;
 extern char *option_game_to_join;
 extern int option_num_to_wait_for;
 
-extern void init_options PARAMS ((void));
-extern void parse_command_line PARAMS ((int argc, char **argv, int spec));
-extern void add_a_module PARAMS ((char *name, char *filename));
-extern void load_all_modules PARAMS ((void));
-extern void set_variants_from_options PARAMS ((void));
-extern void set_players_from_options PARAMS ((void));
-extern void print_instructions PARAMS ((void));
+extern void init_options(void);
+extern void parse_command_line(int argc, char **argv, int spec);
+extern void add_a_module(char *name, char *filename);
+extern void load_all_modules(void);
+extern void set_variants_from_options(void);
+extern void set_players_from_options(void);
+extern void print_instructions(void);
