@@ -1,5 +1,5 @@
 /* General Xconq initialization.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,34 +17,33 @@ any later version.  See the file COPYING.  */
    preventing fatal conflicts. */
 
 #include "conq.h"
-extern char *curdatestr;
 #include "kernel.h"
+#include "ai.h"
 #include "version.h"
 
-static void cache_possible_types PARAMS ((void));
-static void kill_excess_units PARAMS ((void));
-static void trial_assign_player PARAMS ((int i, Player *player));
-static void init_view_2 PARAMS ((Side *side, int terrainset));
-static void init_view_cell PARAMS ((int x, int y));
-static void maybe_init_view_cell PARAMS ((int x, int y));
-static int adj_seen_terrain PARAMS ((int x, int y, Side *side));
-static void init_ranged_views PARAMS ((void));
-
-/* This is true when the set of types has been defined.  Subsequently
-   only scenarios based on those types can be loaded. */
-
-int typesdefined = FALSE;
+static void cache_possible_types(void);
+static void kill_excess_units(void);
+static void trial_assign_player(int i, Player *player);
+static void init_view_2(Side *side, int terrainset);
+static void init_view_cell(int x, int y);
+static void maybe_init_view_cell(int x, int y);
+static int adj_seen_terrain(int x, int y, Side *side);
+static void init_ranged_views(void);
+static int indepside_needed(void);
 
 /* This is true after the game is totally synthesized, but players may
    not all be in the game yet. */
 
 int gamedefined = FALSE;
 
+enum setup_stage current_stage = initial_stage;
+
 /* These are handy tmp vars, usually used with function args. */
 
 short tmputype;
 short tmpmtype;
 short tmpttype;
+short tmpatype;
 
 /* The array of player/side matchups. */
 
@@ -99,7 +98,7 @@ short *u_possible;
 
 struct synthtype {
     int key;
-    int (*fn) PARAMS ((int calls, int runs));
+    int (*fn)(int calls, int runs);
     int dflt;
     int calls;
     int runs;
@@ -123,21 +122,26 @@ struct synthtype {
 /* Allow environment vars to override compiled-in library and game. */
 
 void
-init_library_path(path)
-char *path;
+init_library_path(char *path)
 {
     char *xconqlib;
+    LibraryPath *image_path;
 
     xconqlib = getenv("XCONQLIB");
     if (empty_string(xconqlib)) {
 	if (!empty_string(path)) {
 	    xconqlib = path;
 	} else {
-	    xconqlib = default_library_filename();
+	    xconqlib = default_library_pathname();
 	}
     }
     xconq_libs = (LibraryPath *) xmalloc(sizeof(LibraryPath));
     xconq_libs->path = copy_string(xconqlib);
+    /* Add the pathname to the binary images directory, as an
+       alternate place to look. */
+    image_path = (LibraryPath *) xmalloc(sizeof(LibraryPath));
+    image_path->path = copy_string(default_images_pathname());
+    xconq_libs->next = image_path;
     last_user_xconq_lib = NULL;
     standard_game_name = getenv("XCONQSTANDARDGAME");
     if (empty_string(standard_game_name))
@@ -145,8 +149,7 @@ char *path;
 }
 
 void
-add_library_path(path)
-char *path;
+add_library_path(char *path)
 {
     LibraryPath *lib;
 
@@ -173,7 +176,7 @@ char *path;
    by a game definition.  It should run *before* command line parsing. */
 
 void
-init_data_structures()
+init_data_structures(void)
 {
     init_xrandom(-1);
     init_lisp();
@@ -198,7 +201,7 @@ init_data_structures()
 /* Build the default list of synthesis methods. */
 
 void
-set_g_synth_methods_default()
+set_g_synth_methods_default(void)
 {
     int i;
     Obj *synthlist = lispnil, *synthlistend = lispnil, *tmp;
@@ -220,20 +223,21 @@ set_g_synth_methods_default()
 }
 
 /* Run a doublecheck on plausibility of game parameters.  Additional
-   checks are performed elsewhere as needed, for instance during random
-   generation.  Serious mistakes exit now, since they can cause all sorts
-   of strange behavior and core dumps.  It's a little more friendly to only
-   exit at the end of the tests, so all the mistakes can be found at once. */
+   checks are performed elsewhere as needed, for instance during
+   random generation.  Serious mistakes exit now, since they can cause
+   all sorts of strange behavior and core dumps.  It's a little more
+   friendly to only exit at the end of the tests, so all the mistakes
+   can be found at once. */
 
 /* In theory, if a game passes these tests, then Xconq will never crash. */
 
 void
-check_game_validity()
+check_game_validity(void)
 {
     int failed = FALSE, movers = FALSE, actors = FALSE;
-    int u1, u2, m1, t1, t2, i, methkey, found;
+    int u1, u2, t1, t2, i, methkey, found;
     Obj *synthlist = g_synth_methods(), *methods, *method;
-
+    
     /* We must have at least one kind of unit. */
     if (numutypes < 1) {
 	init_error("no unit types have been defined");
@@ -313,11 +317,6 @@ check_game_validity()
 	    init_error("unit type %d must always be available", u1);
 	    failed = TRUE;
 	}
-	/* Can't make use of this yet, so error out if anybody tries. */
-	if (u_action_priority(u1) != 0) {
-	    init_error("unit type %d cannot have a nonzero action priority", u1);
-	    failed = TRUE;
-	}
 	/* should be part of general bounds check */
 	if (u_cp(u1) <= 0) {
 	    init_error("unit type %d has nonpositive cp", u1);
@@ -341,20 +340,8 @@ check_game_validity()
 	if (u_acp(u1) > 0) {
 	    actors = TRUE;
 	}
-    }
-    /* Check various material type properties. */
-    for_all_material_types(m1) {
-	/* Can't make use of this yet, so error out if anybody tries. */
-	if (m_available(m1) != 1) {
-	    init_error("material type %d not always available", m1);
-	    failed = TRUE;
-	}
-    }
-    /* Check various terrain type properties. */
-    for_all_terrain_types(t1) {
-	/* Can't make use of this yet, so error out if anybody tries. */
-	if (t_available(t1) != 1) {
-	    init_error("terrain type %d not always available", t1);
+	if (u_wrecked_type(u1) == u1) {
+	    init_error("unit type %d wrecks to same type.", u1);
 	    failed = TRUE;
 	}
     }
@@ -371,27 +358,30 @@ check_game_validity()
 	init_error("no terrain type has been allowed for cells");
 	failed = TRUE;
     }
-    /* This is a really bad game definition, leave before we crash.  This would
-       only be executed on systems where init_error doesn't exit immediately. */
+    /* This is a really bad game definition, leave before we crash.
+       This would only be executed on systems where init_error doesn't
+       exit immediately. */
     if (failed) {
 	exit(0);
     }
     Dprintf("Finished checking game design.\n");
-    Dprintf("It defines %d unit types, %d material types, %d terrain types.\n",
+    Dprintf("It defines %d unit types, %d material types, %d terrain types",
 	    numutypes, nummtypes, numttypes);
+    Dprintf(", %d advance types.\n", numatypes);
 }
 
 /* Calculate the values of global variables that are used everywhere. */
 
 void
-calculate_globals()
+calculate_globals(void)
 {
-    /* The game is now completely defined; no further user-specified changes can
-       occur. */
+    /* The game is now completely defined; no further user-specified
+       changes can occur. */
     gamedefined = TRUE;
     calculate_world_globals();
-    /* This needs to be precalculated instead of as-needed, since range can be
-       validly both negative and positive, so no way to distinguish uninitialized. */
+    /* This needs to be precalculated instead of as-needed, since
+       range can be validly both negative and positive, so no way to
+       distinguish uninitialized. */
     {
 	int u1, u2, range;
     	extern int max_detonate_on_approach_range;
@@ -400,7 +390,8 @@ calculate_globals()
 	for_all_unit_types(u1) {
 	    for_all_unit_types(u2) {
 		range = uu_detonate_approach_range(u1, u2);
-		max_detonate_on_approach_range = max(range, max_detonate_on_approach_range);
+		max_detonate_on_approach_range =
+		  max(range, max_detonate_on_approach_range);
 	    }
 	}
     }	
@@ -411,10 +402,10 @@ calculate_globals()
    if called more than once. */
 
 void
-patch_object_references()
+patch_object_references(void)
 {
     int numhere, numoccs, t, id;
-    Unit *unit, *transport, *unit2;
+    Unit *unit, *transport;
     Side *side;
     Obj *utref, *utorig;
 
@@ -568,9 +559,7 @@ patch_object_references()
 		  unit->act->acp = u_acp(unit->type);
 		/* Restore initacp that wasn't written out because it
 		   was the normal value. */
-		if (unit->act
-		    && unit->act->acp > 0
-		    && unit->act->initacp == 0)
+		if (unit->act && unit->act->initacp < u_acp_min(unit->type))
 		  unit->act->initacp = u_acp(unit->type);
 		/* Might already have a plan, leave alone if so. */
 		if (unit->plan == NULL) {
@@ -613,39 +602,58 @@ patch_object_references()
 /* (when does locking get done?) */
 
 void
-make_trial_assignments()
+make_trial_assignments(void)
 {
     int i = 0;
     Side *side;
     Player *player;
 
-    /* Fill in the side's predefined default and range of initial advantage. */
+    Dprintf("Making trial side/player assignments\n");
+    /* Fill in the sides' predefined default and range of initial
+       advantage. */
     for_all_sides(side) {
     	init_side_advantage(side);
     }
-    /* Ensure we have as many sides as will be required. */
+    /* Ensure we have as many sides as will be required. Only real
+       sides increment numsides, so it will equal g_sides_min when we
+       are done. However, the total number of sides including
+       indepside (numtotsides) will equal g_sides_min + 1. */
     while (numsides < g_sides_min()) {
 	make_up_a_side();
     }
-    while (numsides < min(numplayers, g_sides_max())) {
+    /* Note: numplayers - 1 since the indepside now has a player. */
+    while (numsides < min(numplayers - 1, g_sides_max())) {
 	make_up_a_side();
     }
+    /* Kick indepside out of the game if it is unnecessary. */
+    if (!indepside_needed())
+      indepside->ingame = FALSE;
+
     /* Put in all the sides. */
     for_all_sides(side) {
 	assignments[i++].side = side;
     }
-    /* If no players have been created, make one that is human-run,
-       presumably by the person who started up this program. */
-    if (numplayers == 0) {
+    /* First create a player to handle indepside if necessary. */
+    if (numplayers < indepside->ingame) {
+	add_player();
+    }
+    /* Add remote players. */
+    if (numremotes > 0) {
+    	add_remote_players();
+    }
+    /* If only one player (the indepside player) has been created so
+       far, make one that is human-run, presumably by the person who
+       started up this program. */
+    if (numplayers == indepside->ingame) {
 	add_default_player();
     }
-    if (numsides < numplayers) {
+    if (numtotsides < numplayers) {
 	/* We have too many players. */
 	init_warning("too many players (%d) for %d sides, ignoring extra",
-		     numplayers, numsides);
+		     numplayers, numtotsides);
     }
     /* Make any prespecified assignments. */
-    for (i = 0; i < numsides; ++i) {
+    for (i = 0; i < numtotsides; ++i) {
 	if (assignments[i].side != NULL
 	    && assignments[i].side->player != NULL
 	    && assignments[i].player == NULL) {
@@ -655,7 +663,7 @@ make_trial_assignments()
     }
     /* Assign any remaining players. */
     player = playerlist;
-    for (i = 0; i < numsides; ++i) {
+    for (i = 0; i < numtotsides; ++i) {
 	if (assignments[i].side != NULL
 	    && assignments[i].side->ingame
 	    && assignments[i].player == NULL) {
@@ -670,7 +678,7 @@ make_trial_assignments()
     /* Add default players for sides with none (in separate loop so player
        creation doesn't confuse player list traversal above).  Don't
        add for any sides not actually participating in the game. */
-    for (i = 0; i < numsides; ++i) {
+    for (i = 0; i < numtotsides; ++i) {
 	if (assignments[i].side != NULL
 	    && assignments[i].side->ingame
 	    && assignments[i].player == NULL) {
@@ -681,6 +689,14 @@ make_trial_assignments()
 	    trial_assign_player(i, player);
 	}
     }
+    /* Finally set the indepside AI type correctly. */
+    if (indepside->ingame) {
+	if (g_indepside_has_ai() && indepside->ingame) {
+	    indepside->player->aitypename = "iplayer";
+    	} else {
+	    indepside->player->aitypename = NULL;
+    	}
+    }
     /* This warning can happen if there are more players than active
        sides. */
     for_all_players(player) {
@@ -692,10 +708,39 @@ make_trial_assignments()
        rearranged if desired. */
 }
 
+
+/* Returns true if indepside already has units or may get units made. */
+
+static int
+indepside_needed(void)
+{
+    Unit *unit;
+    int	u, t;
+	
+    /* First check what the game designer thought about this. */
+    if (g_no_indepside_ingame())
+      return FALSE;
+    /* Then check existing (preassigned) units. */
+    for_all_units(unit) {
+	if (unit->side == indepside)
+	  return TRUE;
+    }
+    /* Then check if independent units will ever be made. */
+    for_all_unit_types(u) {
+	if (u_indep_growth(u) > 0 || u_indep_near_start(u) > 0)
+	  return TRUE;
+	for_all_terrain_types(t) {
+	    if (ut_indep_density(u, t) > 0 
+		&& ut_vanishes_on(u, t) != TRUE) {
+		return TRUE;
+	    }
+	}
+    }
+    return FALSE;
+}
+
 static void
-trial_assign_player(i, player)
-int i;
-Player *player;
+trial_assign_player(int i, Player *player)
 {
     assignments[i].player = player;
     player->side = assignments[i].side;
@@ -714,7 +759,7 @@ Player *player;
 /* Create a random side with default characteristics. */
 
 Side *
-make_up_a_side()
+make_up_a_side(void)
 {
     extern Obj *side_defaults;
 
@@ -727,8 +772,6 @@ make_up_a_side()
     fill_in_side(side, side_defaults, FALSE);
     make_up_side_name(side);
     init_side_advantage(side);
-    /* A newly-created side starts out in the game, can drop out later. */
-    side->ingame = TRUE;
     Dprintf("Made up a side %s\n", side_desig(side));
     return side;
 }
@@ -737,8 +780,7 @@ make_up_a_side()
    global values. */
 
 void
-init_side_advantage(side)
-Side *side;
+init_side_advantage(Side *side)
 {
     /* Set up the default and range of initial advantages. */
     if (side->advantage == 0)
@@ -778,7 +820,8 @@ Side *side;
 	    }
 	}
     }
-    /* If we're not getting any guidance from global defaults, just set to 1. */
+    /* If we're not getting any guidance from global defaults, just
+       set to 1. */
     if (side->minadvantage == 0)
       side->minadvantage = 1;
     if (side->maxadvantage == 0)
@@ -788,7 +831,7 @@ Side *side;
 /* Add a side and a player to go with it (used by interfaces). */
 
 int
-add_side_and_player()
+add_side_and_player(void)
 {
     int n;
     Side *side;
@@ -796,8 +839,8 @@ add_side_and_player()
 
     side = make_up_a_side();
     if (side == NULL)
-      return FALSE;
-    n = numsides - 1;
+      return -1;
+    n = numtotsides - 1;
     assignments[n].side = side;
     player = add_player();
     assignments[n].player = player;
@@ -808,15 +851,53 @@ add_side_and_player()
     if (player->advantage == 0) {
 	player->advantage = side->advantage;
     }
-    return TRUE;
+    /* Return the new side's position in the assignment array. */
+    return n;
+}
+
+void
+set_player_advantage(int n, int newadv)
+{
+    Player *player;
+
+    player = assignments[n].player;
+    if (player != NULL)
+      player->advantage = newadv;
+    /* (should complain otherwise) */
+    /* (should check limits) */
+}
+
+void
+rename_side_for_player(int n)
+{
+    Side *side;
+
+    side = assignments[n].side;
+    side->name = side->noun = side->pluralnoun = side->adjective = NULL;
+    make_up_side_name(side);
+}
+
+/* Given the name of an AI type, assign it to the player at the given
+   position in the assignments array. */
+
+void
+set_ai_for_player(int n, char *aitype)
+{
+    Player *player;
+
+    player = assignments[n].player;
+    if (empty_string(aitype) || find_ai_type(aitype) > 0) {
+	player->aitypename = aitype;
+    } else {
+	init_warning("\"%s\" is not a known AI type name", aitype);
+    }
 }
 
 /* This can be used by interfaces to exchange players between one side and
    another. */
 
 int
-exchange_players(n, n2)
-int n, n2;
+exchange_players(int n, int n2)
 {
     int i;
     Player *tmpplayer = assignments[n].player;
@@ -824,15 +905,22 @@ int n, n2;
     if (n < 0)
       n = 0;
     if (n2 < 0) {
-	for (i = n + 1; i <= numsides + n; ++i) {
-	    n2 = i % numsides;
+	for (i = n + 1; i <= numtotsides + n; ++i) {
+	    n2 = i % numtotsides;
 	    if (assignments[n2].side && (assignments[n2].side)->ingame)
 	      break;
 	}
 	/* No sides to exchange with, return. */
-    	if (i == numsides + n)
+    	if (i == numtotsides + n)
 	  return -1;
     }
+
+#if 0
+    /* Don't switch with the indepside player. */
+    if (n2 == 0)
+    	n2 = 1;
+#endif
+
     assignments[n].player = assignments[n2].player;
     assignments[n2].player = tmpplayer;
     /* Doesn't seem like these should be needed, but they are. */
@@ -844,7 +932,7 @@ int n, n2;
 }
 
 int
-remove_side_and_player()
+remove_side_and_player(void)
 {
     /* (how to do this?) */
     /* (would need to renumber sides etc) */
@@ -855,11 +943,36 @@ remove_side_and_player()
    the player(s). */
 
 void
-run_synth_methods()
+run_synth_methods(void)
 {
-    int i, methkey, found = FALSE, rslt;
-    Obj *synthlist, *methods, *method, *parms;
+    int i, methkey, found, rslt;
+    Obj *restlist, *done, *synthlist, *methods, *method, *donelist;
 
+    /* Restore any saved state for synthmethods. */
+    if (!gameinited) {
+	for_all_list(g_synth_done(), restlist) {
+	    done = car(restlist);
+	    method = car(done);
+	    found = FALSE;
+	    if (symbolp(method)) {
+		methkey = keyword_code(c_string(method));
+		for (i = 0; synthmethods[i].key >= 0; ++i) {
+		    if (methkey == synthmethods[i].key) {
+			found = TRUE;
+			synthmethods[i].calls = c_number(cadr(done));
+			synthmethods[i].runs = c_number(caddr(done));
+			break;
+		    }
+		}
+	    } else {
+		init_warning("non-symbol in synthesis list");
+	    }
+	    if (!found) {
+		sprintlisp(spbuf, method, BUFSIZE);
+		init_warning("bad synthesis method %s, ignoring", spbuf);
+	    }
+	}
+    }
     /* Make sure no dead units get saved. */
     flush_dead_units();
     /* Make a consistent ordering of units. */
@@ -868,40 +981,43 @@ run_synth_methods()
     Dprintf("Will run syntheses ");
     Dprintlisp(synthlist);
     Dprintf("\n");
+    donelist = lispnil;
     for_all_list(synthlist, methods) {
 	method = car(methods);
+	found = FALSE;
 	if (symbolp(method)) {
 	    methkey = keyword_code(c_string(method));
 	    for (i = 0; synthmethods[i].key >= 0; ++i) {
 		if (methkey == synthmethods[i].key) {
 		    found = TRUE;
-		    rslt = (*synthmethods[i].fn)(synthmethods[i].calls, synthmethods[i].runs);
-		    ++(synthmethods[i].calls);
-		    if (rslt)
-		      ++(synthmethods[i].runs);
+		    if (!gameinited && synthmethods[i].calls == 0) {
+			rslt = (*synthmethods[i].fn)(synthmethods[i].calls,
+						     synthmethods[i].runs);
+			++(synthmethods[i].calls);
+			if (rslt)
+			  ++(synthmethods[i].runs);
+		    }
 		    break;
 		}
 	    }
-	} else if (consp(method)) {
-	    parms = cdr(method);
-	    if (stringp(car(method))) {
-		/* External program. */
-		/* (should format parms, compose outputfile redirection,
-		    call program with os.c function, check return code,
-		    open and read output file) */
-		run_error("No external synth programs yet");
-	    }
+	    donelist = cons(cons(method,
+				 cons(new_number(synthmethods[i].calls),
+				      cons(new_number(synthmethods[i].runs),
+					   lispnil))),
+			    donelist);
+	} else {
+	    init_warning("non-symbol in synthesis list");
 	}
 	if (!found) {
 	    sprintlisp(spbuf, method, BUFSIZE);
 	    init_warning("bad synthesis method %s, ignoring", spbuf);
 	}
     }
+    set_g_synth_done(reverse(donelist));
 }
 
 int
-get_synth_method_uses(methkey, calls, runs)
-int methkey, *calls, *runs;
+get_synth_method_uses(int methkey, int *calls, int *runs)
 {
     int i;
 
@@ -916,8 +1032,7 @@ int methkey, *calls, *runs;
 }
 
 int
-make_weather(calls, runs)
-int calls, runs;
+make_weather(int calls, int runs)
 {
     int x, y, winddir, t, var;
     extern int maxclouds;	
@@ -960,8 +1075,7 @@ int calls, runs;
 /* Set the starting date/time to a random value within a given range. */
 
 int
-make_random_date(calls, runs)
-int calls, runs;
+make_random_date(int calls, int runs)
 {
     int n;
 
@@ -984,7 +1098,7 @@ int calls, runs;
 /* The final init cleans up various stuff. */
 
 void
-final_init()
+final_init(void)
 {
     Side *side;
 
@@ -1031,7 +1145,7 @@ final_init()
 }
 
 static void
-kill_excess_units()
+kill_excess_units(void)
 {
     Unit *unit;
 
@@ -1049,7 +1163,7 @@ kill_excess_units()
 /* Load up any player-specified configuration data. */
 
 void
-configure_sides()
+configure_sides(void)
 {
     Side *side;
 
@@ -1061,30 +1175,30 @@ configure_sides()
 /* Calculate what each side knows about the world. */
 
 void
-init_all_views()
+init_all_views(void)
 {
-    int x, y, i = 0, todo = max(1, numsides * area.numcells);
+    int x, y, i = 0, todo = max(1, numtotsides * area.numcells);
     int terrainset;
     Side *side;
 
-    /* Snapshot the value here, so is permanently on. */
-    all_see_all = g_see_all();
     /* Set up the basic view structures for all sides first. */
     for_all_sides(side) {
-	if (all_see_all) {
+	if (g_see_all()) {
 	    side->see_all = TRUE;
 	} else {
 	    side->see_all = FALSE;
 	    terrainset = init_view(side);
 	    init_view_2(side, terrainset);
 	}
-	/* We're not normally allowed to choose whether to see all or not. */
-	side->may_set_see_all = FALSE;
+	/* We're not normally allowed to choose whether to display all
+           or not. */
+	side->may_set_show_all = FALSE;
+	side->show_all = side->see_all;
 	/* Can't think of any other place to put this... */
 	calc_start_xy(side);
     }
-    /* Nothing more to do here. */
-    if (all_see_all)
+    /* Possibly nothing more to do here. */
+    if (g_see_all())
       return;
     announce_lengthy_process("Computing current view at each location");
     /* (coverage is also done at this point) */
@@ -1101,9 +1215,7 @@ init_all_views()
 }
 
 static void
-init_view_2(side, terrainset)
-Side *side;
-int terrainset;
+init_view_2(Side *side, int terrainset)
 {
     int x, y;
 
@@ -1131,8 +1243,7 @@ int terrainset;
 /* (should only be run once/cell/side, use a scratch layer to keep track?) */
 
 static void
-init_view_cell(x, y)
-int x, y;
+init_view_cell(int x, int y)
 {
     int u, chance;
     Unit *unit;
@@ -1153,22 +1264,21 @@ int x, y;
 	    	return;
 	    }
 	    chance = already_seen_chance(tmpside, unit);
-	    if (probability(chance)) {
+	    if (!tmpside->unit_view_restored && probability(chance)) {
 	    	see_exact(tmpside, x, y);
-	    	/* This view might be overwritten by a view of
-	    	   another might-be-seen unit at this location,
-	    	   so don't return just yet. */
+	    	/* This view might be overwritten by a view of another
+	    	   might-be-seen unit at this location, so don't
+	    	   return just yet. */
 	    }
 	}
     }
     /* We get to see the weather conditions here. */
-    set_wind_view(tmpside, x, y, raw_wind_at(x, y));
+    if (winds_defined())
+      set_wind_view(tmpside, x, y, raw_wind_at(x, y));
 }
 
 int
-already_seen_chance(side, unit)
-Side *side;
-Unit *unit;
+already_seen_chance(Side *side, Unit *unit)
 {
     int u = unit->type, val;
 
@@ -1192,8 +1302,7 @@ Unit *unit;
 }
 
 static void
-maybe_init_view_cell(x, y)
-int x, y;
+maybe_init_view_cell(int x, int y)
 {
     int dir, x1, y1;
 
@@ -1208,13 +1317,11 @@ int x, y;
 }
 
 static int
-adj_seen_terrain(x, y, side)
-int x, y;
-Side *side;
+adj_seen_terrain(int x, int y, Side *side)
 {
     int dir, x1, y1;
 
-    if (!inside_area(x, y) || side == NULL)
+    if (!inside_area(x, y))
       return FALSE;
     for_all_directions(dir) {
 	if (point_in_dir(x, y, dir, &x1, &y1)) {
@@ -1228,7 +1335,7 @@ Side *side;
 /* Do ranged initial views from units. */
 
 static void
-init_ranged_views()
+init_ranged_views(void)
 {
     int rad, x, y, pop, dir, x1, y1, i = 0;
     Unit *unit;
@@ -1291,7 +1398,7 @@ init_ranged_views()
 			  side->finalradius - 2, side->finalradius + 2,
 			  maybe_init_view_cell);
 	}
-	announce_progress((100 * i++) / numsides);
+	announce_progress((100 * i++) / numtotsides);
     }
     finish_lengthy_process();
 }
@@ -1299,10 +1406,9 @@ init_ranged_views()
 /* Method to give all units and terrain a basic stockpile of supply. */
 
 int
-make_initial_materials(calls, runs)
-int calls, runs;
+make_initial_materials(int calls, int runs)
 {
-    int m, t, amts[MAXTTYPES], doany, x, y;
+    int m, t, amts[MAXTTYPES], doany, x, y, didany = FALSE;
 
     /* Go over each material and terrain type, looking for nonzero
        material in terrain possibilities, then alloc and fill in layers
@@ -1320,17 +1426,17 @@ int calls, runs;
 		t = terrain_at(x, y);
 		set_material_at(x, y, m, amts[t]);
 	    }
+	    didany = TRUE;
 	}
     }
-    return TRUE;
+    return didany;
 }
 
 /* Give the unit what it is declared to have stockpiled
    at the start of a game. */
 
 void
-init_supply(unit)
-Unit *unit;
+init_supply(Unit *unit)
 {
     int m, u = unit->type;
 
@@ -1342,11 +1448,11 @@ Unit *unit;
 /* Quicky test needed in a couple places. */
 
 int
-saved_game()
+saved_game(void)
 {
     FILE *fp;
 
-    fp = fopen(saved_game_filename(), "r");
+    fp = open_file(saved_game_filename(), "r");
     if (fp != NULL) {
 	fclose(fp);
 	return TRUE;
@@ -1359,9 +1465,9 @@ saved_game()
    if all the balance sheets are completely blank. */
 
 void
-init_side_balance()
+init_side_balance(void)
 {
-    int u, anything = FALSE;
+    int u;
     Unit *unit;
     Side *side;
 
@@ -1382,7 +1488,7 @@ init_side_balance()
    caused some sort of disaster. */
 
 void
-check_consistency()
+check_consistency(void)
 {
     int x, y;
 
@@ -1397,17 +1503,17 @@ check_consistency()
 	}
 	add_edge_terrain();
     }
-    if (numsides <= 0) {
+    if (numtotsides <= 0) {
 	init_error("There are no player sides at all in this game");
-    } else if (numsides < numplayers) {
+    } else if (numtotsides < numplayers) {
 	init_warning("Only made %d of the %d sides requested",
-		     numsides, numplayers);
+		     numtotsides, numplayers);
     }
     /* Need any kind of unit checks? */
 }
 
 static void
-cache_possible_types()
+cache_possible_types(void)
 {
     int u, u2, rescan;
     Unit *unit;
@@ -1437,15 +1543,15 @@ cache_possible_types()
    side structures appropriately. */
 
 void
-assign_players_to_sides()
+assign_players_to_sides(void)
 {
     int i, numdisplays = 0, numdisplayswanted = 0, numais = 0, n = 0;
     Side *side;
     Player *player;
 
     announce_lengthy_process("Assigning players to sides");
-    for (i = 0; i < numsides; ++i) {
-	announce_progress((100 * i) / numsides);
+    for (i = 0; i < numtotsides; ++i) {
+	announce_progress((100 * i) / numtotsides);
 	side = assignments[i].side;
 	player = assignments[i].player;
 	if (player == NULL)
@@ -1464,12 +1570,11 @@ assign_players_to_sides()
 			 player->advantage, player_desig(player), side->maxadvantage);
 	    player->advantage = side->maxadvantage;
 	}
-	side->rid = player->rid;
 	/* Call the interface code to initialize the side's display, if
 	   it wants to use one (the interface has to decide). */
 	if (side_wants_display(side)) {
 	    ++numdisplayswanted;
-	    if (side->rid == my_rid) {
+	    if (player->rid == my_rid) {
 		init_ui(side);
 	    } else {
 		init_remote_ui(side);
@@ -1483,7 +1588,7 @@ assign_players_to_sides()
 	    ++numais;
 	}
 	Dprintf("Assigned %s to %s\n",
-		 side_desig(side), player_desig(player));
+		side_desig(side), player_desig(player));
     }
     finish_lengthy_process();
     if (numdisplays < numdisplayswanted) {
@@ -1495,7 +1600,7 @@ assign_players_to_sides()
 			 numdisplays, numdisplayswanted);
 	}
     } else if (numdisplays == 0) {
-	init_warning("Need at least one display to run");
+	init_warning("Need at least one local or remote display to run");
     }
 #ifdef DESIGNERS
     /* Make each displayed side into a designer if it was requested. */
@@ -1509,7 +1614,7 @@ assign_players_to_sides()
 #endif /* DESIGNERS */
     if (numais > 0) {
 	announce_lengthy_process("Setting up AIs");
-	for (i = 0; i < numsides; ++i) {
+	for (i = 0; i < numtotsides; ++i) {
 	    if (numais > 1)
 	      announce_progress((100 * n++) / numais);
 	    side = assignments[i].side;
@@ -1530,7 +1635,7 @@ assign_players_to_sides()
    to allocate their working space. */
 
 void
-prealloc_debug()
+prealloc_debug(void)
 {
     side_desig(NULL);
     player_desig(NULL);
@@ -1540,7 +1645,7 @@ prealloc_debug()
 /* Return the program version. */
 
 char *
-version_string()
+version_string(void)
 {
     return VERSION;
 }
@@ -1548,7 +1653,7 @@ version_string()
 /* Return the copyright notice. */
 
 char *
-copyright_string()
+copyright_string(void)
 {
     return COPYRIGHT;
 }
@@ -1556,7 +1661,7 @@ copyright_string()
 /* Return the license string. */
 
 char *
-license_string()
+license_string(void)
 {
     return "\
 Xconq is free software and you are welcome to distribute copies of it\n\
