@@ -1,5 +1,5 @@
 /* Support for Lisp-style data.
-   Copyright (C) 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -12,21 +12,22 @@ any later version.  See the file COPYING.  */
 #include "config.h"
 #include "misc.h"
 #include "lisp.h"
-extern char *escaped_symbol PARAMS ((char *str));
-extern char *escaped_string PARAMS ((char *str));
-extern void interp_short_array PARAMS ((short *arr, Obj *lis, int n));
-extern void interp_long_array PARAMS ((long *arr, Obj *lis, int n));
-extern int eval_number PARAMS ((Obj *val, int *isnumber));
 
-static Symentry *lookup_string PARAMS ((char *str));
-static int hash_name PARAMS ((char *str));
+/* Declarations of local functions. */
 
-static int strmgetc PARAMS ((Strm *strm));
-static void strmungetc PARAMS ((int ch, Strm *strm));
-static void sprintf_context PARAMS ((char *buf, int n, int *start, int *end, Strm *strm));
-static Obj *read_form_aux PARAMS ((Strm *strm));
-static Obj *read_list PARAMS ((Strm *strm));
-static int read_delimited_text PARAMS ((Strm *strm, char *delim, int spacedelimits, int eofdelimits));
+static Obj *newobj(void);
+static Symentry *lookup_string(char *str);
+static int hash_name(char *str);
+
+static int strmgetc(Strm *strm);
+static void strmungetc(int ch, Strm *strm);
+static void sprintf_context(char *buf, int n, int *start, int *end,
+			    Strm *strm);
+static Obj *read_form_aux(Strm *strm);
+static Obj *read_list(Strm *strm);
+static int read_delimited_text(Strm *strm, char *delim, int spacedelimits,
+			       int eofdelimits);
+static void internal_type_error(char *funname, Obj *x, char *typename);
 
 /* Pointer to "nil", the empty list. */
 
@@ -75,7 +76,7 @@ static char *escapedthingbuf;
 /* Allocate a new Lisp object, count it as such. */
 
 static Obj *
-newobj()
+newobj(void)
 {
     lispmalloc += sizeof(Obj);
     return ((Obj *) xmalloc(sizeof(Obj)));
@@ -84,7 +85,7 @@ newobj()
 /* Pre-create some objects that should always exist. */
 
 void
-init_lisp()
+init_lisp(void)
 {
     /* Allocate Lisp's NIL. */
     lispnil = newobj();
@@ -109,8 +110,7 @@ init_lisp()
 /* Ultra-simple "streams" that can be stdio FILEs or strings. */
 
 static int
-strmgetc(strm)
-Strm *strm;
+strmgetc(Strm *strm)
 {
     int ch;
 
@@ -133,9 +133,7 @@ Strm *strm;
 }
 
 static void
-strmungetc(ch, strm)
-int ch;
-Strm *strm;
+strmungetc(int ch, Strm *strm)
 {
     if (strm->type == stringstrm) {
 	--strm->ptr.sp;
@@ -145,14 +143,12 @@ Strm *strm;
     --(strm->numread);
 }
 
-/* El cheapo Lisp reader.  Lisp objects are generally advertised by their
-   first characters, but lots of semantics actions happen while reading, so
-   this isn't really a regular expression reader. */
+/* El cheapo Lisp reader.  Lisp objects are generally advertised by
+   their first characters, but lots of semantics actions happen while
+   reading, so this isn't really a regular expression reader. */
 
 Obj *
-read_form(fp, p1, p2)
-FILE *fp;
-int *p1, *p2;
+read_form(FILE *fp, int *p1, int *p2)
 {
     Obj *rslt;
     Strm tmpstrm;
@@ -175,9 +171,7 @@ int *p1, *p2;
 }
 
 Obj *
-read_form_from_string(str, p1, p2, endstr)
-char *str, **endstr;
-int *p1, *p2;
+read_form_from_string(char *str, int *p1, int *p2, char **endstr)
 {
     Obj *rslt;
     Strm tmpstrm;
@@ -203,10 +197,7 @@ int *p1, *p2;
 }
 
 static void
-sprintf_context(buf, n, start, end, strm)
-char *buf;
-int n, *start, *end;
-Strm *strm;
+sprintf_context(char *buf, int n, int *start, int *end, Strm *strm)
 {
     int printedlineno = FALSE;
 
@@ -233,9 +224,11 @@ Strm *strm;
     strcat(buf, ")");
 }
 
+/* The main body of the the Lisp reader, works from a stream and returns
+   an object. */
+
 static Obj *
-read_form_aux(strm)
-Strm *strm;
+read_form_aux(Strm *strm)
 {
     int minus, factor, commentclosed, ch, ch2, ch3, ch4, num;
     int numdice, dice, indice;
@@ -356,6 +349,16 @@ Strm *strm;
 	    if (minus)
 	      num = 0 - num;
 	    if (numdice > 0) {
+		/* Warn about out-of-bounds values. */
+		if (!between(0, numdice, 7))
+		  init_warning("Number of dice in %dd%d+%d is %d, not between 0 and 7",
+			       numdice, dice, num, numdice);
+		if (!between(0, dice, 15))
+		  init_warning("Dice size in %dd%d+%d is %d, not between 0 and 15",
+			       numdice, dice, num, dice);
+		if (!between(0, num, 127))
+		  init_warning("Dice addon in %dd%d+%d is %d, not between 0 and 127",
+			       numdice, dice, num, num);
 	    	num = (1 << 14) | (numdice << 11) | (dice << 7) | (num & 0x7f);
 	    } else {
 	    	num = factor * num;
@@ -390,14 +393,13 @@ Strm *strm;
     return lispeof;
 }
 
-/* Read a sequence of expressions terminated by a closing paren.  This works
-   by looping; although recursion is more elegant, if the compiler does not
-   turn tail-recursion into loops, long lists can blow the stack.  (This has
-   happened with real saved games.) */
+/* Read a sequence of expressions terminated by a closing paren.  This
+   works by looping; although recursion is more elegant, if the
+   compiler does not turn tail-recursion into loops, long lists can
+   blow the stack.  (This has happened with real saved games.) */
 
 static Obj *
-read_list(strm)
-Strm *strm;
+read_list(Strm *strm)
 {
     Obj *thecar, *thenext, *lis, *endlis;
 
@@ -434,10 +436,8 @@ Strm *strm;
    possibly also by whitespace or EOF. */
 
 static int
-read_delimited_text(strm, delim, spacedelimits, eofdelimits)
-Strm *strm;
-char *delim;
-int spacedelimits, eofdelimits;
+read_delimited_text(Strm *strm, char *delim, int spacedelimits,
+		    int eofdelimits)
 {
     int ch, octch, j = 0, warned = FALSE;
 
@@ -483,8 +483,7 @@ int spacedelimits, eofdelimits;
 /* The usual list length function. */
 
 int
-length(list)
-Obj *list;
+length(Obj *list)
 {
     int rslt = 0;
 
@@ -499,8 +498,7 @@ Obj *list;
 /* Basic allocation routines. */
 
 Obj *
-new_string(str)
-char *str;
+new_string(char *str)
 {
     Obj *new = newobj();
 
@@ -510,8 +508,7 @@ char *str;
 }
 
 Obj *
-new_number(num)
-int num;
+new_number(int num)
 {
     Obj *new = newobj();
 
@@ -521,8 +518,7 @@ int num;
 }
 
 Obj *
-new_utype(u)
-int u;
+new_utype(int u)
 {
     Obj *new = newobj();
 
@@ -532,19 +528,17 @@ int u;
 }
 
 Obj *
-new_mtype(r)
-int r;
+new_mtype(int m)
 {
     Obj *new = newobj();
 
     new->type = MTYPE;
-    new->v.num = r;
+    new->v.num = m;
     return new;
 }
 
 Obj *
-new_ttype(t)
-int t;
+new_ttype(int t)
 {
     Obj *new = newobj();
 
@@ -554,9 +548,17 @@ int t;
 }
 
 Obj *
-new_pointer(sym, ptr)
-Obj *sym;
-char *ptr;
+new_atype(int a)
+{
+    Obj *new = newobj();
+
+    new->type = ATYPE;
+    new->v.num = a;
+    return new;
+}
+
+Obj *
+new_pointer(Obj *sym, char *ptr)
 {
     Obj *new = newobj();
 
@@ -567,22 +569,21 @@ char *ptr;
 }
 
 Obj *
-cons(x, y)
-Obj *x, *y;
+cons(Obj *x, Obj *y)
 {
     Obj *new = newobj();
 
     new->type = CONS;  
     new->v.cons.car = x;  new->v.cons.cdr = y;
+    /* Xconq's Lisp does not include dot notation for consing, so this
+       can only happen if there is an internal error somewhere. */
     if (!listp(y))
-      run_warning("cdr of cons is not a list");
+      run_error("cdr of cons is not a list");
     return new;
 }
 
 void
-type_warning(funname, x, typename, subst)
-char *funname, *typename;
-Obj *x, *subst;
+type_warning(char *funname, Obj *x, char *typename, Obj *subst)
 {
     char buf1[BUFSIZE], buf2[BUFSIZE];
 
@@ -592,28 +593,37 @@ Obj *x, *subst;
                 funname, typename, buf1, buf2);
 }
 
+/* This routine reports fatal errors with handling objects. */
+
+static void
+internal_type_error(char *funname, Obj *x, char *typename)
+{
+    char buf1[BUFSIZE];
+
+    sprintlisp(buf1, x, BUFSIZE);
+    run_error("%s of non-%s `%s'", funname, typename, buf1);
+}
+
 /* The usual suspects. */
 
 Obj *
-car(x)
-Obj *x;
+car(Obj *x)
 {
     if (x->type == CONS || x->type == NIL) {
 	return x->v.cons.car;
     } else {
-    	type_warning("Car", x, "list", lispnil);
+    	internal_type_error("car", x, "list");
 	return lispnil;
     }
 }
 
 Obj *
-cdr(x)
-Obj *x;
+cdr(Obj *x)
 {
     if (x->type == CONS || x->type == NIL) {
 	return x->v.cons.cdr;
     } else {
-    	type_warning("Cdr", x, "list", lispnil);
+    	internal_type_error("cdr", x, "list");
 	return lispnil;
     }
 }
@@ -653,7 +663,7 @@ Obj *x, *v;
     if (x->type == CONS) {
 	x->v.cons.cdr = v;
     } else {
-    	type_warning("set_cdr", x, "cons", lispnil);
+    	internal_type_error("set_cdr", x, "cons");
     }
 }
 
@@ -668,6 +678,7 @@ Obj *x;
     } else if (x->type == SYMBOL) {
 	return x->v.sym.symentry->name;
     } else {
+	/* (should be internal_type_error?) */
      	type_warning("c_string", x, "string/symbol", lispnil);
 	return "";
    }
@@ -682,9 +693,11 @@ Obj *x;
     if (x->type == NUMBER
 	|| x->type == UTYPE
 	|| x->type == MTYPE
-	|| x->type == TTYPE) {
+	|| x->type == TTYPE
+	|| x->type == ATYPE) {
 	return x->v.num;
     } else {
+	/* (should be internal_type_error?) */
      	type_warning("c_number", x, "number", lispnil);
 	return 0;
     }
@@ -736,15 +749,13 @@ char *str;
 }
 
 static int
-hash_name(str)
-char *str;
+hash_name(char *str)
 {
     return str[0];
 }
 
 Obj *
-symbol_value(sym)
-Obj *sym;
+symbol_value(Obj *sym)
 {
     Obj *val = sym->v.sym.value;
 
@@ -756,12 +767,11 @@ Obj *sym;
 }
 
 Obj *
-setq(sym, x)
-Obj *sym, *x;
+setq(Obj *sym, Obj *x)
 {
+    /* All the callers check for symbolness, but be careful. */
     if (!symbolp(sym)) {
-	run_warning("Can't set a non-symbol, ignoring attempt");
-	return x;
+	internal_type_error("setq", sym, "symbol");
     }
     if (constantp(sym)) {
     	run_warning("Can't alter the constant `%s', ignoring attempt",
@@ -773,112 +783,104 @@ Obj *sym, *x;
 }
 
 void
-makunbound(sym)
-Obj *sym;
+makunbound(Obj *sym)
 {
     sym->v.sym.value = lispunbound;
 }
 
 void
-flag_as_constant(sym)
-Obj *sym;
+flag_as_constant(Obj *sym)
 {
     sym->v.sym.symentry->constantp = TRUE;
 }
 
 int
-constantp(sym)
-Obj *sym;
+constantp(Obj *sym)
 {	
     return (sym->v.sym.symentry->constantp);
 }
 
 int
-numberp(x)
-Obj *x;
+numberp(Obj *x)
 {
     return (x->type == NUMBER);
 }
 
 int
-stringp(x)
-Obj *x;
+stringp(Obj *x)
 {
     return (x->type == STRING);
 }
 
 int
-symbolp(x)
-Obj *x;
+symbolp(Obj *x)
 {
     return (x->type == SYMBOL);
 }
 
 int
-consp(x)
-Obj *x;
+consp(Obj *x)
 {
     return (x->type == CONS);
 }
 
 int
-utypep(x)
-Obj *x;
+utypep(Obj *x)
 {
     return (x->type == UTYPE);
 }
 
 int
-mtypep(x)
-Obj *x;
+mtypep(Obj *x)
 {
     return (x->type == MTYPE);
 }
 
 int
-ttypep(x)
-Obj *x;
+ttypep(Obj *x)
 {
     return (x->type == TTYPE);
 }
 
 int
-pointerp(x)
-Obj *x;
+atypep(Obj *x)
+{
+    return (x->type == ATYPE);
+}
+
+int
+pointerp(Obj *x)
 {
     return (x->type == POINTER);
 }
 
 int
-boundp(sym)
-Obj *sym;
+boundp(Obj *sym)
 {
     return (sym->v.sym.value != lispunbound);
 }
 
 int
-numberishp(x)
-Obj *x;
+numberishp(Obj *x)
 {
     return (x->type == NUMBER
 	    || x->type == UTYPE
 	    || x->type == MTYPE
-	    || x->type == TTYPE);
+	    || x->type == TTYPE
+	    || x->type == ATYPE);
 }
 
 int
-listp(x)
-Obj *x;
+listp(Obj *x)
 {
     return (x->type == NIL || x->type == CONS);
 }
 
 /* General structural equality test.  Assumes that it is not getting
-   passed any circular structures. */
+   passed any circular structures, which should never happen in Xconq. */
 
 int
-equal(x, y)
-Obj *x, *y;
+equal(Obj *x, Obj *y)
 {
     /* Objects of different types can never be equal. */
     if (x->type != y->type)
@@ -891,6 +893,7 @@ Obj *x, *y;
       case UTYPE:
       case MTYPE:
       case TTYPE:
+      case ATYPE:
 	return (c_number(x) == c_number(y));
       case STRING:
 	return (strcmp(c_string(x), c_string(y)) == 0);
@@ -907,8 +910,7 @@ Obj *x, *y;
 }
 
 int
-member(x, lis)
-Obj *x, *lis;
+member(Obj *x, Obj *lis)
 {
     if (lis == lispnil) {
 	return FALSE;
@@ -925,9 +927,7 @@ Obj *x, *lis;
 /* Return the nth element of a list. */
 
 Obj *
-elt(lis, n)
-Obj *lis;
-int n;
+elt(Obj *lis, int n)
 {
     while (n-- > 0) {
 	lis = cdr(lis);
@@ -936,8 +936,7 @@ int n;
 }
 
 Obj *
-reverse(lis)
-Obj *lis;
+reverse(Obj *lis)
 {
     Obj *rslt = lispnil;
 
@@ -948,9 +947,7 @@ Obj *lis;
 }
 
 Obj *
-find_at_key(lis, key)
-Obj *lis;
-char *key;
+find_at_key(Obj *lis, char *key)
 {
     Obj *rest, *bdgs, *bdg;
 
@@ -965,9 +962,7 @@ char *key;
 }
 
 Obj *
-replace_at_key(lis, key, newval)
-Obj *lis, *newval;
-char *key;
+replace_at_key(Obj *lis, char *key, Obj *newval)
 {
     Obj *rest, *bdgs, *bdg;
 
@@ -983,9 +978,7 @@ char *key;
 }
 
 void
-fprintlisp(fp, obj)
-FILE *fp;
-Obj *obj;
+fprintlisp(FILE *fp, Obj *obj)
 {
     int needescape;
     char *str, *tmp;
@@ -1051,6 +1044,9 @@ Obj *obj;
       case TTYPE:
 	fprintf(fp, "t#%d", obj->v.num);
 	break;
+      case ATYPE:
+	fprintf(fp, "a#%d", obj->v.num);
+	break;
       case POINTER:
 	fprintlisp(fp, obj->v.ptr.sym);
 	fprintf(fp, " #|0x%lx|#", (long) obj->v.ptr.data);
@@ -1062,9 +1058,7 @@ Obj *obj;
 }
 
 void
-fprint_list(fp, obj)
-FILE *fp;
-Obj *obj;
+fprint_list(FILE *fp, Obj *obj)
 {
     Obj *tmp;
 
@@ -1076,10 +1070,7 @@ Obj *obj;
 }
 
 void
-sprintlisp(buf, obj, maxlen)
-char *buf;
-Obj *obj;
-int maxlen;
+sprintlisp(char *buf, Obj *obj, int maxlen)
 {
     if (maxlen < 10) {
 	strcpy(buf, " ... ");
@@ -1123,6 +1114,9 @@ int maxlen;
       case TTYPE:
 	sprintf(buf, "t#%d", obj->v.num);
 	break;
+      case ATYPE:
+	sprintf(buf, "a#%d", obj->v.num);
+	break;
       case POINTER:
 	sprintlisp(buf, obj->v.ptr.sym, maxlen);
 	sprintf(buf+strlen(buf), " #|0x%lx|#", (long) obj->v.ptr.data);
@@ -1134,10 +1128,7 @@ int maxlen;
 }
 
 void
-sprint_list(buf, obj, maxlen)
-char *buf;
-Obj *obj;
-int maxlen;
+sprint_list(char *buf, Obj *obj, int maxlen)
 {
     Obj *tmp;
 
@@ -1157,8 +1148,7 @@ int maxlen;
    be read in again. */
 
 char *
-escaped_symbol(str)
-char *str;
+escaped_symbol(char *str)
 {
     char *tmp = str;
 
@@ -1182,8 +1172,7 @@ char *str;
    strings of length 0. */
 
 char *
-escaped_string(str)
-char *str;
+escaped_string(char *str)
 {
     char *tmp = str, *rslt = escapedthingbuf;
 
@@ -1280,47 +1269,37 @@ Obj *elt, *lis;
 }
 
 void
-push_binding(lis, key, val)
-Obj **lis, *key, *val;
+push_binding(Obj **lis, Obj *key, Obj *val)
 {
     *lis = cons(cons(key, cons(val, lispnil)), *lis);
 }
 
 void
-push_cdr_binding(lis, key, val)
-Obj **lis, *key, *val;
+push_cdr_binding(Obj **lis, Obj *key, Obj *val)
 {
     *lis = cons(cons(key, val), *lis);
 }
 
 void
-push_int_binding(lis, key, val)
-Obj **lis, *key;
-int val;
+push_int_binding(Obj **lis, Obj *key, int val)
 {
     *lis = cons(cons(key, cons(new_number(val), lispnil)), *lis);
 }
 
 void
-push_key_binding(lis, key, val)
-Obj **lis, *val;
-int key;
+push_key_binding(Obj **lis, int key, Obj *val)
 {
     *lis = cons(cons(intern_symbol(keyword_name(key)), cons(val, lispnil)), *lis);
 }
 
 void
-push_key_cdr_binding(lis, key, val)
-Obj **lis, *val;
-int key;
+push_key_cdr_binding(Obj **lis, int key, Obj *val)
 {
     *lis = cons(cons(intern_symbol(keyword_name(key)), val), *lis);
 }
 
 void
-push_key_int_binding(lis, key, val)
-Obj **lis;
-int key, val;
+push_key_int_binding(Obj **lis, int key, int val)
 {
     *lis = cons(cons(intern_symbol(keyword_name(key)), cons(new_number(val), lispnil)),
 			    *lis);
@@ -1330,8 +1309,7 @@ int key, val;
    unless the list car is a "special form". */
 
 Obj *
-eval(x)
-Obj *x;
+eval(Obj *x)
 {
     int code;
     Obj *specialform;
@@ -1369,8 +1347,7 @@ Obj *x;
    until it is first asked for. */
 	    
 Obj *
-eval_symbol(sym)
-Obj *sym;
+eval_symbol(Obj *sym)
 {
     if (boundp(sym)) {
 	return symbol_value(sym);
@@ -1386,8 +1363,7 @@ Obj *sym;
 /* List evaluation just blasts straight through the list. */
 
 Obj *
-eval_list(lis)
-Obj *lis;
+eval_list(Obj *lis)
 {
     if (lis == lispnil) {
 	return lispnil;
@@ -1397,9 +1373,7 @@ Obj *lis;
 }
 
 int
-eval_boolean_expression(expr, fn, dflt)
-Obj *expr;
-int (*fn) PARAMS ((Obj *)), dflt;
+eval_boolean_expression(Obj *expr, int (*fn)(Obj *), int dflt)
 {
     char *opname;
 
@@ -1425,12 +1399,8 @@ int (*fn) PARAMS ((Obj *)), dflt;
 }
 
 int
-eval_number(val, isnumber)
-Obj *val;
-int *isnumber;
+eval_number(Obj *val, int *isnumber)
 {
-    Obj *rslt;
-
     /* (should have a non-complaining eval for this) */
     if (numberp(val)) {
 	*isnumber = TRUE;
@@ -1446,10 +1416,71 @@ int *isnumber;
     }
 }
 
+/* Choose from a list of weights and values, which can be formatted as
+   a flat list of (n1 v1 n2 v2 ...), or as ((n1 v1) (n2 v2) ...) */
+
+Obj *
+choose_from_weighted_list(Obj *lis, int *totalweightp, int flat)
+{
+    int n, sofar, weight;
+    char buf[BUFSIZE];
+    Obj *rest, *head, *tail, *rslt;
+
+    if (*totalweightp <= 0) {
+	for_all_list(lis, rest) {
+	    if (flat) {
+		if (numberp(car(rest))) {
+		    weight = c_number(car(rest));
+		    rest = cdr(rest);
+		} else {
+		    weight = 1;
+		}
+	    } else {
+		head = car(rest);
+		weight = ((consp(head) && numberp(car(head)))
+			  ? c_number(car(head)) : 1);
+	    }
+	    *totalweightp += weight;
+	}
+    }
+    /* Warn about dubious weights - note that we can continue to
+       execute, xrandom on 0 is still 0. */
+    if (*totalweightp == 0) {
+	sprintlisp(buf, lis, BUFSIZE);
+	run_warning("Sum of weights in weighted list `%s' is 0", buf);
+    }
+    n = xrandom(*totalweightp);
+    sofar = 0;
+    rslt = lispnil;
+    for_all_list(lis, rest) {
+	if (flat) {
+	    if (numberp(car(rest))) {
+		sofar += c_number(car(rest));
+		rest = cdr(rest);
+	    } else {
+		sofar += 1;
+	    }
+	    tail = car(rest);
+	} else {
+	    head = car(rest);
+	    if (consp(head) && numberp(car(head))) {
+		sofar += c_number(car(head));
+		tail = cdr(head);
+	    } else {
+		sofar += 1;
+		tail = head;
+	    }
+	}
+	if (sofar > n) {
+	    rslt = tail;
+	    break;
+	}
+    }
+    return rslt;
+}
+
 int
-interpolate_in_list(val, lis, rslt)
-int val, *rslt;
-Obj *lis;
+interpolate_in_list(int val, Obj *lis, int *rslt)
 {
     int first, thisin, thisval, nextin, nextval;
     Obj *rest, *head, *next;
@@ -1499,9 +1530,8 @@ Obj *lis;
 }
 
 int
-interpolate_in_list_ext(val, lis, mindo, minval, minrslt, maxdo, maxval, maxrslt, rslt)
-int val, mindo, minval, minrslt, maxdo, maxval, maxrslt, *rslt;
-Obj *lis;
+interpolate_in_list_ext(int val, Obj *lis, int mindo, int minval, int minrslt,
+			int maxdo, int maxval, int maxrslt, int *rslt)
 {
     int first, thisin, thisval, nextin, nextval;
     Obj *rest, *head, *next;
@@ -1552,10 +1582,7 @@ Obj *lis;
 }
 
 void
-interp_short_array(arr, lis, n)
-short *arr;
-Obj *lis;
-int n;
+interp_short_array(short *arr, Obj *lis, int n)
 {
     int i = 0;
     Obj *rest, *head;
@@ -1578,10 +1605,7 @@ int n;
 }
 
 void
-interp_long_array(arr, lis, n)
-long *arr;
-Obj *lis;
-int n;
+interp_long_array(long *arr, Obj *lis, int n)
 {
     int i = 0;
     Obj *rest, *head;
