@@ -1,5 +1,5 @@
 /* Name generation for Xconq.
-   Copyright (C) 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,83 +13,53 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 
-Obj *choose_from_weighted_list PARAMS ((Obj *lis, int *totalweightp));
-
 Obj *namerlist;
 
 Obj *lastnamer;
 
-static int totalsideweights;
+static int total_side_weights;
+static int total_color_weights;
 
 void
-init_namers()
+init_namers(void)
 {
     namerlist = lastnamer = lispnil;
-    totalsideweights = 0;
-}
-
-Obj *
-choose_from_weighted_list(lis, totalweightp)
-Obj *lis;
-int *totalweightp;
-{
-    int n, sofar;
-    Obj *rest, *head, *rslt;
-
-    if (*totalweightp <= 0) {
-	for_all_list(lis, rest) {
-	    head = car(rest);
-	    *totalweightp +=
-	      ((consp(head) && numberp(car(head))) ? c_number(car(head)) : 1);
-	}
-    }
-    n = xrandom(*totalweightp);
-    sofar = 0;
-    rslt = lispnil;
-    for_all_list(lis, rest) {
-	head = car(rest);
-	sofar += ((consp(head) && numberp(car(head))) ? c_number(car(head)) : 1);
-	if (sofar > n) {
-	    rslt = head;
-	    break;
-	}
-    }
-    /* Remove the weighting if present. */
-    if (consp(rslt) && numberp(car(rslt))) {
-	rslt = cdr(rslt);
-    }
-    return rslt;
+    total_side_weights = total_color_weights = 0;
 }
 
 /* Pick a side name that is not already being used. */
-/* (should reindent) */
+
 void
-make_up_side_name(side)
-Side *side;
+make_up_side_name(Side *side)
 {
-    int uniq = FALSE, tries = 0, n, sofar;
-    Obj *sidelib, *subobj, *subelts, *head, *filler;
+    int uniq, tries, n;
+    char *colstr;
+    Obj *sidelib, *subobj, *subelts, *head, *filler, *colorlib, *col;
 
     sidelib = g_side_lib();
     filler = lispnil;
     if (sidelib != lispnil) {
-      while (tries++ < 100 * numsides) {
-	subobj = choose_from_weighted_list(sidelib, &totalsideweights);
-	uniq = TRUE;
-	for_all_list(subobj, subelts) {
-	    head = car(subelts);
-	    if (stringp(cadr(head))) {
-		if (name_in_use(side, c_string(cadr(head)))) {
-		    uniq = FALSE;
-		    break;
+	tries = 0;
+	while (tries++ < 100 * numsides) {
+	    subobj = choose_from_weighted_list(sidelib, &total_side_weights,
+					       FALSE);
+	    /* Scan the properties of the chosen side library element,
+	       looking for names already in use. */
+	    uniq = TRUE;
+	    for_all_list(subobj, subelts) {
+		head = car(subelts);
+		if (stringp(cadr(head))) {
+		    if (name_in_use(side, c_string(cadr(head)))) {
+			uniq = FALSE;
+			break;
+		    }
 		}
 	    }
+	    if (uniq) {
+		filler = subobj;
+		break;
+	    }
 	}
-	if (uniq) {
-	    filler = subobj;
-	    break;
-	}
-      }
     } else {
 	/* The fallback is to label the side 'A', 'B', etc.  Note that
 	   we don't test for these "names" being already in use, but this
@@ -108,14 +78,38 @@ Side *side;
     }
     /* Now fill the side from the chosen obj - no effect if it is nil. */
     fill_in_side(side, filler, FALSE);
+    /* Add a color scheme if necessary and possible. */
+    colorlib = g_side_color_lib();
+    if (colorlib != lispnil
+	&& (empty_string(side->colorscheme)
+	    /* check for the default from side.c */
+	    || strcmp(side->colorscheme, "black,black,white") == 0)) {
+	tries = 0;
+	while (tries++ < 100 * numsides) {
+	    col = choose_from_weighted_list(colorlib, &total_color_weights,
+					    FALSE);
+	    /* See if the chosen color scheme is already in use. */
+	    /* Library entries are always one string. */
+	    /* (should complain if not, instead of ignoring) */
+	    if (consp(col))
+	      col = car(col);
+	    if (stringp(col)) {
+		colstr = c_string(col);
+		/* Note that this also rejects colors that are used
+		   as the names of sides, but that's OK. */
+		if (!name_in_use(side, colstr)) {
+		    side->colorscheme = colstr;
+		    break;
+		}
+	    }
+	}
+    }
 }
 
 /* This tests whether a given string is already being used by a side. */
 
 int
-name_in_use(side, str)
-Side *side;
-char *str;
+name_in_use(Side *side, char *str)
 {
     Side *side2;
 
@@ -127,6 +121,7 @@ char *str;
 		|| (side2->noun && strcmp(str, side2->noun) == 0)
 		|| (side2->pluralnoun && strcmp(str, side2->pluralnoun) == 0)
 		|| (side2->adjective && strcmp(str, side2->adjective) == 0)
+		|| (side2->colorscheme && strcmp(str, side2->colorscheme) == 0)
 		)
 	      return TRUE;
 	}
@@ -135,8 +130,7 @@ char *str;
 }
 
 Obj *
-make_namer(sym, meth)
-Obj *sym, *meth;
+make_namer(Obj *sym, Obj *meth)
 {
     Obj *namer = new_pointer(sym, (char *) meth), *tmpobj;
 
@@ -154,8 +148,7 @@ Obj *sym, *meth;
 /* Method to add names to units that want them and don't have them already. */
 
 int
-name_units_randomly(calls, runs)
-int calls, runs;
+name_units_randomly(int calls, int runs)
 {
     Unit *unit;
 
@@ -171,15 +164,13 @@ int calls, runs;
 /* Given a unit, return its naming method if it has one. */
 
 char *
-unit_namer(unit)
-Unit *unit;
+unit_namer(Unit *unit)
 {
-    Side *side;
+    Side *side = unit->side;
 
     if (unit == NULL)
       return NULL;
     /* Look for and return a side-specific namer if found. */
-    side = (unit->side ? unit->side : indepside);
     if (side->unitnamers != NULL && side->unitnamers[unit->type] != NULL) {
     	return side->unitnamers[unit->type];
     }
@@ -193,11 +184,10 @@ Unit *unit;
    don't give the desired results. */
 
 char *
-propose_unit_name(unit)
-Unit *unit;
+propose_unit_name(Unit *unit)
 {
     int u;
-    char *method;
+    char *method, *sidename;
 
     if (unit == NULL)
       return NULL;
@@ -212,12 +202,14 @@ Unit *unit;
 	switch (keyword_code(method)) {
 	  case K_JUNKY:
 	    /* Kind of a bizarre thing, but flavorful sometimes. */
-	    if (unit->side) {
-		sprintf(spbuf, "%c%c-%s-%02ld",
-			uppercase(unit->side->name[0]),
-			uppercase(unit->side->name[1]),
+	    sidename = side_name(unit->side);
+	    if (!empty_string(sidename) && strlen(sidename) >= 2) {
+		sprintf(spbuf, "%c%c-%s-%02d",
+			uppercase(sidename[0]), uppercase(sidename[1]),
 			utype_name_n(u, 3), unit->number);
 	    } else {
+		/* If no side name, use unit id to get better chance
+		   of uniqueness. */
 		sprintf(spbuf, "%s-%d", utype_name_n(u, 3), unit->id);
 	    }
 	    return copy_string(spbuf);
@@ -232,8 +224,7 @@ Unit *unit;
 /* This names only units that do not already have names. */
 
 void
-make_up_unit_name(unit)
-Unit *unit;
+make_up_unit_name(Unit *unit)
 {
     if (unit == NULL || unit->name != NULL)
       return;
@@ -244,11 +235,10 @@ Unit *unit;
 /* Unit numbering only happens to designated types that are on a side. */
 
 void
-assign_unit_number(unit)
-Unit *unit;
+assign_unit_number(Unit *unit)
 {
     if (u_assign_number(unit->type)
-	&& unit->side != NULL) {
+	&& unit->side != indepside) {
 	/* If unnumbered, give it the next available number and increment. */
 	if (unit->number == 0)
 	  unit->number = (unit->side->counts)[unit->type]++;
@@ -262,14 +252,11 @@ Unit *unit;
 /* Given a naming method, run it and get back a string. */
 
 char *
-run_namer(namer)
-Obj *namer;
+run_namer(Obj *namer)
 {
     int len, ix;
-    Obj *prev;
-    Obj *rslt;
+    Obj *prev, *rslt, *type, *sym;
     Obj *code = (Obj *) namer->v.ptr.data;
-    Obj *type;
 
     if (!consp(code))
       return "?format?";
@@ -289,6 +276,13 @@ Obj *namer;
 	    /* Splice out our desired name. */
 	    set_cdr(prev, cddr(prev));
 	    return c_string(rslt);
+	} else if (!empty_string(g_default_namer())
+		   && boundp(sym = intern_symbol(g_default_namer()))
+		   /* Note that this test prevents infinite recursion. */
+		   && namer != symbol_value(sym)) {
+	    /* The default namer comes in to help out when the namer is
+	       exhausted. */
+	    return run_namer(symbol_value(sym));
 	} else {
 	    return "?no more names?";
 	}
@@ -303,8 +297,7 @@ Obj *namer;
 static int maxdepth;
 
 char *
-name_from_grammar(grammar)
-Obj *grammar;
+name_from_grammar(Obj *grammar)
 {
     char rslt[500];  /* not really safe... */
     Obj *root = cadr(grammar);
@@ -321,19 +314,17 @@ Obj *grammar;
     return copy_string(rslt);
 }
 
-/* Given a nonterminal and a set of rules, find and apply the right rule. */
+/* Given a nonterminal and a set of rules, find and apply the right
+   rule. */
 
 void
-gen_name(nonterm, rules, depth, rslt)
-Obj *nonterm, *rules;
-int depth;
-char *rslt;
+gen_name(Obj *nonterm, Obj *rules, int depth, char *rslt)
 {
-    Obj *lis;
+    Obj *rest;
 
-    for (lis = rules; lis != lispnil; lis = cdr(lis)) {
-	if (equal(nonterm, car(car(lis)))) {
-	    gen_from_rule(cadr(car(lis)), rules, depth, rslt);
+    for_all_list(rules, rest) {
+	if (equal(nonterm, car(car(rest)))) {
+	    gen_from_rule(cadr(car(rest)), rules, depth, rslt);
 	    return;
 	}
     }
@@ -342,7 +333,8 @@ char *rslt;
 	&& pointerp(symbol_value(nonterm))) {
 	strcat(rslt, run_namer(symbol_value(nonterm)));
     } else {
-	/* Assume that the purported nonterm symbol is actually a terminal. */
+	/* Assume that the purported nonterm symbol is actually a
+           terminal. */
 	strcat(rslt, c_string(nonterm));
     }
 }
@@ -351,13 +343,10 @@ char *rslt;
    recurse, so there is a limit check. */
 
 void
-gen_from_rule(rule, rules, depth, rslt)
-Obj *rule, *rules;
-int depth;
-char *rslt;
+gen_from_rule(Obj *rule, Obj *rules, int depth, char *rslt)
 {
-    Obj *lis;
-    int total, num, oldlen;
+    int total, oldlen;
+    Obj *rest, *subrule;
     
     if (depth >= maxdepth)
       return;
@@ -367,46 +356,26 @@ char *rslt;
       case UTYPE:
       case MTYPE:
       case TTYPE:
+      case ATYPE:
       case POINTER:
       case EOFOBJ:
-	break;  /* ignore for now.. */
+	break;  /* ignore for now.. (but should warn?) */
       case SYMBOL:
 	gen_name(rule, rules, depth, rslt);
 	break;
       case STRING:
+	/* Strings are terminals, just append them directly. */
 	strcat(rslt, c_string(rule));
 	break;
       case CONS:
         if (symbolp(car(rule))) {
 	    switch (keyword_code(c_string(car(rule)))) {
 	      case K_OR:
-		/* weighted selection */
+		/* Make a weighted selection from the alternatives. */
 		total = 0;
-		for (lis = cdr(rule); lis != lispnil; lis = cdr(lis)) {
-		    if (numberp(car(lis))) {
-			total += c_number(car(lis));
-			lis = cdr(lis);
-		    } else {
-			total += 1;
-		    }
-		}
-		/* We now know the range, make a random index into it. */
-		num = xrandom(total);
-		/* Go through again to figure out which choice the index
-		   references. */
-		total = 0;
-		for (lis = cdr(rule); lis != lispnil; lis = cdr(lis)) {
-		    if (numberp(car(lis))) {
-			total += c_number(car(lis));
-			lis = cdr(lis);
-		    } else {
-			total += 1;
-		    }
-		    if (total > num) {
-			gen_from_rule(car(lis), rules, depth + 1, rslt);
-			return;
-		    }
-		}
+		subrule = choose_from_weighted_list(rule, &total, TRUE);
+		if (subrule != lispnil)
+		  gen_from_rule(subrule, rules, depth + 1, rslt);
 		break;
 	      case K_ANY:
 		/* uniform selection */
@@ -420,13 +389,18 @@ char *rslt;
 		  rslt[oldlen] = uppercase(rslt[oldlen]);
 		break;
 	      default:
-		/* Nested subsequence. */
-		for (lis = rule; lis != lispnil; lis = cdr(lis)) {
-		    gen_from_rule(car(lis), rules, depth + 1, rslt);
-		}
+		/* Assume anything else to be a nested subsequence. */
+		for_all_list(rule, rest)
+		  gen_from_rule(car(rest), rules, depth + 1, rslt);
 	    }
+	} else if (stringp(car(rule))) {
+	    /* Assume a nested subsequence. */
+	    for_all_list(rule, rest)
+	      gen_from_rule(car(rest), rules, depth + 1, rslt);
 	} else {
-	    /* syntax error */
+	    char buf[BUFSIZE];
+	    sprintlisp(buf, rule, BUFSIZE);
+	    run_warning("Grammar rule `%s' does not begin with a symbol", buf);
 	}
 	break;
       default:
