@@ -1,5 +1,5 @@
 /* River generation for Xconq.
-   Copyright (C) 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -11,59 +11,56 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 
-static void make_up_river_borders PARAMS ((int rivertype));
-static int border_is_compatible PARAMS ((int x, int y, int dir, int b));
-static int elev_in_dir PARAMS ((int x, int y, int dir));
-static int elev_at_meet PARAMS ((int x, int y, int dir));
-static void set_river_at PARAMS ((int x, int y, int dir, int t));
-static void make_up_river_connections PARAMS ((int rivertype));
+static void make_up_river_borders(int rivertype);
+static int border_is_compatible(int x, int y, int dir, int b);
+static int elev_in_dir(int x, int y, int dir);
+static int elev_at_meet(int x, int y, int dir);
+static void set_river_at(int x, int y, int dir, int t);
+static void make_up_river_connections(int rivertype);
+
+extern int select_by_weight(int* arr, int numvals);
 
 static int totalrivers;
 
+static int touchedwater;
+
+/* Turn this on to record (as features) information about rivers
+   that are being constructed. */
+
+static int DebugRiver = FALSE;
+
 /* Add rivers by picking a headwater randomly, then running downhill. */
 
 /* Can do as either a border or connection type, depending on which
    terrain type is called "river". */
 
 int
-make_rivers(calls, runs)
-int calls, runs;
+make_rivers(int calls, int runs)
 {
-    int x, y, t;
+    int x, y, t, keyval;
 
     totalrivers = 0;
-    for_all_interior_cells(x, y) {
-	totalrivers += t_river_chance(terrain_at(x, y));
-    }
+    for_all_interior_cells(x, y)
+      totalrivers += t_river_chance(terrain_at(x, y));
     if (totalrivers <= 0)
       return FALSE;
     totalrivers /= 10000;
     totalrivers = max(1, totalrivers);
-    if (elevations_defined()) {
-	for_all_terrain_types(t) {
-	    if (t_subtype_x(t) == 10 /*keyword_value(K_RIVER_X)*/
-		&& !aux_terrain_defined(t)) {
-		if (t_is_border(t)) {
-		    make_up_river_borders(t);
-		} else if (t_is_connection(t)) {
-		    make_up_river_connections(t);
-		}
+    keyval = c_number(symbol_value(intern_symbol(keyword_name(K_RIVER_X))));
+    for_all_terrain_types(t) {
+	if (t_subtype_x(t) == keyval && !aux_terrain_defined(t)) {
+	    if (t_is_border(t)) {
+		make_up_river_borders(t);
+	    } else if (t_is_connection(t)) {
+		make_up_river_connections(t);
 	    }
 	}
     }
     return TRUE;
 }
 
-static int touchedwater;
-
-/* Turn this on to record (as features) information about rivers
-   are being constructed. */
-
-static int DebugRiver = FALSE;
-
 static void
-make_up_river_borders(rivertype)
-int rivertype;
+make_up_river_borders(int rivertype)
 {
     int j, x, y, dir, elev, x1, y1, d1, e1, x2, y2, d2, e2;
     int x0, y0, numrivers = 0, numrivs, numtouches;
@@ -71,6 +68,9 @@ int rivertype;
     int t, watery[MAXTTYPES];
     Feature *rfeat;
 
+    /* (should do a random walk if no elevs) */
+    if (!elevations_defined())
+      return;
     announce_lengthy_process("Creating rivers (as borders)");
     allocate_area_aux_terrain(rivertype);
     if (DebugRiver)
@@ -160,7 +160,8 @@ int rivertype;
 		/* Make an actual piece of the river. */
 		if (border_is_compatible(x, y, dir, rivertype)) {
 		    set_river_at(x, y, dir, rivertype);
-		    if (DebugRiver && rfeat != NULL && raw_feature_at(x, y) == 0) {
+		    if (DebugRiver && rfeat != NULL
+			&& raw_feature_at(x, y) == 0) {
 		    	set_raw_feature_at(x, y, rfeat->id);
 		    	++(rfeat->size);
 		    }
@@ -175,7 +176,8 @@ int rivertype;
 	    if (inside_area(x, y)) {
 		numrivs = 0;
 		for_all_directions(dir) {
-		    if (border_at(x, y, dir, rivertype)) ++numrivs;
+		    if (border_at(x, y, dir, rivertype))
+		      ++numrivs;
 		}
 		if (numrivs >= NUMDIRS) {
 		    set_terrain_at(x, y, g_river_sink_terrain());
@@ -198,8 +200,7 @@ int rivertype;
 }
 
 static int
-border_is_compatible(x, y, dir, b)
-int x, y, dir, b;
+border_is_compatible(int x, int y, int dir, int b)
 {
     int x1, y1;
 
@@ -209,8 +210,7 @@ int x, y, dir, b;
 }
 
 static int
-elev_in_dir(x, y, dir)
-int x, y, dir;
+elev_in_dir(int x, int y, int dir)
 {
     int x1, y1;
 
@@ -227,8 +227,7 @@ int x, y, dir;
 /* The elevation at the junction of three cells is the lowest of the three. */
 
 static int
-elev_at_meet(x, y, dir)
-int x, y, dir;
+elev_at_meet(int x, int y, int dir)
 {
     int elev = elev_at(x, y);
 
@@ -242,8 +241,7 @@ int x, y, dir;
 /* should have various conditions */
 
 static void
-set_river_at(x, y, dir, t)
-int x, y, dir, t;
+set_river_at(int x, int y, int dir, int t)
 {
     int x1, y1;
 
@@ -259,11 +257,10 @@ int x, y, dir, t;
 }
 
 static void
-make_up_river_connections(rivertype)
-int rivertype;
+make_up_river_connections(int rivertype)
 {
-    int x0, y0, i, x, y, x1, y1, dir;
-    int numrivers = 0, lowdir, lowx, lowy, low;
+    int x0, y0, i, x, y, x1, y1, dir, stopearly;
+    int numrivers = 0, lowdir, lowx, lowy, low, weights[NUMDIRS], lastdir;
     
     allocate_area_aux_terrain(rivertype);
     announce_lengthy_process("Creating rivers (as connections)");
@@ -273,26 +270,77 @@ int rivertype;
 	    if (numrivers % 5 == 0)
 	      announce_progress((100 * numrivers) / totalrivers);
 	    x = x0;  y = y0;
-	    for (i = 0; i < 20; ++i) {
+	    /* If we're doing the random walk on a flat world, only start
+	       if the cell has no rivers already. */
+	    if (!elevations_defined() && any_connections_at(x, y, rivertype))
+	      break;
+	    lastdir = -1;
+	    stopearly = FALSE;
+	    /* (should make the river length settable) */
+	    for (i = 0; i < 40; ++i) {
 		/* Find the direction to the lowest adjacent cell. */
 		lowdir = -1;
-		low = elev_at(x, y);
-		for_all_directions(dir) {
-		    if (point_in_dir(x, y, dir, &x1, &y1)) {
-			if (elev_at(x1, y1) <= low) {
-			    lowx = x1;  lowy = y1;
-			    lowdir = dir;
-			    low = elev_at(x1, y1);
+		if (elevations_defined()) {
+		    low = elev_at(x, y);
+		    for_all_directions(dir) {
+			if (point_in_dir(x, y, dir, &x1, &y1)) {
+			    if (elev_at(x1, y1) <= low) {
+				lowx = x1;  lowy = y1;
+				lowdir = dir;
+				low = elev_at(x1, y1);
+			    }
 			}
 		    }
+		} else {
+		    /* If we're flat, then choose directions randomly,
+		       but weighting so as to make the rivers look
+		       better; discourage sharp bends and loops, run
+		       into the sea if the opportunity arises. */
+		    for_all_directions(dir) {
+			weights[dir] = 0;
+			if (point_in_dir(x, y, dir, &x1, &y1)) {
+			    /* Don't ever double back. */
+			    if (lastdir >= 0 && (dir == opposite_dir(lastdir)))
+			      continue;
+			    if (t_liquid(terrain_at(x1, y1))) {
+				weights[dir] = 1000;
+			    } else if (any_connections_at(x1, y1, rivertype)) {
+				/* Discourage rejoins, but allow some. */
+				weights[dir] = 10;
+			    } else {
+				weights[dir] = 100;
+			    }
+			    if (lastdir >= 0) {
+				if (dir == lastdir)
+				  weights[dir] *= 5;
+				else if (dir == left_dir(lastdir)
+					 || dir == right_dir(lastdir))
+				  weights[dir] *= 4;
+			    }
+			}
+			lowdir = select_by_weight(weights, NUMDIRS);
+			point_in_dir(x, y, lowdir, &lowx, &lowy);
+		    }
 		}
-		if (lowdir < 0) break;
+		if (lowdir < 0)
+		  break;
+		/* If we've joined up with any other river, we can
+                   stop now. */
+		if (any_connections_at(lowx, lowy, rivertype))
+		  stopearly = TRUE;
+		/* If we're running downhill, don't stop just because we
+		   hit liquid - might be a lake in the hills.  But on
+		   a flat world, be more careful and stop. */
+		if (!elevations_defined() && t_liquid(terrain_at(lowx, lowy)))
+		  stopearly = TRUE;
 		if (!(t_liquid(terrain_at(x, y))
 		      && t_liquid(terrain_at(lowx, lowy)))) {
 		    set_connection_at(x, y, lowdir, rivertype, TRUE);
 		}
 		x = lowx;  y = lowy;
-		if (!inside_area(x, y)) break;
+		lastdir = lowdir;
+		if (stopearly || !inside_area(x, y))
+		  break;
 	    }
 	}
     }
