@@ -1,5 +1,5 @@
 /* Unit generation for Xconq.
-   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,26 +7,24 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-Obj *choose_from_weighted_list PARAMS ((Obj *lis, int *totalweightp));
 #include "kernel.h"
 
-static void add_preexisting_units PARAMS ((Side *side, int runs));
-static int country_is_complete PARAMS ((Side *side, int runs));
-static int average_numunits PARAMS ((int u));
-static void count_cells PARAMS ((int x, int y));
-static int good_place PARAMS ((int cx, int cy));
-static void find_a_place PARAMS ((Side *side));
-static int valid_unit_place PARAMS ((int x, int y));
-static int possible_unit_place PARAMS ((int x, int y));
-static int find_unit_place PARAMS ((int u, int cx, int cy, int *xp, int *yp));
-static void mung_terrain PARAMS ((int x, int y, int u));
-static Unit *find_occupant_place PARAMS ((Side *side, int u, int x0, int y0));
-static int at_country_units_max PARAMS ((Side *side, int u));
-static void place_unit PARAMS ((int u, Side *side, int x, int y,
-				Unit *transport));
-static void expand_countries PARAMS ((void));
-static void expand_country PARAMS ((int x, int y));
-static void set_people_on_side PARAMS ((int x, int y));
+static void add_preexisting_units(Side *side, int runs);
+static int country_is_complete(Side *side, int runs);
+static int average_numunits(int u);
+static void count_cells(int x, int y);
+static int good_place(int cx, int cy);
+static void find_a_place(Side *side);
+static int valid_unit_place(int x, int y);
+static int possible_unit_place(int x, int y);
+static int find_unit_place(int u, int cx, int cy, int *xp, int *yp);
+static void mung_terrain(int x, int y, int u);
+static Unit *find_occupant_place(Side *side, int u, int x0, int y0);
+static int at_country_units_max(Side *side, int u);
+static void place_unit(int u, Side *side, int x, int y, Unit *transport);
+static void expand_countries(void);
+static void expand_country(int x, int y);
+static void set_people_on_side(int x, int y);
 
 /* Tmp array for counting terrain. */
 
@@ -61,22 +59,19 @@ static int growth;
 static int totnumtodo;
 static int totnumdone;
 
-#define announce_unit_progress(n) \
-  
-
 /* Place all the units belonging to countries. */
 
 int any_side_possible_units = -1;
 
 int
-make_countries(calls, runs)
-int calls, runs;
+make_countries(int calls, int runs)
 {
     int alldone, x0, y0, u, t, i, tot, x, y;
-    int advantage, sideadvantage, favor, x1, y1, dir;
+    int advantage, sideadvantage, favor;
     int canbeinopen[MAXUTYPES];  /* true if type need not be occ at start */
     int numtodo[MAXUTYPES], numindeptodo[MAXUTYPES];
-    int numleft[MAXUTYPES], numindepleft[MAXUTYPES], totleft, numlisted, numfails;
+    int numleft[MAXUTYPES], numindepleft[MAXUTYPES], totleft;
+    int numlisted, numfails;
     int checkmins = FALSE;
     int dopeoplesides = FALSE, dopeopleindeps = FALSE;
     int numdone = 0;
@@ -110,7 +105,7 @@ int calls, runs;
     if (checkmins) {
     	/* Edge cells are useless for country placement. */
     	for_all_interior_cells(x, y) {
-    	    ++totnumcells[terrain_at(x, y)];
+    	    ++totnumcells[(int) terrain_at(x, y)];
     	}
     	for_all_terrain_types(t) {
     	    if (t_country_min(t) * numsides > totnumcells[t]) {
@@ -183,7 +178,9 @@ int calls, runs;
 	       that belong to the side, and the independents nearby. */
 	    for_all_unit_types(u) {
 		numtodo[u] = numindeptodo[u] = 0;
-		if (type_allowed_on_side(u, side)) {
+		   /* Seems logical that side does not get units at start it can't build. */
+		if (type_allowed_on_side(u, side)
+		    && has_advance_to_build(side, u)) {
 		    if (runs > 0) {
 			/* If the side is just coming into the game,
 			   give it an average number of each type that
@@ -196,9 +193,11 @@ int calls, runs;
 		    }
 		    totnumtodo += numtodo[u];
 		}
-		if (type_allowed_on_side(u, NULL)) {
-		    numindeptodo[u] = (u_indep_near_start(u) * advantage) / sideadvantage;
-		    totnumtodo += numindeptodo[u];
+		/* Seems logical that null side does not get units at start it can't build. */
+		if (type_allowed_on_side(u, indepside)
+		    && has_advance_to_build(indepside, u)) {
+			numindeptodo[u] = (u_indep_near_start(u) * advantage) / sideadvantage;
+			totnumtodo += numindeptodo[u];
 		}
 		numleft[u] = numindepleft[u] = 0;
 	    }
@@ -220,7 +219,7 @@ int calls, runs;
 	    for_all_unit_types(u) {
 		for (i = 0; i < numindeptodo[u]; ++i) {
 		    if (find_unit_place(u, x0, y0, &x, &y)) {
-			place_unit(u, NULL, x, y, NULL);
+			place_unit(u, indepside, x, y, NULL);
 		    } else {
 			/* If can't find places for this type, give up. */
 			numindepleft[u] = numindeptodo[u] - i;
@@ -290,15 +289,17 @@ int calls, runs;
    as a country in this game. */
 
 static int
-country_is_complete(side, runs)
-Side *side;
-int runs;
+country_is_complete(Side *side, int runs)
 {
     int u, totunits, numunits[MAXUTYPES];
     Unit *unit;
     int sideadvantage = max(1, side->advantage);
     int advantage = (side->player ? side->player->advantage : sideadvantage);
 
+    /* Don't make any more independent units than preassigned. This prevents 
+    the formation of a third side in games like flattop.g and duel.g. */
+    if (side == indepside)
+      return TRUE;
     /* If side no longer in game, then we don't want to do country
        synth on it. */
     if (!side->ingame)
@@ -326,9 +327,7 @@ int runs;
 }
 
 static void
-add_preexisting_units(side, runs)
-Side *side;
-int runs;
+add_preexisting_units(Side *side, int runs)
 {
     int tries, total;
     Obj *possunits, *origpossunits, *chosen, *lis, *rest, *head;
@@ -378,7 +377,7 @@ int runs;
 		if (--tries < 0)
 		  break;
 		total = 0;
-		chosen = choose_from_weighted_list(possunits, &total);
+		chosen = choose_from_weighted_list(possunits, &total, FALSE);
 		possunits = remove_from_list(chosen, possunits);
 		if (Debug) {
 		    Dprintf("For %s, chose ", side_desig(side));
@@ -408,8 +407,7 @@ int runs;
    in the game. */
 
 static int
-average_numunits(u)
-int u;
+average_numunits(int u)
 {
     int count = 0, nsides = 0;
     Unit *unit;
@@ -430,19 +428,17 @@ int u;
 /* Count the cell as being of a particular type. */
 
 static void
-count_cells(x, y)
-int x, y;
+count_cells(int x, int y)
 {
-    ++numcells[terrain_at(x, y)];
+    ++numcells[(int) terrain_at(x, y)];
 }
 
-/* Test whether a given location is desirable for a country.  It should be
-   in the right distance range from other countries, and have enough of
-   the right sorts of terrain. */
+/* Test whether a given location is desirable for a country.  It
+   should be in the right distance range from other countries, and
+   have enough of the right sorts of terrain. */
 
 static int
-good_place(cx, cy)
-int cx, cy;
+good_place(int cx, int cy)
 {
     int toofar = TRUE, notfirst = FALSE, px, py, dist, t;
     Side *side;
@@ -483,8 +479,7 @@ int cx, cy;
    Then just pick a place and plan to patch up the terrain later. */
 
 static void
-find_a_place(side)
-Side *side;
+find_a_place(Side *side)
 {
     int tries, x, y, maxtries = area.numcells / 5;
     Unit *unit;
@@ -550,8 +545,7 @@ Side *side;
 /* The basic conditions that *must* be met by an initial unit placement. */
 
 static int
-valid_unit_place(x, y)
-int x, y;
+valid_unit_place(int x, int y)
 {
     int t = terrain_at(x, y);
 
@@ -563,8 +557,7 @@ int x, y;
 }
 
 static int
-possible_unit_place(x, y)
-int x, y;
+possible_unit_place(int x, int y)
 {
     return (inside_area(x, y) && unit_at(x, y) == NULL);
 }
@@ -573,8 +566,7 @@ int x, y;
    on terrain and adjacency.  Returns success of the search. */
 
 static int
-find_unit_place(u, cx, cy, xp, yp)
-int u, cx, cy, *xp, *yp;
+find_unit_place(int u, int cx, int cy, int *xp, int *yp)
 {
     int tries, x, y;
     int maxtries = tmpradius * tmpradius * 5;
@@ -619,8 +611,7 @@ int u, cx, cy, *xp, *yp;
 }
 
 static void
-mung_terrain(x, y, u)
-int x, y, u;
+mung_terrain(int x, int y, int u)
 {
     int dir, x1, y1;
 
@@ -637,9 +628,7 @@ int x, y, u;
 }
 
 static Unit *
-find_occupant_place(side, u, x0, y0)
-Side *side;
-int u, x0, y0;
+find_occupant_place(Side *side, int u, int x0, int y0)
 {
     Unit *transport;
 
@@ -656,13 +645,16 @@ int u, x0, y0;
 }
 
 static void
-place_unit(u, side, x, y, transport)
-int u, x, y;
-Side *side;
-Unit *transport;
+place_unit(int u, Side *side, int x, int y, Unit *transport)
 {
     Unit *unit;
 
+    /* Check for this BEFORE creating the unit, or it will end up as an independent ghost! */
+    /* has_advance_to_build no longer included by type_allowed_on_side. */
+    /* Reasonable to limit start units to those we can build ourselves. */
+    if (!new_unit_allowed_on_side(u, side)
+       || !has_advance_to_build(side, u))
+	    return;
     unit = create_unit(u, TRUE);
     if (unit != NULL) {
 	if (side != NULL) {
@@ -679,7 +671,7 @@ Unit *transport;
 }
 
 static void
-expand_countries()
+expand_countries(void)
 {
     int i, maxradius, growth, x, y, dir, x1, y1;
     int pop, pop2, majoritypop, broken;
@@ -747,8 +739,7 @@ expand_countries()
 /* Expand a country into the given cell. */
 
 static void
-expand_country(x, y)
-int x, y;
+expand_country(int x, int y)
 {
     int u, t = terrain_at(x, y), ours, theirs, prob, prob2;
     int advantage, sideadvantage;
@@ -841,9 +832,7 @@ int x, y;
 }
 
 static int
-at_country_units_max(side, u)
-Side *side;
-int u;
+at_country_units_max(Side *side, int u)
 {
     int themax = u_country_units_max(u);
 
@@ -856,13 +845,15 @@ int u;
 /* A method that scatters independent units over the world. */
 
 int
-make_independent_units(calls, runs)
-int calls, runs;
+make_independent_units(int calls, int runs)
 {
     int u, t, x, y;
     int doindeptype[MAXUTYPES], doanytype = FALSE, doindeppeople = FALSE;
     Unit *unit;
 
+    /* Only ever run this once. */
+    if (runs > 0)
+      return FALSE;
     /* Can't do anything without some terrain. */
     if (!terrain_defined())
       return FALSE;
@@ -929,8 +920,7 @@ int calls, runs;
    Also don't always overwrite existing people. */
 
 static void
-set_people_on_side(x, y)
-int x, y;
+set_people_on_side(int x, int y)
 {
     if ((distance(tmpside->startx, tmpside->starty, x, y) <= tmpradius
          || flip_coin())
