@@ -1,5 +1,5 @@
 /* Game modules for Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1991-1996, 1998-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,7 +10,8 @@ any later version.  See the file COPYING.  */
 
 extern char *readerrbuf;
 
-static void do_one_variant PARAMS ((Module *module, struct a_variant *var, Obj *varsetdata));
+static void do_one_variant(Module *module, struct a_variant *var,
+			   Obj *varsetdata);
 
 /* List of all known modules. Their descriptions can co-exist in memory,
    even if their contents cannot. */
@@ -26,7 +27,7 @@ char *moduledesigbuf = NULL;
 /* Empty out the list of modules. */
 
 void
-clear_game_modules()
+clear_game_modules(void)
 {
     modulelist = mainmodule = NULL;
 }
@@ -34,21 +35,16 @@ clear_game_modules()
 /* Create a brand-new game module. */
 
 Module *
-create_game_module(name)
-char *name;
+create_game_module(char *name)
 {
     Module *module = (Module *) xmalloc(sizeof(Module));
 
     /* Fill in nonzero slots. */
+    copy_module(module, NULL);
     /* The module's name must never be NULL. */
     if (name == NULL)
       name = "";
     module->name = name;
-    module->instructions = lispnil;
-    module->notes = lispnil;
-    module->designnotes = lispnil;
-    module->startlineno = 1;
-    module->endlineno = 1;
     /* Add to front of module list. */
     module->next = modulelist;
     modulelist = module;
@@ -56,8 +52,7 @@ char *name;
 }
 
 Module *
-find_game_module(name)
-char *name;
+find_game_module(char *name)
 {
     Module *module;
 
@@ -74,8 +69,7 @@ char *name;
 /* Produce a module of the given name, either by finding it or creating it. */
 
 Module *
-get_game_module(name)
-char *name;
+get_game_module(char *name)
 {
     Module *module = find_game_module(name);
 
@@ -88,9 +82,7 @@ char *name;
    list of another module. */
 
 Module *
-add_game_module(name, includer)
-char *name;
-Module *includer;
+add_game_module(char *name, Module *includer)
 {
     Module *module = get_game_module(name), *other;
 
@@ -119,7 +111,7 @@ Module *includer;
    it the main module if none defined. */
 
 void
-load_default_game()
+load_default_game(void)
 {
     extern char *standard_game_name;
     char *defaultname = standard_game_name;
@@ -148,8 +140,7 @@ load_default_game()
    otherwise. */
 
 int
-load_game_description(module)
-Module *module;
+load_game_description(Module *module)
 {
     Obj *form, *thecar;
     char *name;
@@ -177,9 +168,7 @@ Module *module;
    function tries to find a file, open it, and load the contents. */
 
 void
-load_game_module(module, dowarn)
-Module *module;
-int dowarn;
+load_game_module(Module *module, int dowarn)
 {
     char ch;
 
@@ -200,30 +189,25 @@ int dowarn;
 	}
 	/* We're done, can close. */
 	close_module(module);
-	/* Mark the module as having been loaded - note that this will happen
-	   even if there were horrible errors. */
+	/* Mark the module as having been loaded - note that this will
+	   happen even if there were horrible errors. */
 	module->loaded = TRUE;
-	/* If the turn number has been set explicitly to a positive value,
-	   assume that a saved game is being restored into the middle of the turn. */
+	/* If the turn number has been set explicitly to a positive
+	   value, assume that a saved game is being restored into the
+	   middle of the turn. */
 	if (g_turn() > 0)
 	  midturnrestore = TRUE;
-	/* If the random state has been set explicitly to a nonnegative value,
-	   use it to reseed the generator. */
+	/* If the random state has been set explicitly to a
+	   nonnegative value, use it to reseed the generator. */
 	if (g_random_state() >= 0)
 	  init_xrandom(g_random_state());
 	/* Make all the cross-references right. */
 	patch_object_references();
-	/* Are all the types staked down now? */
-	if (!canaddutype && !canaddmtype && !canaddttype) {
-	    typesdefined = TRUE;
-	    /* (should also record this as a sort of base module?) */
-	}
     }
 }
 
 void
-load_base_module(module)
-Module *module;
+load_base_module(Module *module)
 {
     char *basename = module->basemodulename;
     Module *basemodule;
@@ -245,9 +229,7 @@ Module *module;
 /* Given a module, attempt to open it. */
 
 int
-open_module(module, dowarn)
-Module *module;
-int dowarn;
+open_module(Module *module, int dowarn)
 {
     FILE *fp = NULL;
 
@@ -303,8 +285,7 @@ int dowarn;
 /* (fix so that correct name can be found reliably) */
 
 int
-load_side_config(side)
-Side *side;
+load_side_config(Side *side)
 {
 #if 0
     FILE *fp;
@@ -314,7 +295,9 @@ Side *side;
     /* (should incorp config name somehow, also be sys-dependent) */
     module = create_game_module(side->player->name);
 
-    if ((module->fp = fopen(module->filename, "r")) != NULL) {
+    /* Don't use fopen directly, use open_module which indirectly calls 
+    open_file to ensure mac/unix cross-platform compatibility. */    
+    if (open_module(module, FALSE)) {
 	if ((config = read_form(module->fp,
 				&(module->startlineno),
 				&(module->endlineno)))
@@ -341,8 +324,7 @@ Side *side;
    loaded?) */
 
 void
-read_forms(module)
-Module *module;
+read_forms(Module *module)
 {
     Obj *form;
 
@@ -371,9 +353,7 @@ Module *module;
 /* Interpret the given list of variants. */
 
 void
-do_module_variants(module, lis)
-Module *module;
-Obj *lis;
+do_module_variants(Module *module, Obj *lis)
 {
     int i, found, didsome;
     Obj *restset, *varset;
@@ -414,10 +394,7 @@ Obj *lis;
 }
 
 static void
-do_one_variant(module, var, varsetdata)
-Module *module;
-Variant *var;
-Obj *varsetdata;
+do_one_variant(Module *module, Variant *var, Obj *varsetdata)
 {
     int val, caseval;
     int width = 0, height = 0, circumference, latitude, longitude;
@@ -457,6 +434,30 @@ Obj *varsetdata;
 	var->intvalue = val;
 	set_g_use_side_priority(val);
 	break;
+      case K_PEOPLE:
+	val = (varsetdata == lispnil ?
+	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
+	       c_number(eval(car(varsetdata))));
+	var->hasintvalue = TRUE;
+	var->intvalue = val;
+	set_g_people(val);
+	break;
+      case K_ECONOMY:
+	val = (varsetdata == lispnil ?
+	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
+	       c_number(eval(car(varsetdata))));
+	var->hasintvalue = TRUE;
+	var->intvalue = val;
+	set_g_economy(val);
+	break;
+      case K_SUPPLY:
+	val = (varsetdata == lispnil ?
+	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
+	       c_number(eval(car(varsetdata))));
+	var->hasintvalue = TRUE;
+	var->intvalue = val;
+	set_g_supply(val);
+	break;
       case K_WORLD_SIZE:
       	filler = lispnil;
 	if (varsetdata != lispnil) {
@@ -573,9 +574,60 @@ Obj *varsetdata;
     var->used = TRUE;
 }
 
+/* Given a module and a original, copy over properties of the original
+   module. */
+
 void
-init_module_reshape(module)
-Module *module;
+copy_module(Module *module, Module *origmodule)
+{
+    char *name;
+    Module *next, *incl, *nextincl, *lastincl;
+
+    /* No matter what, preserve the existing name. */
+    name = module->name;
+    if (origmodule != NULL) {
+	/* Save away the module's links to other modules. */
+	next = module->next;
+	incl = module->include;
+	nextincl = module->nextinclude;
+	lastincl = module->lastinclude;
+	memcpy(module, origmodule, sizeof(Module));
+	/* By default, we copy everything through; but keep a separate
+	   copy of the original name and variants, because the calling
+	   routine may overwrite with other values. */
+	/* (should test each property individually here?) */
+	if (empty_string(origmodule->origmodulename)) {
+	    /* If no original module, use the main module. */
+	    module->origmodulename = origmodule->name;
+	    module->origvariants = origmodule->variants;
+	    module->origversion = origmodule->version;
+	}
+	/* Kill off caches that should not be replicated. */
+	module->contents = NULL;
+	module->sp = NULL;
+	module->fp = NULL;
+	module->open = FALSE;
+	module->loaded = FALSE;
+	/* Restore links. */
+	module->next = next;
+	module->include = incl;
+	module->nextinclude = nextincl;
+	module->lastinclude = lastincl;
+    } else {
+	/* If there is no original from which to copy, just do some
+	   generic clearing/init. */
+	memset(module, 0, sizeof(Module));
+	module->instructions = lispnil;
+	module->notes = lispnil;
+	module->designnotes = lispnil;
+	module->startlineno = 1;
+	module->endlineno = 1;
+    }
+    module->name = name;
+}
+
+void
+init_module_reshape(Module *module)
 {
     /* Seed all the reshaping parameters with reasonable values. */
     module->maybe_reshape = TRUE;
@@ -587,14 +639,12 @@ Module *module;
     module->final_subarea_x = module->final_subarea_y = 0;
     module->final_width = area.width;  module->final_height = area.height;
     module->final_circumference = world.circumference;
-    module->fill_type = 0;
 }
 
 /* This is true if any actual reshaping is required. */
 
 int
-reshape_the_output(module)
-Module *module;
+reshape_the_output(Module *module)
 {
     return (module->maybe_reshape
 	    && (module->subarea_width != area.width
@@ -613,8 +663,7 @@ Module *module;
 /* Check if the proposed reshape will actually work. */
 
 int
-valid_reshape(module)
-Module *module;
+valid_reshape(Module *module)
 {
     /* (should check hexagon shaping) */
     if (module->subarea_width > area.width
@@ -629,8 +678,7 @@ Module *module;
 /* Close the module. */
 
 void
-close_module(module)
-Module *module;
+close_module(Module *module)
 {
     if (module->sp) {
 	module->sp = NULL;
@@ -645,8 +693,7 @@ Module *module;
 /* Return a description of the module. */
 
 char *
-module_desig(module)
-Module *module;
+module_desig(Module *module)
 {
     if (moduledesigbuf == NULL)
       moduledesigbuf = xmalloc(BUFSIZE);
@@ -661,8 +708,7 @@ Module *module;
 /* Use this to ensure that everything is cool. */
 
 void
-doublecheck_state(side)
-Side *side;
+doublecheck_state(Side *side)
 {
     Unit *unit;
 
@@ -676,18 +722,18 @@ Side *side;
 }
 #endif /* DEBUGGING */
 
-void set_u_internal_name(u, s) int u; char *s; { utypes[u].iname = s; }
-void set_u_type_name(u, s) int u; char *s; { utypes[u].name = s; }
-void set_m_type_name(m, s) int m; char *s; { mtypes[m].name = s; }
-void set_t_type_name(t, s) int t; char *s; { ttypes[t].name = s; }
+void set_u_internal_name(int u, char *s) { utypes[u].iname = s; }
+void set_u_type_name(int u, char *s) { utypes[u].name = s; }
+void set_m_type_name(int m, char *s) { mtypes[m].name = s; }
+void set_t_type_name(int t, char *s) { ttypes[t].name = s; }
+void set_a_type_name(int a, char *s) { atypes[a].name = s; }
 
 /* If a special symbol, we might not have to fail. */
 
 int
-lazy_bind(sym)
-Obj *sym;
+lazy_bind(Obj *sym)
 {
-    int u, m, t;
+    int u, m, t, a;
     Obj *value;
 
     switch (keyword_code(c_string(sym))) {
@@ -695,21 +741,23 @@ Obj *sym;
 	value = lispnil;
 	/* Since consing glues onto the front, iterate backwards
 	   through the types. */
-	for (u = numutypes - 1; u >= 0; --u) {
-	    value = cons(new_utype(u), value);
-	}
+	for (u = numutypes - 1; u >= 0; --u)
+	  value = cons(new_utype(u), value);
 	break;
       case K_MSTAR:
 	value = lispnil;
-	for (m = nummtypes - 1; m >= 0; --m) {
-	    value = cons(new_mtype(m), value);
-	}
+	for (m = nummtypes - 1; m >= 0; --m)
+	  value = cons(new_mtype(m), value);
 	break;
       case K_TSTAR:
 	value = lispnil;
-	for (t = numttypes - 1; t >= 0; --t) {
-	    value = cons(new_ttype(t), value);
-	}
+	for (t = numttypes - 1; t >= 0; --t)
+	  value = cons(new_ttype(t), value);
+	break;
+      case K_ASTAR:
+	value = lispnil;
+	for (a = numatypes - 1; a >= 0; --a)
+	  value = cons(new_atype(a), value);
 	break;
       default:
 	return FALSE;
@@ -719,8 +767,7 @@ Obj *sym;
 }
 
 int
-coerce_to_side_id(x)
-Obj *x;
+coerce_to_side_id(Obj *x)
 {
     if (numberp(x)) {
 	return c_number(x);
@@ -729,15 +776,13 @@ Obj *x;
 }
 
 Side *
-coerce_to_side(x)
-Obj *x;
+coerce_to_side(Obj *x)
 {
     return side_n(coerce_to_side_id(x));
 }
 
 int
-coerce_to_unit_id(x)
-Obj *x;
+coerce_to_unit_id(Obj *x)
 {
     if (numberp(x)) {
 	return c_number(x);
@@ -746,8 +791,7 @@ Obj *x;
 }
 
 Unit *
-coerce_to_unit(x)
-Obj *x;
+coerce_to_unit(Obj *x)
 {
     return find_unit(coerce_to_unit_id(x));
 }
