@@ -1,5 +1,5 @@
 /* Definitions for game modules in Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1991-1996, 1999, 2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -12,6 +12,7 @@ any later version.  See the file COPYING.  */
 typedef struct a_variant {
     Obj *id;			/* unique id */
     char *name;			/* displayable name */
+    char *help;			/* help string */
     Obj *dflt;			/* default value */
     Obj *range;			/* description of range of values */
     Obj *cases;			/* actions to do on matches */
@@ -37,19 +38,17 @@ typedef struct a_module {
     char *version;		/* the version of this module */
     char *programversion;	/* compatible program versions */
     Variant *variants;		/* array of player-choosable variants */
-    char *origmodulename;	/* module that this was originally (before save) */
+    char *origmodulename;	/* module this was originally (before save) */
     Variant *origvariants;	/* variants chosen for the original module */
     char *origversion;		/* the version of the original module */
     char *contents;		/* a string with the actual contents */
     char *sp;			/* "string pointer" a la file pointer */
     char *filename;		/* the filename */
     FILE *fp;			/* the stdio file buffer */
-    char *hook;			/* space for system-specific file info */
     int startlineno;		/* line number being read at start of form */
     int endlineno;		/* line number being read at end of form */
     /* Primarily for designer use, to control writing of a module */
     short def_all;		/* true if all data is to be written out */
-    short read_only;		/* true if shouldn't be modified */
     short def_types;		/* flags indicating what to put in a module */
     short def_tables;
     short compress_tables;
@@ -61,7 +60,8 @@ typedef struct a_module {
     short def_area_misc;
     short def_area_weather;
     short def_area_material;
-    short compress_layers;	/* true if layer data should be compressed when written */
+    short compress_layers;	/* true if layer data should be compressed
+				   when written */
     short def_sides;
     short def_side_views;
     short def_side_doctrines;
@@ -73,7 +73,7 @@ typedef struct a_module {
     short def_unit_acts;
     short def_unit_plans;
     short def_history;
-    short maybe_reshape;
+    short maybe_reshape;	/* true if going to save diff shape of area */
     int subarea_width;
     int subarea_height;
     int subarea_x;
@@ -85,7 +85,6 @@ typedef struct a_module {
     int final_width;
     int final_height;
     int final_circumference;
-    short fill_type;
     short open;			/* true if currently open */
     short loaded;		/* true if already loaded */
     struct a_module *next;	/* pointer to next module */
@@ -107,22 +106,23 @@ extern Module *mainmodule;
 
 /* Declarations of module functions. */
 
-extern void clear_game_modules PARAMS ((void));
-extern Module *create_game_module PARAMS ((char *name));
-extern Module *find_game_module PARAMS ((char *name));
-extern Module *get_game_module PARAMS ((char *name));
-extern Module *add_game_module PARAMS ((char *name, Module *includer));
-extern void load_default_game PARAMS ((void));
-extern int load_game_description PARAMS ((Module *module));
-extern void load_game_module PARAMS ((Module *module, int dowarn));
-extern void load_base_module PARAMS ((Module *module));
-extern int open_module PARAMS ((Module *module, int dowarn));
-extern void read_forms PARAMS ((Module *module));
-extern void init_module_reshape PARAMS ((Module *module));
-extern int reshape_the_output PARAMS ((Module *module));
-extern int valid_reshape PARAMS ((Module *module));
-extern void close_module PARAMS ((Module *module));
-extern char *module_desig PARAMS ((Module *module));
+extern void clear_game_modules(void);
+extern Module *create_game_module(char *name);
+extern Module *find_game_module(char *name);
+extern Module *get_game_module(char *name);
+extern Module *add_game_module(char *name, Module *includer);
+extern void load_default_game(void);
+extern int load_game_description(Module *module);
+extern void load_game_module(Module *module, int dowarn);
+extern void load_base_module(Module *module);
+extern int open_module(Module *module, int dowarn);
+extern void read_forms(Module *module);
+extern void copy_module(Module *module, Module *origmodule);
+extern void init_module_reshape(Module *module);
+extern int reshape_the_output(Module *module);
+extern int valid_reshape(Module *module);
+extern void close_module(Module *module);
+extern char *module_desig(Module *module);
 
 
 
