@@ -1,5 +1,5 @@
 /* Sides in Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -11,24 +11,25 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kernel.h"
+#include "ai.h"
 
 #define checked_elev_at(x, y) (elevations_defined() ? elev_at(x, y) : 0)
 
-static void init_visible_elevation PARAMS ((int x, int y));
-static void init_visible_elevation_2 PARAMS ((int x, int y));
-static void calc_visible_elevation PARAMS ((int x, int y));
-static void calc_visible_elevation_2 PARAMS ((int x, int y));
-static void cover_area_1 PARAMS ((Side *side, struct a_unit *unit,
-				  int x0, int y0, int x1, int y1));
-static int see_materials PARAMS ((Side *side, int x, int y));
-static int see_weather PARAMS ((Side *side, int x, int y));
-static int mistaken_type PARAMS ((int u2));
+static void init_visible_elevation(int x, int y);
+static void init_visible_elevation_2(int x, int y);
+static void calc_visible_elevation(int x, int y);
+static void calc_visible_elevation_2(int x, int y);
+static void cover_area_1(Side *side, struct a_unit *unit, int x0, int y0,
+			 int x1, int y1);
+static int see_materials(Side *side, int x, int y);
+static int see_weather(Side *side, int x, int y);
+static int mistaken_type(int u2);
 
 /* Head of the list of all sides. */
 
 Side *sidelist;
 
-/* Pointer to the side representing independence. */
+/* Pointer to the side representing independent units. */
 
 Side *indepside;
 
@@ -45,14 +46,19 @@ Side *tmpside;
 
 int numsides;
 
+/* This is the number of sides including indepside. We need it for the
+   AI assignment in init.c. We want to keep numsides referring to
+   normal sides only, since it is used a lot in the interface code,
+   where we typically don't want to include indepside. */
+
+int numtotsides;
+
 /* Used to generate the id number of the side. */
 
 int nextsideid;
 
 /* Cached values of global vision vars. */
 
-int all_see_all;
-
 short any_los = -1;
 
 int any_material_views = -1;
@@ -89,63 +95,61 @@ Doctrine *last_doctrine;
 
 int next_doctrine_id;
 
-/* Init side machinery.   We always have an "independent" side hanging around;
-   it will be at the head of the list of sides, but normal side iteration
-   finesses this by starting from the second list element.  The independent
-   side is also a good placeholder for random stuff. */
+/* Init side machinery. Don't fill in indepside yet since numutypes
+   etc. may not be defined.  */
 
 void
-init_sides()
+init_sides(void)
 {
     /* Set up the list of sides. */
     sidelist = lastside = (Side *) xmalloc(sizeof(Side));
-    /* The independent units' side will be the first one, is always created. */
+    /* The independent units' side will be the first one in the list,
+       is always created.  It cannot be filled in yet, because the
+       numbers of the various types is not yet known. */
     indepside = sidelist;
-    /* Fill in some properties of the independents' side. */
-    indepside->name = "- indep -";
-    indepside->noun = "- indep -";
-    indepside->pluralnoun = "- indeps -";
-    indepside->adjective = "independent";
-    indepside->colorscheme = NULL;
-    /* By default, independent units are not identified by an emblem. */
-    /* A game design can set this appropriately if desired, however. */
-    indepside->emblemname = "none";
-    indepside->id = 0;
-    /* Put valid Lisp data into slots that need it. */
-    indepside->symbol = lispnil;
-    indepside->instructions = lispnil;
-    indepside->rawscores = lispnil;
-    indepside->aidata = lispnil;
-    indepside->uidata = lispnil;
-    indepside->possible_units = lispnil;
-    /* The independent side is not counted in numsides. */
+    /* Note: indepside is included in numtotsides, but not numsides. */
+    numtotsides = 0;
     numsides = 0;
-    /* Regular side ids start at 1 and go up. */
-    nextsideid = 1;
+    nextsideid = 0;
     /* Set up the list of players. */
     playerlist = last_player = NULL;
     numplayers = 0;
-    nextplayerid = 1;
+    /* Indepside also needs a player, so we now start with id 0. */
+    nextplayerid = 0;
     /* Set up the player/side assignment array. */
     assignments = (Assign *) xmalloc(MAXSIDES * sizeof(Assign));
     /* Set up the list of doctrines. */
     doctrine_list = last_doctrine = NULL;
-    next_doctrine_id = 1;
+    /* Indepside also needs a doctrine, so we now start with id 0. */
+    next_doctrine_id = 0;
 }
 
 /* Create an object representing a side. */
 
 Side *
-create_side()
+create_side(void)
 {
-    int u;
+    int u, m;
     Side *newside;
 
-    if (numsides >= g_sides_max()) {
+    if (numsides >=  g_sides_max()) {
 	run_error("Cannot have more than %d sides total!", g_sides_max());
     }
-    /* Allocate the side object proper. */
-    newside = (Side *) xmalloc(sizeof(Side));
+    /* Just fill in the names if dealing with indepside. */
+    if (nextsideid == 0) {
+    	newside = indepside;
+	newside->name = "Independents";
+	newside->noun = "Independent";
+	newside->pluralnoun = "Independents";
+	newside->adjective = "independent";
+	newside->colorscheme = "black,black,white";
+	newside->emblemname = "none";
+    } else {
+	/* Else allocate a new side object and increase numsides. */
+	newside = (Side *) xmalloc(sizeof(Side));
+	/* numsides does not include indepside, thus increment it here. */
+    	++numsides;
+    }
     /* Fill in various side slots.  Only those with non-zero/non-NULL
        defaults need have anything done to them. */
     newside->id = nextsideid++;
@@ -184,21 +188,62 @@ create_side()
     newside->losscounts = (short *) xmalloc(numutypes * num_loss_reasons * sizeof(short));
     newside->atkstats = (long **) xmalloc(numutypes * sizeof(long *));
     newside->hitstats = (long **) xmalloc(numutypes * sizeof(long *));
+    /* Necessary to enable AI control toggling of selected units! */
+    newside->prefixarg = -1; 
+    if (numatypes > 0) {
+   	 newside->advance = (short *) xmalloc(numatypes * sizeof(short));
+    	 newside->numadvances = 0;
+	 newside->research_topic = NOADVANCE;
+	 newside->autoresearch = FALSE;
+    }
+    if (nummtypes > 0) {
+	/* Set up the side's supply of materials. */
+	newside->treasury = (short *) xmalloc(nummtypes * sizeof(short));
+	for_all_material_types(m)
+	     if (side_has_treasury(newside, m))
+	  newside->treasury[m] = m_initial_treasury(m);
+	/* Make space for default conversions and copy them over. */
+	newside->c_rates = (short *) xmalloc(nummtypes * sizeof(short));
+	{
+	    int m1, m2;
+	    
+	    /* First initilaize m. */
+	    m = NONMTYPE;
+	    /* For now, assume only one m. */
+	    for_all_material_types(m1) {
+		for_all_material_types(m2) {
+		    if (mm_conversion(m1, m2) > 0) {
+			m = m1;
+			break;
+		    }
+		}
+	    }
+	    if (m != NONMTYPE) {
+		for_all_material_types(m2) {
+		    newside->c_rates[m2] = mm_conversion_default(m, m2);
+		}
+	    }
+	}
+    }
     /* Link in at the end of the list of sides. */
     newside->next = NULL;
-    lastside->next = newside;
-    lastside = newside;
+    /* Important to avoid the indepside->next = indepside loop! */
+    if (newside != indepside) {
+	lastside->next = newside;
+	lastside = newside;
+    }
     init_side_unithead(newside);
-    ++numsides;
+    /* numtotsides also includes indepside. */
+    ++numtotsides;
     return newside;
 }
 
-/* To make the double links work, we have to have one pseudo-unit to serve
-   as a head.  This unit should not be seen by any outside code. */
+/* To make the double links work, we have to have one pseudo-unit to
+   serve as a head.  This unit should not be seen by any outside
+   code. */
 
 void
-init_side_unithead(side)
-Side *side;
+init_side_unithead(Side *side)
 {
     if (side->unithead == NULL) {
 	side->unithead = create_bare_unit(0);
@@ -212,8 +257,7 @@ Side *side;
 /* This should be improved to not be fooled by dead units? */
 
 int
-side_has_units(side)
-Side *side;
+side_has_units(Side *side)
 {
     return (side->unithead != NULL
 	    && (side->unithead->next != side->unithead));
@@ -222,8 +266,7 @@ Side *side;
 /* Set up doctrine structures. */
 
 void
-init_doctrine(side)
-Side *side;
+init_doctrine(Side *side)
 {
     int u;
 
@@ -231,8 +274,10 @@ Side *side;
 	side->default_doctrine = new_doctrine(0);
     }
     if (side->udoctrine == NULL) {
-	/* Make each individual unit doctrine point to the generic doctrine by default. */
-	side->udoctrine = (Doctrine **) xmalloc(numutypes * sizeof(Doctrine *));
+	/* Make each individual unit doctrine point to the generic
+           doctrine by default. */
+	side->udoctrine =
+	  (Doctrine **) xmalloc(numutypes * sizeof(Doctrine *));
     }
     for_all_unit_types(u) {
 	if (side->udoctrine[u] == NULL)
@@ -241,8 +286,7 @@ Side *side;
 }
 
 void
-init_self_unit(side)
-Side *side;
+init_self_unit(Side *side)
 {
     Unit *unit;
 
@@ -261,15 +305,15 @@ Side *side;
    side is created (during/after reading but before synthesis). */
 
 int
-init_view(side)
-Side *side;
+init_view(Side *side)
 {
     int terrainset = FALSE, t, u, m;
 
     /* Be sure that we're not trying to set this up too soon. */
     check_area_shape();
     /* Allocate terrain view layers. */
-    if (side->terrview == NULL) {
+    if ((!g_terrain_seen() || !g_see_terrain_always())
+	&& side->terrview == NULL) {
 	side->terrview = malloc_area_layer(char);
 	if (!g_see_terrain_always()) {
 	    side->terrviewdate = malloc_area_layer(short);
@@ -393,8 +437,7 @@ Side *side;
 /* Calculate the centroid of all the starting units. */
 
 void
-calc_start_xy(side)
-Side *side;
+calc_start_xy(Side *side)
 {
     int num = 0, sumx = 0, sumy = 0;
     Unit *unit;
@@ -413,23 +456,19 @@ Side *side;
 /* Given a side, get its "number", which is same as its "id". */
 
 int
-side_number(side)
-Side *side;
+side_number(Side *side)
 {
-    return (side == NULL ? indepside->id : side->id);
+    return side->id;
 }
 
 /* The inverse function - given a number, figure out which side it is.
-   Returns NULL for independent side number and for any failures. */
+   Returns NULL for any failure. */
 
 Side *
-side_n(n)
-int n;
+side_n(int n)
 {
     Side *side;
 
-    if (n < 1)
-      return NULL;
     for_all_sides(side)
       if (side->id == n)
         return side;
@@ -437,8 +476,7 @@ int n;
 }
 
 Side *
-find_side_by_name(str)
-char *str;
+find_side_by_name(char *str)
 {
     Side *side;
 
@@ -458,8 +496,7 @@ char *str;
 }
 
 Side *
-parse_side_spec(str)
-char *str;
+parse_side_spec(char *str)
 {
     int s;
     char *reststr;
@@ -478,21 +515,17 @@ char *str;
 /* This is true when one side controls another. */
 
 int
-side_controls_side(side, side2)
-Side *side, *side2;
+side_controls_side(Side *side, Side *side2)
 {
-    if (side == NULL || side2 == NULL)
-      return FALSE;
     return (side == side2 || side2->controlled_by == side);
 }
 
 short *max_control_ranges;
 
-static int controller_here PARAMS ((int x, int y));
+static int controller_here(int x, int y);
 
 static int
-controller_here(x, y)
-int x, y;
+controller_here(int x, int y)
 {
     Unit *unit2;
 
@@ -509,17 +542,18 @@ int x, y;
 /* This is true if the given side may operate on the given unit. */
 
 int
-side_controls_unit(side, unit)
-Side *side;
-Unit *unit;
+side_controls_unit(Side *side, Unit *unit)
 {
     int dir, x1, y1;
     Unit *unit2;
 
-    if (side == NULL || unit == NULL)
+    if (unit == NULL)
       return FALSE;
     if (is_designer(side))
       return TRUE;
+    /* Also give debuggers full control. */
+    if (Debug || DebugG || DebugM)
+      return TRUE;
     if (side_controls_side(side, unit->side)) {
 	/* The *unit* side must have the tech to use the unit, the controlling
 	   side would have to actually take over the unit if it wants to use
@@ -577,36 +611,52 @@ Unit *unit;
 /* This is true if the given side may examine the given unit. */
 
 int
-side_sees_unit(side, unit)
-Side *side;
-Unit *unit;
+side_sees_unit(Side *side, Unit *unit)
 {
-    if (side == NULL || unit == NULL)
+    if (unit == NULL)
       return FALSE;
-    if (is_designer(side))
+    if (side->see_all)
       return TRUE;
     if (side_controls_side(side, unit->side))
       return TRUE;
+    /* We may also examine units that trust us, */
+    if (trusted_side(unit->side, side))
+      return TRUE;
+    /* or if the game is over. */
+    if (endofgame)
+      return TRUE;      
     return FALSE;
 }
 
+/* This is true if the side can see the image of a unit. */
+/* (will be more accurate when glimpse objects impled, this
+   reports that a concealed unit is visible if it might at
+   some point actually be spotted) */
+
 int
-side_sees_image(side, unit)
-Side *side;
-Unit *unit;
+side_sees_image(Side *side, Unit *unit)
 {
-    if (side == NULL || unit == NULL)
+    int uview;
+
+    if (side == NULL)
+      run_error("NULL side to side_sees_image");
+    if (unit == NULL)
       return FALSE;
-    if (is_designer(side))
-      return TRUE;
-    if (all_see_all)
+    if (side->see_all)
       return TRUE;
     if (side_controls_side(side, unit->side))
       return TRUE;
+    if (u_see_always(unit->type))
+      return TRUE;
     if (in_area(unit->x, unit->y)
 	&& side->coverage != NULL
 	&& cover(side, unit->x, unit->y) > 0)
       return TRUE;
+    /* (should be true only if see chance >= 100) */
+    uview = unit_view(side, unit->x, unit->y);
+    if (unit->type == vtype(uview)
+	&& side_number(unit->side) == vside(uview))
+      return TRUE;
     return FALSE;
 }
 
@@ -730,9 +780,7 @@ Unit *nextunit;
 }
 
 Unit *
-find_next_mover(side, prevunit)
-Side *side;
-Unit *prevunit;
+find_next_mover(Side *side, Unit *prevunit)
 {
     Unit *unit = NULL;
 
@@ -743,7 +791,7 @@ Unit *prevunit;
 	    if (is_unit(unit) && unit->id > 0
 		&& alive(unit)
 		&& inside_area(unit->x, unit->y)
-		&& (unit->act && unit->act->acp > 0)) {
+		&& has_acp_left(unit)) {
 		return unit;
 	    }
 	}
@@ -752,9 +800,7 @@ Unit *prevunit;
 }
 
 Unit *
-find_prev_mover(side, nextunit)
-Side *side;
-Unit *nextunit;
+find_prev_mover(Side *side, Unit *nextunit)
 {
     Unit *unit = NULL;
 
@@ -765,7 +811,7 @@ Unit *nextunit;
 	    if (is_unit(unit) && unit->id > 0
 		&& alive(unit)
 		&& inside_area(unit->x, unit->y)
-		&& (unit->act && unit->act->acp > 0)) {
+		&& has_acp_left(unit)) {
 		return unit;
 	    }
 	}
@@ -774,9 +820,7 @@ Unit *nextunit;
 }
 
 Unit *
-find_next_awake_mover(side, prevunit)
-Side *side;
-Unit *prevunit;
+find_next_awake_mover(Side *side, Unit *prevunit)
 {
     Unit *unit = NULL;
 
@@ -788,7 +832,7 @@ Unit *prevunit;
 		&& unit->id > 0
 		&& alive(unit)
 		&& inside_area(unit->x, unit->y)
-		&& (unit->act && unit->act->acp > 0)
+		&& has_acp_left(unit)
 		&& (unit->plan
 		    && !unit->plan->asleep
 		    && !unit->plan->reserve
@@ -801,9 +845,7 @@ Unit *prevunit;
 }
 
 Unit *
-find_prev_awake_mover(side, nextunit)
-Side *side;
-Unit *nextunit;
+find_prev_awake_mover(Side *side, Unit *nextunit)
 {
     Unit *unit = NULL;
 
@@ -813,11 +855,11 @@ Unit *nextunit;
 	    if (is_unit(unit) && unit->id > 0
 		&& alive(unit)
 		&& inside_area(unit->x, unit->y)
-				&& (unit->act && unit->act->acp > 0)
-				&& (unit->plan
-				    && !unit->plan->asleep
-				    && !unit->plan->reserve
-				    && !unit->plan->delayed)) {
+		&& has_acp_left(unit)
+		&& (unit->plan
+		    && !unit->plan->asleep
+		    && !unit->plan->reserve
+		    && !unit->plan->delayed)) {
 		return unit;
 	    }
 	}
@@ -895,8 +937,7 @@ Side *side;
 	if (unit != NULL
 	    && alive(unit)
 	    && inside_area(unit->x, unit->y)
-	    && (unit->act
-		&& unit->act->acp > 0)
+	    && has_acp_left(unit)
 	    && (unit->plan
 		&& !unit->plan->asleep
 		&& !unit->plan->reserve
@@ -1053,8 +1094,7 @@ char *newname;
 #ifdef DESIGNERS
 
 void
-become_designer(side)
-Side *side;
+become_designer(Side *side)
 {
     Side *side2;
 
@@ -1065,9 +1105,8 @@ Side *side;
     /* Designers have godlike power in the game, so mark it (permanently)
        as no longer a normal game. */
     compromised = TRUE;
-    side->may_set_see_all = TRUE;
-    /* See everything by default - can turn off manually if desired. */
-    side->see_all = TRUE;
+    side->may_set_show_all = TRUE;
+    side->show_all = TRUE;
     update_everything();
     for_all_sides(side2) {
 	update_side_display(side2, side, TRUE);
@@ -1075,8 +1114,7 @@ Side *side;
 }
 
 void
-become_nondesigner(side)
-Side *side;
+become_nondesigner(Side *side)
 {
     Side *side2;
 
@@ -1084,8 +1122,8 @@ Side *side;
       return;
     side->designer = FALSE;
     --numdesigners;
-    side->may_set_see_all = FALSE;
-    side->see_all = all_see_all;
+    side->may_set_show_all = FALSE;
+    side->show_all = side->see_all;
     update_everything();
     for_all_sides(side2) {
 	update_side_display(side2, side, TRUE);
@@ -1113,7 +1151,7 @@ int val;
     int oldval;
     Side *side3;
 
-    if (side == NULL || side2 == NULL || side == side2)
+    if (side == side2)
       return;
     if (side->trusts == NULL)
       return;
@@ -1144,7 +1182,7 @@ int val;
     int oldval, oldval2;
     Side *side3;
 
-    if (side == NULL || side2 == NULL || side == side2)
+    if (side == side2)
       return;
     if (side->trusts == NULL || side2->trusts == NULL)
       return;
@@ -1210,6 +1248,16 @@ int value;
     side->autofinish = value;
 }
 
+/* What interfaces should use to tweak the autoresearch flag. */
+
+void
+set_autoresearch(side, value)
+Side *side;
+int value;
+{
+    side->autoresearch = value;
+}
+
 /* Being at war requires only ones of the sides to consider itself so. */
 
 /* (Should the other side's relationships be tweaked also?) */
@@ -1377,7 +1425,7 @@ int *types;
 
     if (sender == NULL)
       return;
-    if (all_see_all)
+    if (g_see_all())
       return;
     if (!g_terrain_seen()) {
 	for_all_cells(x, y) {
@@ -1385,31 +1433,28 @@ int *types;
 	        && terrain_view(recipient, x, y) == UNSEEN) {
 	        set_terrain_view(recipient, x, y, terrain_view(sender, x, y));
 	        /* (should update unit views also) */
-		update_cell_display(recipient, x, y, TRUE);
 	    }
 	}
     }
     for_all_side_units(sender, unit) {
 	if (in_play(unit) && (types == NULL || types[unit->type])) {
 	    see_exact(recipient, unit->x, unit->y);
-	    update_cell_display(recipient, unit->x, unit->y, TRUE);
 	}
     }
+    /* Whether the amount of data is small or large, there is little value
+       in being selective about updates - just redo entire maps. */
+    update_area_display(recipient);
 }
 
 /* Modify doctrine according to a specification. */
 
 void
-set_doctrine(side, spec)
-Side *side;
-char *spec;
+set_doctrine(Side *side, char *spec)
 {
     int u;
     char *arg, *arg2, *rest, substr[BUFSIZE];
     Doctrine *doctrine;
 
-    if (side == NULL)
-      return;
     rest = get_next_arg(spec, substr, &arg);
     if ((doctrine = find_doctrine_by_name(arg)) != NULL) {
 	/* Found a specific named doctrine. */
@@ -1441,6 +1486,12 @@ char *spec;
     }
 }
 
+void
+set_side_research(Side *side, int a)
+{
+    side->research_topic = a;
+}
+
 /* Vision. */
 
 /* What happens when a unit appears on a given cell. */
@@ -1457,17 +1508,14 @@ char *spec;
    when it first moves! */
 
 void
-all_see_occupy(unit, x, y, inopen)
-Unit *unit;
-int x, y;
-int inopen;
+all_see_occupy(Unit *unit, int x, int y, int inopen)
 {
     Side *side;
     int always = u_see_always(unit->type);
     
     for_all_sides(side) {
 	if (side->see_all) {
-	    update_cell_display(side, x, y, TRUE);
+	    update_cell_display(side, x, y, UPDATE_ALWAYS);
 	} else if (side_sees_unit(side, unit)) {
 	    see_cell(side, x, y);
 	} else if (side_tracking_unit(side, unit)) {
@@ -1482,7 +1530,10 @@ int inopen;
 	    }
 	}
     }
-    {
+    /* If the occupation happens during the game, nearby units that
+       are on unfriendly sides should wake up.  (should also be:
+       "unless they were directed to sleep in presence of enemy") */
+    if (gameinited) {
 	int dir, x1, y1;
 	Unit *unit2, *unit3;
 	    	
@@ -1515,10 +1566,7 @@ int inopen;
    is decremented. */
 
 void
-all_see_leave(unit, x, y, inopen)
-Unit *unit;
-int x, y;
-int inopen;
+all_see_leave(Unit *unit, int x, int y, int inopen)
 {
     Side *side;
     int always = u_see_always(unit->type);
@@ -1526,7 +1574,7 @@ int inopen;
 
     for_all_sides(side) {
 	if (side->see_all) {
-	    update_cell_display(side, x, y, TRUE);
+	    update_cell_display(side, x, y, UPDATE_ALWAYS);
 	} else if (side_sees_unit(side, unit)) {
 	    see_cell(side, x, y);
 	} else if (side_tracking_unit(side, unit)) {
@@ -1558,8 +1606,7 @@ static int tmpx0, tmpy0, tmpz0;
 static int tmpnx, tmpny, tmpnz;
 
 static void
-init_visible_elevation(x, y)
-int x, y;
+init_visible_elevation(int x, int y)
 {
     int elev = checked_elev_at(x, y);
 
@@ -1567,8 +1614,7 @@ int x, y;
 }
 
 static void
-init_visible_elevation_2(x, y)
-int x, y;
+init_visible_elevation_2(int x, int y)
 {
     int elev = checked_elev_at(x, y);
 
@@ -1577,8 +1623,7 @@ int x, y;
 
 
 static void
-calc_visible_elevation(x, y)
-int x, y;
+calc_visible_elevation(int x, int y)
 {
     int dir, x1, y1, elev, tmp, tmpa, tmpb;
     int adjelev = 9999, viselev, dist, dist1, cellwid = area.cellwidth;
@@ -1610,8 +1655,7 @@ int x, y;
 }
 
 static void
-calc_visible_elevation_2(x, y)
-int x, y;
+calc_visible_elevation_2(int x, int y)
 {
     int dir, x1, y1, elev, tmp, tmpa, tmpb;
     int adjelev = 9999, viselev, dist, dist1, cellwid = area.cellwidth;
@@ -1649,32 +1693,36 @@ int x, y;
 /* (LOS comes in here, to make irregular coverage areas) */
 
 void
-cover_area(side, unit, oldtransport, x0, y0, nx, ny)
-Side *side;
-Unit *unit, *oldtransport;
-int x0, y0, nx, ny;
+cover_area(Side *side, Unit *unit, Unit *oldtransport, int x0, int y0,
+	   int nx, int ny)
 {
     Side *side2;
 
     if (side != NULL
-	&& !all_see_all
-	&& !indep(unit)
+	&& !g_see_all()
 	&& completed(unit)
         && (oldtransport == NULL
 	   || uu_occ_can_see(unit->type, oldtransport->type))) {
-	for_all_sides(side2) {
-	    if (trusted_side(side, side2)) {
-		cover_area_1(side2, unit, x0, y0, nx, ny);
+	if (side->ingame) {
+	    /* Active sides keep their allies informed.  Note that
+	       this will effectively leak information to allies of
+	       allies, even if the unit's side is not an ally of the
+	       side "side". */
+	    for_all_sides(side2) {
+		if (trusted_side(side, side2)) {
+		    cover_area_1(side2, unit, x0, y0, nx, ny);
+		}
 	    }
+	} else {
+	    /* Inactive sides just have their own data maintained
+	       (not clear if this is critical, but seems sensible) */
+	    cover_area_1(side, unit, x0, y0, nx, ny);
 	}
     }
 }
 
 static void
-cover_area_1(side, unit, x0, y0, nx, ny)
-Side *side;
-Unit *unit;
-int x0, y0, nx, ny;
+cover_area_1(Side *side, Unit *unit, int x0, int y0, int nx, int ny)
 {
     int u = unit->type, range0, nrange, range, x, y, x1, y1, x2, y2;
     int y1c, y2c, xw, cov, los, r;
@@ -1871,7 +1919,7 @@ int x0, y0, nx, ny;
 		    if ((newcov > 0 && oldcov == 0)
 			|| (newcov == 0 && oldcov > 0)
 			|| (DebugG && newcov != oldcov))
-		      update_cell_display(side, xw, y, 36);
+		      update_cell_display(side, xw, y, UPDATE_COVER);
 		    anychanges = TRUE;
 		}
 	    }
@@ -1885,11 +1933,11 @@ int x0, y0, nx, ny;
 /* Use this to clear out garbled view coverage. */
 
 void
-reset_coverage()
+reset_coverage(void)
 {
     Side *side;
 
-    if (all_see_all)
+    if (g_see_all())
       return;
     for_all_sides(side)
       calc_coverage(side);
@@ -1898,8 +1946,7 @@ reset_coverage()
 /* Calculate/recalculate the view coverage layers of a side. */
 
 void
-calc_coverage(side)
-Side *side;
+calc_coverage(Side *side)
 {
     int x, y, s2, pop, visible[MAXSIDES];
     Unit *unit;
@@ -1934,25 +1981,27 @@ Side *side;
     }
 }
 
+/* Look for and clear out any bogus view data. */
+
 void
-reset_all_views()
+reset_all_views(void)
 {
     Side *side;
 
-    if (all_see_all)
-      return;
     for_all_sides(side) {
-	reset_view(side);
+	if (!side->see_all) {
+	    reset_view(side);
+	}
     }
 }
 
 void
-reset_view(side)
-Side *side;
+reset_view(Side *side)
 {
     int x, y, uview;
 
     for_all_cells(x, y) {
+	/* Remove any leftover images of our own units. */
 	if (cover(side, x, y) == 0
 	    && ((uview = unit_view(side, x, y)) != EMPTY)
 	    && vside(uview) == side_number(side)) {
@@ -1962,16 +2011,13 @@ Side *side;
 }
 
 void
-react_to_seen_unit(side, unit, x, y)
-Side *side;
-Unit *unit;
-int x, y;
+react_to_seen_unit(Side *side, Unit *unit, int x, int y)
 {
     int uview, eu;
     Unit *eunit;
     Side *es;
 
-    if (all_see_all /* see real unit */) {
+    if (g_see_all() /* see real unit */) {
     	/* (should look at all of stack if can be mixed) */
 	if ((eunit = unit_at(x, y)) != NULL) {
 	    if (react_to_enemies(unit) && !allied_side(eunit->side, side)) {
@@ -2073,7 +2119,7 @@ int nx, ny;
     }
 }
 
-extern void compute_see_chances PARAMS ((void));
+extern void compute_see_chances(void);
 
 int any_see_chances = -1;
 
@@ -2137,14 +2183,13 @@ compute_see_chances()
     }
 }
 
-int test_for_viewer PARAMS ((int x, int y));
-int test_for_mistaken_viewer PARAMS ((int x, int y));
+int test_for_viewer(int x, int y);
+int test_for_mistaken_viewer(int x, int y);
 
 static Unit *tmpunittosee, *tmpunitseen, *tmpseer;
 
 int
-test_for_viewer(x, y)
-int x, y;
+test_for_viewer(int x, int y)
 {
     int u2 = tmpunittosee->type, x2 = tmpunittosee->x, y2 = tmpunittosee->y;
     int dist, chance;
@@ -2172,11 +2217,10 @@ int x, y;
 }
 
 int
-test_for_mistaken_viewer(x, y)
-int x, y;
+test_for_mistaken_viewer(int x, int y)
 {
-    int u2 = tmpunittosee->type, x2 = tmpunittosee->x, y2 = tmpunittosee->y;
-    int dist, chance;
+    int x2 = tmpunittosee->x, y2 = tmpunittosee->y;
+    int dist;
     Unit *unit;
 
     dist = distance(x, y, x2, y2);
@@ -2195,8 +2239,7 @@ int x, y;
    to many displays. */
 
 void
-all_see_cell(x, y)
-int x, y;
+all_see_cell(int x, int y)
 {
     register Side *side;
 
@@ -2205,25 +2248,23 @@ int x, y;
     }
 }
 
-/* Look at the given position, possibly not seeing anything.  Return true if
-   a unit was spotted. */
+/* Look at the given position, possibly not seeing anything.  Return
+   true if a unit was spotted. */
 
 int
-see_cell(side, x, y)
-Side *side;
-int x, y;
+see_cell(Side *side, int x, int y)
 {
-    int update, chance, t, curview, curuview, newuview, x1, y1;
-    int m, mupdate, wupdate, seentype, mistakechance;
+    int update, updatet, chance, t, curview, curuview, newuview, x1, y1;
+    int m, mupdate, wupdate, seentype, mistakechance, flags;
     Unit *unit, *unitseen, *seer;
 
-    if (side == NULL || side == indepside || !in_area(x, y))
+    if (!in_area(x, y))
       return FALSE;
-    update = FALSE;
+    update = updatet = FALSE;
     unitseen = NULL;
     /* If we see everything, just pass through to updating the display. */
     if (side->see_all) {
-    	update = TRUE;
+    	update = updatet = TRUE;
     	unitseen = unit_at(x, y);
     } else if (cover(side, x, y) > 0) {
     	/* Always update our knowledge of the terrain. */
@@ -2232,7 +2273,7 @@ int x, y;
 	    set_terrain_view(side, x, y, buildtview(terrain_at(x, y)));
 	    if (!g_see_terrain_always())
 	      set_terrain_view_date(side, x, y, g_turn());
-	    update = TRUE;
+	    update = updatet = TRUE;
 	}
 	if (any_material_views) {
 	    mupdate = see_materials(side, x, y);
@@ -2249,7 +2290,8 @@ int x, y;
    	curuview = unit_view(side, x, y);
    	seer = NULL;
    	for_all_stack(x, y, unit) {
-	    if (side_sees_unit(side, unit)) {
+	    if (side_sees_unit(side, unit)
+		|| u_see_always(unit->type)) {
 		unitseen = unit;
 		break;
 	    } else if (any_see_chances) {
@@ -2336,19 +2378,28 @@ int x, y;
 	    set_unit_view(side, x, y, EMPTY);
 	    update = TRUE;
 	}
+#if 0 /* not used, and adds noise to savefiles */
 	/* In any case, our info is now up-to-date. */
 	set_unit_view_date(side, x, y, g_turn());
+#endif
     }
     /* If there was any change in what was visible, tell the display. */
     if (update) {
-	update_cell_display(side, x, y, FALSE);
+	flags = UPDATE_ALWAYS;
+	/* If the view of the terrain changed, adjacent cells probably
+	   need to be redrawn as well. */
+	if (updatet)
+	  flags |= UPDATE_ADJ;
+	update_cell_display(side, x, y, flags);
     }
     return (unitseen != NULL);
 }
 
+/* Given a unit type, return a similar-looking type that it might be
+   confused with. */
+
 static int
-mistaken_type(u2)
-int u2;
+mistaken_type(int u2)
 {
     int u3, totalweight, randval;
 
@@ -2373,23 +2424,22 @@ int u2;
    This is the lowest level of all viewing routines, and executed a lot. */
 
 void
-see_exact(side, x, y)
-Side *side;
-int x, y;
+see_exact(Side *side, int x, int y)
 {
-    register int olduview, oldtview, newtview, newuview, update, mupdate, wupdate;
-    register Unit *unit;
+    int olduview, oldtview, newtview, newuview, update, mupdate, wupdate;
+    Unit *unit;
 
-    if (side == NULL || side == indepside || !in_area(x, y))
+    if (!in_area(x, y))
       return;
-    if (all_see_all) {
-	/* It may not really be necessary to do anything to the display, but
-	   the kernel doesn't know if the interface is drawing all the units
-	   that are visible, or is only drawing "interesting" ones, or whatever.
-	   It would be up to the interface to decide that, say, its magnification
-	   power for a map is such that only one unit is being displayed, and
-	   that the update from here doesn't result in any visible changes to
-	   what's already been drawn on the screen. */
+    if (side->see_all) {
+	/* It may not really be necessary to do anything to the
+	   display, but the kernel doesn't know if the interface is
+	   drawing all the units that are visible, or is only drawing
+	   "interesting" ones, or whatever.  It would be up to the
+	   interface to decide that, say, its magnification power for
+	   a map is such that only one unit is being displayed, and
+	   that the update from here doesn't result in any visible
+	   changes to what's already been drawn on the screen. */
     	update = TRUE;
     } else {
 	update = FALSE;
@@ -2411,20 +2461,20 @@ int x, y;
 	if (unit != NULL)
 	  newuview = builduview(side_number(unit->side), unit->type);
 	set_unit_view(side, x, y, newuview);
+#if 0 /* not used, and adds noise to savefiles */
 	set_unit_view_date(side, x, y, g_turn());
+#endif
 	if (olduview != newuview)
 	  update = TRUE;
     }
     /* If there was any change in what was visible, tell the display. */
     if (update) {
-	update_cell_display(side, x, y, FALSE);
+	update_cell_display(side, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
 static int
-see_materials(side, x, y)
-Side *side;
-int x, y;
+see_materials(Side *side, int x, int y)
 {
     int m, curview, update;
 
@@ -2502,7 +2552,7 @@ int x, y;
     return update;
 }
 
-/* A border has been seen if the cell on either side has been seen. */
+/* A border has been seen if the cells on both sides has been seen. */
 
 int
 seen_border(side, x, y, dir)
@@ -2513,12 +2563,12 @@ int x, y, dir;
 
     if (side->see_all)
       return TRUE;
-    if (terrain_view(side, x, y) != UNSEEN)
-      return TRUE;
+    if (terrain_view(side, x, y) == UNSEEN)
+      return FALSE;
     if (point_in_dir(x, y, dir, &x1, &y1))
-      if (terrain_view(side, x1, y1) != UNSEEN)
-	return TRUE;
-    return FALSE;
+      if (terrain_view(side, x1, y1) == UNSEEN)
+	return FALSE;
+    return TRUE;
 }
 
 /* Make a printable identification of the given side.  This should be
@@ -2704,7 +2754,7 @@ new_standing_order()
 /* Add a new standing order for the side.  This function can add to front
    or back of existing list of orders. */
 
-int order_conds_match PARAMS ((StandingOrder *sorder, StandingOrder *sorder2));
+int order_conds_match(StandingOrder *sorder, StandingOrder *sorder2);
 
 void
 add_standing_order(side, sorder, pos)
@@ -2921,7 +2971,7 @@ char *buf;
 	tprintf(buf, "%s %d", taskdefns[i].name, args[0]);
 	break;
     default:
-	task_desc(buf+strlen(buf), NULL, sorder->task);
+	task_desc(buf+strlen(buf), NULL, NULL, sorder->task);
 	break;
     }
     return buf;
@@ -2960,19 +3010,21 @@ char *str, *buf, **rsltp;
 
 /* Agreement handling here for now. */
 
-Agreement *agreementlist = NULL;
+Agreement *agreement_list = NULL;
 
-Agreement *lastagreement;
+Agreement *last_agreement;
 
-int numagreements = 0;
+int num_agreements = 0;
 
-char *agreementdesigbuf;
+int next_agreement_id = 1;
+
+char *agreement_desig_buf;
 
 void
 init_agreements()
 {
-    agreementlist = lastagreement = NULL;
-    numagreements = 0;
+    agreement_list = last_agreement = NULL;
+    num_agreements = 0;
 }
 
 Agreement *
@@ -2981,29 +3033,44 @@ int id;
 {
     Agreement *ag = (Agreement *) xmalloc(sizeof(Agreement));
 
+    if (id == 0)
+      id = next_agreement_id++;
     ag->id = id;
     ag->terms = lispnil;
     ag->next = NULL;
-    if (agreementlist != NULL) {
-	lastagreement->next = ag;
+    if (agreement_list != NULL) {
+	last_agreement->next = ag;
     } else {
-	agreementlist = lastagreement = ag;
+	agreement_list = ag;
     }
-    ++numagreements;
+    last_agreement = ag;
+    ++num_agreements;
     return ag;
 }
 
+Agreement *
+find_agreement(int id)
+{
+    Agreement *ag;
+
+    for_all_agreements(ag) {
+	if (ag->id == id)
+	  return ag;
+    }
+    return NULL;
+}
+
 char *
 agreement_desig(ag)
 Agreement *ag;
 {
-    if (agreementdesigbuf == NULL)
-      agreementdesigbuf = xmalloc(BUFSIZE);
-    sprintf(agreementdesigbuf, "<ag %s %s %s>",
+    if (agreement_desig_buf == NULL)
+      agreement_desig_buf = xmalloc(BUFSIZE);
+    sprintf(agreement_desig_buf, "<ag %s %s %s>",
 	    (ag->typename ? ag->typename : "(null)"),
 	    (ag->name ? ag->name : "(null)"),
 	    (ag->terms == lispnil ? "(no terms)" : "...terms..."));
-    return agreementdesigbuf;
+    return agreement_desig_buf;
 }
 
 /* Helper functions to init view layers from rle encoding. */
@@ -3132,7 +3199,7 @@ int x, y, val;
 
 #ifdef DESIGNERS
 
-static void paint_view_1 PARAMS ((int x, int y));
+static void paint_view_1(int x, int y);
 
 static int tmptview;
 static int tmpuview;
@@ -3140,9 +3207,7 @@ static int tmpuview;
 /* Paint the side's view with given values. */
 
 void
-paint_view(side, x, y, r, tview, uview)
-Side *side;
-int x, y, r, tview, uview;
+paint_view(Side *side, int x, int y, int r, int tview, int uview)
 {
     tmpside = side;
     tmptview = tview;
@@ -3151,16 +3216,28 @@ int x, y, r, tview, uview;
 }
 
 static void
-paint_view_1(x, y)
-int x, y;
+paint_view_1(int x, int y)
 {
+    int tview, uview;
     int oldtview = terrain_view(tmpside, x, y);
     int olduview = unit_view(tmpside, x, y);
+    Unit *unit;
 
     if (oldtview != tmptview || olduview != tmpuview) {
-	set_terrain_view(tmpside, x, y, tmptview);
-	set_unit_view(tmpside, x, y, tmpuview);
-	update_cell_display(tmpside, x, y, TRUE);
+	tview = tmptview;
+	if (tview == 1234567) {
+	    tview = buildtview(terrain_at(x, y));
+	}
+	set_terrain_view(tmpside, x, y, tview);
+	uview = tmpuview;
+	if (uview == 1234567) {
+	    uview = EMPTY;
+	    unit = unit_at(x, y);
+	    if (unit != NULL)
+	      uview = builduview(side_number(unit->side), unit->type);
+	}
+	set_unit_view(tmpside, x, y, uview);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
