@@ -1,5 +1,5 @@
 /* Definitions for the Mac interface to Xconq.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,20 +18,21 @@ any later version.  See the file COPYING.  */
 #include <Sound.h>
 #endif /* THINK_C */
 
-#ifdef MPW
+#if 1 /*def MPW*/
 #ifndef __MWERKS__ /* actually, "old headers" */
 #include <Values.h>
 #endif
 #ifdef NEW_HEADERS
 #include <MacTypes.h>
+#include <MacWindows.h>
 #else
 #include <Types.h>
-#endif
+#include <Windows.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
 #include <QuickDraw.h>
 #include <Fonts.h>
 #include <Events.h>
-#include <Windows.h>
 #include <Menus.h>
 #include <TextEdit.h>
 #include <Dialogs.h>
@@ -49,7 +50,9 @@ any later version.  See the file COPYING.  */
 #include <Files.h>
 #include <Folders.h>
 #include <OSUtils.h>
+#ifndef __MWERKS__
 #include <OSEvents.h>
+#endif
 #include <DiskInit.h>
 #include <Packages.h>
 #include <Traps.h>
@@ -74,11 +77,16 @@ any later version.  See the file COPYING.  */
 #include "imf.h"
 #include "macimf.h"
 #include "ui.h"
+#include "kpublic.h"
 
 /* Bring in the definitions of resources. */
 
 #include "macdefs.h"
 
+/* Bring in floating window support. */
+
+#include "Floating.h"
+
 #ifdef MPW_C
 #define QD(whatever) (qd.##whatever)
 #define QDPat(whatever) (&(qd.##whatever))
@@ -208,7 +216,6 @@ typedef struct a_ui {
 typedef struct a_map {
 	short id;				/* Identifying number for the map */
 	VP *vp;                 		/* This map's generic view parameters */
-	short osx, osy;         		/* Tracks shifted origin so can be restored */
 	WindowPtr window;
 	ControlHandle hscrollbar;
 	ControlHandle vscrollbar;
@@ -219,11 +226,15 @@ typedef struct a_map {
 	QDErr qdErr;
 	short offsetx;			/* Displacement (right, down) in pixels of map window within its gworld */
 	short offsety;			/* offsetx = offsety = 0 means the window is in the center of its gworld */
+	short xwrap;			/* TRUE if the map itself is wrapped (wider than the world circumference). */
+	short leaky_grid;		/* TRUE if the grid is leaky and therefore needs to be drawn explicitly. */
+
 	/* How to draw the map. */
 	short conw;
 	short toplineh;
 	short topunith;
 	short toph;
+	short sbarwid;
 	short bufx;				/* Actual size of horizontal offscreen scrolling buffer */
 	short bufy;				/* Actual size of vertical offscreen scrolling buffer */
 	short max_bufx;			/* Max size of horizontal offscreen scrolling buffer as set in dialog */
@@ -241,9 +252,14 @@ typedef struct a_map {
 	short draw_topline;		/* Draw top line */
 	short draw_topunit;		/* Draw top unit info */	
 	short erase_names;		/* Draw row in update_cell_display to erase name of moving unit */	
+	short drawsizes;		/* Draw unit (city) size numbers */
+	short draw_emblems;
+	short draw_ai;
 	short see_all;
 	short autoselect;
 	Unit *curunit;
+	short blinking_curunit;
+	short selectionmasks;
 	short moveonclick;
 	short scrolltocurunit;
 	short follow_action;	/* Scroll to scene of actions? */
@@ -262,7 +278,10 @@ typedef struct a_map {
 #define MAXINLIST 500
 
 typedef struct a_list {
+	short id;
 	short sidecolors;		/* Use side colors for units */
+	short drawsizes;		/* Draw unit (city) size numbers */
+	short draw_emblems;
 	SideMask sides;			/* Sides to show the units of */
 	short completed_units;
 	short incomplete_units;
@@ -412,22 +431,19 @@ extern int nummaps;
 extern ImageFamily **uimages;
 extern ImageFamily **timages;
 extern ImageFamily **eimages;
-
 extern MacImage **tcolors;
 
 extern WindowPtr helpwin;
-extern struct a_helpnode *curhelpnode;
 extern char *helpstring;
 extern TEHandle helptext;
+extern DialogPtr researchwin;
 
+extern WindowPtr commandwin;
 extern WindowPtr instructionswin;
-
 extern WindowPtr gamewin;
-extern ControlHandle gamevscrollbar;
-
 extern WindowPtr constructionwin;
-
 extern WindowPtr designwin;
+
 extern enum tooltype tooltype;
 extern short curutype;
 extern short curttype;
@@ -436,73 +452,59 @@ extern short curfid;
 extern Feature *curfeature;
 
 extern WindowPtr historywin;
-
 extern WindowPtr noticewin;
-extern TEHandle noticetext;
-
 extern WindowPtr scoreswin;
-extern TEHandle scorestext;
-
 extern MenuHandle sidemenu;
 extern MenuHandle utypemenu;
 extern MenuHandle mtypemenu;
 extern MenuHandle ttypemenu;
 
 extern int hasColorQD;
-
 extern int minscreendepth;
 extern int maxscreendepth;
-
 extern int sbarwid;
-
+extern int floatsbarwid;
+extern WindowPtr worldmapwin;
+extern Map *worldmap;
+extern Map *frontmap;
+extern int numwindows;
+extern int mapnum;
 extern WindowPtr *winmenuwins;
 
 extern int default_draw_grid;
 extern int default_draw_names;
-extern int default_draw_latlong;
+extern int default_draw_meridians;
 
 extern int map_modal;
 
 extern int defaultmoveonclick;
 extern int defaultautoselect;
 
-extern PolyHandle polygons[];
-extern int lastpolyx[], lastpolyy[];
-extern PolyHandle gridpolygons[];
-extern int lastgridpolyx[], lastgridpolyy[];
+extern PolyHandle *polygons;
+extern int *lastpolyx, *lastpolyy;
 
-extern RgnHandle cellrgns[];
-extern int lastcellrgnx[], lastcellrgny[];
-extern RgnHandle gridcellrgns[];
-extern int lastgridcellrgnx[], lastgridcellrgny[];
+extern RgnHandle *cellrgns;
+extern int *lastcellrgnx, *lastcellrgny;
+extern RgnHandle *gridcellrgns;
+extern int *lastgridcellrgnx, *lastgridcellrgny;
 
-extern RgnHandle cellrgns30[];
-extern int lastcellrgnx30[], lastcellrgny30[];
-extern RgnHandle gridcellrgns30[];
-extern int lastgridcellrgnx30[], lastgridcellrgny30[];
+extern RgnHandle *cellrgns30;
+extern int *lastcellrgnx30, *lastcellrgny30;
+extern RgnHandle *gridcellrgns30;
+extern int *lastgridcellrgnx30, *lastgridcellrgny30;
 
-extern RgnHandle cellrgns15[];
-extern int lastcellrgnx15[], lastcellrgny15[];
-extern RgnHandle gridcellrgns15[];
-extern int lastgridcellrgnx15[], lastgridcellrgny15[];
+extern RgnHandle *cellrgns15;
+extern int *lastcellrgnx15, *lastcellrgny15;
+extern RgnHandle *gridcellrgns15;
+extern int *lastgridcellrgnx15, *lastgridcellrgny15;
 
 extern struct a_map *maplist;	   /* chain of maps that we're using */
 extern struct a_list *listlist;	 /* chain of lists */
 extern struct a_unit_closeup *unitcloseuplist;	 /* chain of unit closeups */
 
 extern CursHandle paintcursors[];
-
-extern CursHandle cellpaintor;
 extern CursHandle bordpaintor;
 extern CursHandle connpaintor;
-extern CursHandle unitpaintor;
-extern CursHandle peoplepaintor;
-extern CursHandle featurepaintor;
-extern CursHandle materialpaintor;
-extern CursHandle elevpaintor;
-extern CursHandle cloudpaintor;
-extern CursHandle temppaintor;
-
 extern CursHandle movecursors[];
 extern CursHandle nomovecursor;
 extern CursHandle allmovecursor;
@@ -510,19 +512,21 @@ extern CursHandle grayarrowcursor;
 extern CursHandle opencrosscursor;
 extern CursHandle watchcursor;
 extern CursHandle firecursor;
-
 extern CursHandle sendcursor;
 extern CursHandle receivecursor;
 
-extern RGBColor graycolor, blackcolor;
+/* Autoscroll cursors. */
+extern CursHandle leftcursor;
+extern CursHandle rightcursor;
+extern CursHandle downcursor;
+extern CursHandle upcursor;
 
 extern enum grays gridgray;
 extern enum grays unseengray;
 extern enum grays bggray;
 
-extern RGBColor gridcolor;
-extern RGBColor unseencolor;
 extern RGBColor blackcolor;
+extern RGBColor graycolor;
 
 extern int conwid;
 extern int tophgt;
@@ -542,7 +546,6 @@ extern int animation_pattern_state;
 extern WindowPtr playersetupwin;
 
 extern int connection_method;
-extern int hosting;
 extern Rect dragrect;
 extern int foundimagesfile;
 
@@ -552,7 +555,6 @@ extern TEHandle command_text;
 
 extern TEHandle run_length_text;
 
-extern int gameinited;
 extern int position_set_modally;
 extern Point modal_point;
 extern Map *modal_map;
@@ -573,7 +575,7 @@ extern short title_line_spacing;
 extern int gamewinw;
 extern int topunithgt;
 
-extern int default_latlong_interval;
+extern int default_meridian_interval;
 
 extern int use_colornames;			/* Save and read colors and colorschemes by name */
 extern int use_RGB_values;			/* Save and read RGB colors in XCol resource */
@@ -583,6 +585,8 @@ extern int unified_cicns;			/* Use BitMap as mask for color PixMap */
 
 extern int default_draw_topline;		/* Draw top line box (toggle function declared elsewhere) */
 extern int default_draw_topunit;		/* Draw unit info box (toggle function declared elsewhere) */
+extern int default_draw_scrollbars;	/* Draw scrollbars in maps. */
+extern int default_draw_cpanel;		/* Draw control panel in maps. */
 extern int default_drawothermaps;	/* Draw other-map boxes */
 extern int default_sidecolors;		/* Use side colors for unit icons */					
 extern int default_iconmasks;		/* Draw unit icon masks */
@@ -591,15 +595,24 @@ extern int default_textmasks;		/* Draw text masks */
 extern int default_featureborders;	/* Draw feature borders */
 extern int default_featurenames;		/* Draw feature names */		
 extern int default_shorelines;		/* Draw 3D shorelines */
+extern int default_draw_ai;			/* Draw AI regions (if present). */
+extern int alert_256_colors;		/* Suggest switch to 256 colors at start. */
+extern int default_draw_emblems;	/* Draw unit emblems in maps and lists. */
 extern int default_simple_borders;	/* Do not draw borders to sea or empty land */
 extern int default_optimize_fonts;	/* Use optimized set of font sizes */
 extern int default_erase_names;		/* Erase names of moving units */
 extern int default_power;			/* Default power of new map */
 extern int default_max_bufx;		/* Default size of horizonal offscreen scrolling buffer */
 extern int default_max_bufy;		/* Default size of vertical offscreen scrolling buffer */
+extern int default_drawsizes;		/* Draw unit (city) sizes */
+extern int default_blinking_curunit;	/* Draw blinking current unit instead of animated box */
+extern int default_selectionmasks;	/* Draw selection masks instead of selection boxes */
+
+#define BACKTTYPE -1					/* Differs from unseen terrain which is NONTTYPE. */
 
 #define FEATURES 10						/* Should equal the number of feature colors below! */
 
+#define backcolor 		featColor[0]		/* Window background color. */
 #define forecolor 		featColor[1]		/* Note: differs from ForeColor !!! */
 #define maskcolor 		featColor[2]		/* Default mask color for icons, text and boxes */
 #define textcolor 		featColor[3]		/* Text color (except for feature names) */
@@ -624,7 +637,7 @@ extern char **featColorName; /*[FEATURES+1][32];		/* Name of featColor value (re
 extern char **sideColorName; /*[MAXSIDES+1][4][32];		/* Name of sideColor value (red etc)*/
 extern char **default_featColorName; /*[FEATURES+1][32];	/* Default feature color values (by name) */
 extern char **default_colorscheme; /*[MAXSIDES+1][96];	/* Default side colorscheme (red,pink,blue) */
-extern char **colorscheme; /*[MAXSIDES+1][96];			/* Side colorscheme (red,pink,blue) */
+extern char **current_colorscheme; /*[MAXSIDES+1][96];			/* Side colorscheme (red,pink,blue) */
 
 #define get_sideColor(s,n) (sideColor[4 * (s) + (n)])
 
@@ -711,11 +724,14 @@ extern int at_all_visible(Map *map, int x, int y);
 extern int in_middle(Map *map, int x, int y);
 
 extern void xform(Map *map, int x, int y, int *sxp, int *syp);
+extern void xform_flat(Map *map, int x, int y, int *sxp, int *syp);
+extern void xform_fractional(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
+extern void xform_fractional_flat(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
 extern void m_xform_unit(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
 extern void m_xform_unit_self(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
 extern void m_xform_occupant(Map *map, Unit *transport, Unit *unit, int sx, int sy, int sw, int sh, int *sxp, int *syp, int *swp, int *shp);
 
-extern Map *create_map(int power);
+extern Map *create_map(int power, int id);
 extern void set_content_rect(Map *map);
 extern void m_focus_on_center(Map *map);
 extern void m_center_on_focus(Map *map);
@@ -744,35 +760,23 @@ extern void draw_related_maps(Map *map);
 extern void draw_other_map(Map *map, Map *map2);
 extern void draw_row(Map *map, int x0, int y0, int len, int clearit);
 
-// --- MOVED TO MACROW.C -----------------------------
-
-//	extern int cell_update(Map *map, int x, int y);
-//	extern void draw_terrain_row(Map *map, int x0, int y0, int len);
-
-// --- NO LONGER USED --------------------------------
-
-//	extern void draw_legend(Map *map, int x, int y);
-
-// ------------------------------------------------
+extern void draw_growicon_nolines(WindowPtr win);
 
 extern void draw_selections(Map *map);
 extern void draw_selections_at(Map *map, int x, int y);
-extern void draw_selected_unit_setport(Map *map, Unit *unit);
 extern void draw_selected_unit(Map *map, Unit *unit);
-extern void erase_selections(Map *map);
-extern void erase_selection(Map *map, Unit *unit);
-extern void draw_unselected_unit(Map *map, Unit *unit);
-
+extern void update_cell(Map *map, int x, int y);
 extern void force_map_update(Map *map);
 extern void destroy_map(Map *map);
 extern void activate_map(Map *map, int activate);
 extern void print_map(Map *map);
 
-extern void draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int s, int mod, int emblem);
+extern void draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int s, int sidecolors, int selected, int mod, int emblem);
 extern void draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style);
 extern void draw_cell_block(int sx, int sy, int n, int power, int t, int over, int angle, int dosolid);
 extern void calc_best_terrain_images(void);
-extern void draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogrid, int dosolid);
+extern void draw_hex_region(int sx, int sy, int power, int t, int over, int angle,
+							int dogrid, int dosolid, int var);
 extern void draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
 extern void draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
 extern void draw_terrain_sample(WindowPtr win, Rect tmprect, int t);
@@ -796,7 +800,7 @@ extern int picture_height(PicHandle pichandle);
 extern void plot_sicn(WindowPtr win, int sx, int sy, Handle sicnhandle, int n, int erase, int mode);
 extern void gray_out_rect(Rect *rectptr);
 extern void draw_unit_blast(Map *map, Unit *unit, int blast);
-extern void clear_unit_blast(Map *map, Unit *unit, int blast);
+extern void draw_unit_size(Unit *unit, WindowPtr win, int sx, int sy, int sw, int sh);
 
 extern void do_mouse_down_map(Map *map, Point mouse, int mods);
 extern void do_mouse_down_map_control_panel(Map *map, int h, int v, int mods);
@@ -806,6 +810,8 @@ extern void set_map_mag(Map *map, int newpower);
 extern void toggle_map_grid(Map *map);
 extern void toggle_map_topline(Map *map);
 extern void toggle_map_topunit(Map *map);
+extern void toggle_map_scrollbars(Map *map);
+extern void toggle_map_cpanel(Map *map);
 extern void toggle_map_other_maps(Map *map);
 extern void toggle_map_lighting(Map *map);
 extern void toggle_map_coverage(Map *map);
@@ -821,6 +827,10 @@ extern void toggle_map_clouds(Map *map);
 extern void toggle_map_storms(Map *map);
 extern void toggle_map_plans(Map *map);
 extern void toggle_map_ai(Map *map);
+extern void toggle_map_draw_emblems(Map *map);
+extern void toggle_blinking_curunit(Map *map);
+extern void toggle_selectionmasks(Map *map);
+
 extern void do_mouse_down_map_content(Map *map, int h, int v, int mods);
 extern void select_all_dragged_over(Map *map, int h0, int v0, int mods);
 extern void select_area_and_zoom(Map *map, int h0, int v0, int mods);
@@ -861,6 +871,7 @@ extern void grow_list(List *list, int w, int h);
 extern void zoom_list(List *list, int part);
 extern void adjust_list_decor(List *list);
 extern void do_mouse_down_list(List *list, Point mouse, int mods);
+extern int do_key_down_list(List *list, char key);
 extern void set_list_sorting(List *list, enum sortkeys newkey, int mi);
 extern void toggle_list_completed(List *list);
 extern void toggle_list_incomplete(List *list);
@@ -888,22 +899,18 @@ extern int open_game_dialog(void);
 extern int open_game_from_fsspec(FSSpec *fsspec);
 extern int open_game_from_name_and_volume(Str255 name, int vrefnum);
 extern void handle_player_setup_event(EventRecord *event);
-extern int hit_player_setup_dialog(int itemhit, Point mouse);
+extern void hit_player_setup_dialog(int itemhit, Point mouse);
+extern void hit_closeup_dialog(WindowPtr win, int itemhit, Point mouse);
+extern int do_key_down_closeup(UnitCloseup *closeup, char key);
+extern int launch_game(void);
 extern void launch_game_2(void);
 extern void check_for_missing_images(void);
 extern void open_progress_dialog(void);
 extern void close_progress_dialog(void);
 extern void init_display(void);
-extern void init_terrain_images(void);
-extern void interp_named_color(char *name, enum grays *grayvar, RGBColor *colorvar);
-extern void init_terrain_colors(void);
 extern void init_unit_images(void);
-extern void init_all_emblem_images(void);
-extern void reinit_all_emblem_images(void);
-extern void init_emblem_images(Side *side2);
-
-extern int construction_ever_possible(void);
-
+extern void init_emblem_images(void);
+extern void init_terrain_images(void);
 extern void init_menus(void);
 extern void add_window_menu_item(char *name, WindowPtr win);
 extern void remove_window_menu_item(WindowPtr win);
@@ -925,32 +932,29 @@ extern void quit_the_game(void);
 extern void apply_to_all_selected(int (*fn)(), int beepfailure);
 extern void do_reserve_command(int value, int radius, int recurse);
 extern void do_sleep_command(int value, int radius, int recurse);
-extern void do_attack_command(void);
-extern void enable_construction(void);
+extern void show_construction_window(void);
 extern int do_move_to_command(void);
-extern int do_fire_command(void);
-extern int do_fire_into_command(void);
-extern void do_profile(void);
-extern void do_trace(void);
 extern void adjust_menus(void);
 
 extern int add_unit_position(Unit *unit);
-extern void set_focus(Map *map, int x, int y);
+extern void set_map_focus(Map *map, int x, int y);
+extern void set_map_angle(Map *map, int angle);
 extern void message_dialog(int sidenum);
-extern void command_dialog(void);
 extern void do_keyboard_command(int key);
-extern void execute_named_command(char *cmdstr);
-extern void describe_commands(int arg, char *key, TextBuffer *buf);
+
 struct cmdtab;
-extern void describe_command_table(int arg, char *key, TextBuffer *buf, struct cmdtab *cmdtab);
+
 extern void toggle_profiling(void);
 
 extern char *get_string_from_item(Handle itemhandle);
 
-extern void instructions_dialog(void);
-extern void create_instructions_dialog(void);
-extern int hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt);
+extern void show_instructions_window(void);
+extern void create_instructions_window(void);
+extern void activate_instructions(int activate);
+extern void do_mouse_down_instructions(Point mouse, int mods);
+extern void draw_instructions(void);
 
+extern void show_game_window(void);
 extern void create_game_window(void);
 extern void draw_game(void);
 extern void draw_game_date(void);
@@ -988,14 +992,18 @@ extern void side_rename_dialog(Side *side);
 
 extern int unit_rename_dialog(Unit *unit);
 
+extern void show_unit_closeup(Unit *unit);
 extern UnitCloseup *find_unit_closeup(Unit *unit);
-extern void create_unit_closeup(Unit *unit);
+extern UnitCloseup *create_unit_closeup(Unit *unit);
 extern void preferred_closeup_size(int u, int *widp, int *hgtp);
 extern void draw_unit_closeup(UnitCloseup *unitcloseup);
 extern UnitCloseup *unit_closeup_from_window(WindowPtr win);
 extern int do_mouse_down_unit_closeup(UnitCloseup *unitcloseup, Point mouse, int mods);
 extern void destroy_unit_closeup(UnitCloseup *unitcloseup);
+extern MenuHandle build_construction_menu(Unit *unit);
+extern MenuHandle build_research_menu(Side *side);
 
+extern void show_history_window(void);
 extern void create_history_window(void);
 extern void draw_history(void);
 extern void draw_historical(HistEvent *hevt, int y, int drawevt);
@@ -1005,7 +1013,7 @@ extern void grow_history(int h, int v);
 extern void zoom_history(int part);
 extern void update_history_window(HistEvent *hevt);
 
-extern void notice_dialog(void);
+extern void show_notice_window(void);
 extern void create_notice_window(void);
 extern void append_notice(char *str);
 extern void draw_notice(void);
@@ -1015,7 +1023,7 @@ extern void do_mouse_down_notice(Point mouse, int mods);
 extern void grow_notice(int h, int v);
 extern void zoom_notice(int part);
 
-extern void scores_dialog(void);
+extern void show_scores_window(void);
 extern void create_scores_window(void);
 extern void append_scores(char *str);
 extern void draw_scores(void);
@@ -1025,23 +1033,26 @@ extern void do_mouse_down_scores(Point mouse, int mods);
 extern void grow_scores(int h, int v);
 extern void zoom_scores(int part);
 
-extern WindowPtr commandwin;
+extern void show_research_dialog();
+extern void draw_research_dialog(int force);
+extern int do_key_down_research(char key);
+extern void hit_research_dialog(int ditem);
+
 extern void draw_command(void);
 extern void activate_command(int activate);
 extern int do_key_down_command(int key);
 extern void do_mouse_down_command(Point mouse, int mods);
 
-extern void help_dialog(HelpNode *helpnode);
+extern void show_help_window(HelpNode *helpnode);
 extern void describe_menus(int arg, char *key, TextBuffer *buf);
 extern void describe_mouse(int arg, char *key, TextBuffer *buf);
-extern void describe_keyboard(int arg, char *key, TextBuffer *buf);
-extern void describe_text_commands(int arg, char *key, TextBuffer *buf);
 extern void describe_help(int arg, char *key, TextBuffer *buf);
 extern void create_help_window(void);
 extern void set_help_content(HelpNode *helpnode);
 extern void draw_help(void);
 extern void adjust_help_scrollbar(void);
 extern void activate_help(int activate);
+extern int do_key_down_help(char key);
 extern void do_mouse_down_help(Point mouse, int mods);
 extern void grow_help(int h, int v);
 extern void zoom_help(int part);
@@ -1051,8 +1062,8 @@ extern void enable_designing(int forsure);
 extern void disable_designing(void);
 extern void init_design_cursors(void);
 extern CursHandle adjust_designer_cursor(Point mouse, RgnHandle region);
+extern void show_design_window(void);
 extern void create_design_window(void);
-extern void position_design_window(void);
 extern void draw_design_window(void);
 extern void draw_design_window_tool(enum tooltype tool);
 extern void do_mouse_down_design(Point mouse, int mods);
@@ -1078,20 +1089,20 @@ extern void interp_mac_ui_data(Obj *uispec);
 
 extern void draw_unit_info(Map *map);
 extern void connection_method_dialog(void);
-extern void init_connection_method(void);
-
 extern void play_sound(char *soundname);
 
 extern void query_position_modally(int mode);
 extern void set_position_modally(void);
 
-extern void enable_command(void);
+extern void show_command_window(void);
 
 extern void invert_map(Map *map, int x, int y, int r);
 extern Map *front_map(void);
 
+extern void update_all_dialogs(void);
 extern void update_all_windows(void);
 extern void update_all_map_windows(void);
+extern void update_all_visible_dialogs(void);
 extern void update_all_visible_windows(void);
 extern void update_all_visible_map_windows(void);
 
@@ -1099,6 +1110,8 @@ extern void scroll_map_window(Map *map, int dx, int dy);	/* Scroll map window wi
 extern void copy_from_gworld(Map *map, Rect destRect);	/* Copy rect from gworld to screen */
 extern void update_resized_map(Map *map);			/* Update map and then redraw its gworld */
 extern void update_gworld(Map *map);					/* Update the gworld (and redraw the map)*/
+
+void set_meridians(Map *map);
 extern void set_offscreen_buffers(Map *map);			/* Set size of offscreen scrolling buffers */
 extern int handle_numeric_keypad(EventRecord *event);	/* Enables special use of numeric keypad */
 extern int cell_is_in_gworld(Map *map, int x, int y);		/* Test if cell is within current gworld */
@@ -1113,9 +1126,10 @@ extern void toggle_shorelines(Map *map);
 extern void toggle_simple_borders(Map *map);
 extern void toggle_optimize_fonts(Map *map);
 extern void toggle_erase_names(Map *map);
-extern void toggle_draw_latlong(Map *map);			/* The associated default is declared elsewhere */
-extern void toggle_solid_color_terrain(Map *map);	/* The associated default is declared elsewhere */
-extern void toggle_list_sidecolors(List *list);		/* Uses the same default as toggle_map_sidecolors */
+extern void toggle_draw_meridians(Map *map);
+extern void toggle_solid_color_terrain(Map *map);
+extern void toggle_list_sidecolors(List *list);
+extern void toggle_list_draw_emblems(List *list);
 
 extern void read_xcol(void);				/* Read RGB colors form XCol resource */
 extern void write_xcol(void);				/* Write RGB colors to XCol resource */
@@ -1130,23 +1144,7 @@ extern void set_terrain_color(Str255 tname);	/* Feature color Dialog for terrain
 extern void build_colors_menu(void);			/* Build the Colors menu */
 extern void build_sides_menu(void);			/* Build popup menu for prefs */
 extern void build_features_menu(void);		/* Build popup menu for prefs */
-extern void init_unit_cicns(void);			/* Intitialize cicns by a dummy PlotCIcon call */
-extern void init_emblem_cicns(void);			/* Intitialize cicns ny a dummy PlotCIcon call */
-extern void dispose_unit_cicns(void);			/* Trash every unit cicn in use */
-extern void dispose_emblem_cicns(void);		/* Trash every emblem cicn in use */
-
-// --- 68K COLORSCHEME BUG FIX -----------------------------------------------
 
 extern void dissect_colorscheme(int e);	 	/* Dissect a colorscheme into sideColorNames */
 
-//  extern void dissect_colorscheme(int sidenum, 	/* Dissect a colorscheme into sideColorNames */
-//		 char* colorscheme);				
-
-// -----------------------------------------------------------------------
-
-/* Functions in maccmd.c now declared here for use elsewhere */
-
-extern void do_recenter(void);
-extern void do_mac_zoom_in(void);
-extern void do_mac_zoom_out(void);
-extern void do_mac_set_map_angle(void);
+extern void add_remote_player(char *name);
