@@ -1,5 +1,5 @@
 /* Game designer handling for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,6 +16,8 @@ any later version.  See the file COPYING. */
 
 extern MenuHandle featuremenu;
 
+extern short *feature_menu_fids;
+
 WindowPtr designwin = nil;
 
 ControlHandle ttypepopup = nil;
@@ -32,10 +34,14 @@ CursHandle bordpaintor;
 CursHandle connpaintor;
 CursHandle coatpaintor;
 
+/* This is the number of tools actually used. */
+
+short numtoolsused = 0;
+
 /* This is the width and height of each design tool's entry. */
 
-int dtoolw = 120;
-int dtoolh = 36;
+short dtoolw = 120;
+short dtoolh = 36;
 
 /* The type of designer tool currently in use. */
 
@@ -43,39 +49,43 @@ enum tooltype tooltype = notool;
 
 /* All the state of the designer palette. */
 
-short curbrushradius = 0;
-short curttype = 0;
-short curbgttype = 0;
+short curbrushradius;
+short curttype;
+short curbgttype;
 short curdepth = 1;
-short curutype = 0;
-short cursidenumber = 0;
-short curmtype = 0;
-short curmamount = 0;
-short curfid = 0;
-Feature *curfeature = NULL;
-short curelevation = 0;
-short curtemperature = 0;
-short curcloudtype = 0;
-short curcloudbottom = 0;
-short curcloudheight = 0;
-short curwinddir = 0;
-short curwindforce = 0;
+short curutype;
+short cursidenumber;
+short curmtype;
+short curmamount;
+short curfid;
+Feature *curfeature;
+short curelevation;
+short curtemperature;
+short curcloudtype;
+short curcloudbottom;
+short curcloudheight;
+short curwinddir;
+short curwindforce;
 short curtview;
 short curuview;
 
+short paintcontrolmode;
+
 /* These are globals used in drag painting. */
 
 short painttype;
 short paintpeop;
+short paintctrl;
 short paintfid;
 short painttview;
 short paintuview;
 
-short enabledtooltype[20];
+short enabledtooltype[12];
 
 void
 enable_designing(int forsure)
 {
+	Map *map;
 	extern int compromised;
 
 	if (dside == NULL || dside->designer)
@@ -88,34 +98,59 @@ enable_designing(int forsure)
 				return;
 		}
 	}
-	/* Actually change designer status, this will call back to alter all displays. */
-	become_designer(dside);
+	/* Actually change designer status, this will call back to alter displays. */
+	net_become_designer(dside);
 	/* Create and add the designer's palette. */
-	if (designwin == nil) {
-		create_design_window();
+	show_design_window();
+	/* This is necessary in order to enable closeups that have no map 
+	or vp associated with them. Also needed to permit the designer to 
+	select an enemy unit. */
+	dside->see_all = TRUE;
+	/* Force all maps to see_all; designer can disable manually if
+	   desired. */
+	for_all_maps(map) {
+		if (!map->see_all) {
+			map->see_all = TRUE;
+			map->vp->show_all = TRUE;
+			force_update(map->window);
+		}
 	}
-	if (designwin != nil) {
-		position_design_window();
-		ShowWindow(designwin);
-/*		SelectWindow(designwin); */
-	}
-	/* Recache visibility flags. */
-	calc_vision(dside);
 }
 
 void
 disable_designing()
 {
+	Map *map;
+
 	if (dside == NULL || !dside->designer)
 	  return;
 	/* Hide (but don't destroy) the designer's palette. */
 	if (designwin != nil) {
-		HideWindow(designwin);
+		HideTheWindow(designwin);
 	}
 	/* Actually change designer status, this will call back to alter all displays. */
-	become_nondesigner(dside);
-	/* Recache visibility flags. */
-	calc_vision(dside);
+	net_become_nondesigner(dside);
+#if 0	
+	/* Set maps back to old see_all state if necessary. */
+	if (!dside->see_all) {
+		for_all_maps(map) {
+			map->see_all = FALSE;
+			/* Note that we have to update all maps, because a designer's
+			   no-see_all map draws unseen areas grayed out. */
+			force_update(map->window);
+		}
+	}
+#else
+	/* Set dside->see-all to whatever was true for the frontmap or to
+	true by default. */
+	if (frontmap)
+		dside->see_all = frontmap->see_all;
+	else	dside->see_all = TRUE;
+	for_all_maps(map) {
+		map->see_all = dside->see_all;
+		force_update(map->window);
+	}
+#endif
 }
 
 /* Each type of design tool has a distinct cursor. */
@@ -135,28 +170,42 @@ init_design_cursors()
 	paintcursors[temperaturetool] = GetCursor(cTemperature);
 	paintcursors[cloudstool] = GetCursor(cClouds);
 	paintcursors[windstool] = GetCursor(cWinds);
-	/* (should have a view-painting cursor) */
+	paintcursors[viewtool] = GetCursor(cView);
 }
 
 /* Adjust the cursor to reflect the current designer tool. */
 
-CursPtr
+CursHandle
 adjust_designer_cursor(Point mouse, RgnHandle region)
 {
 	if (tooltype == terraintool && !t_is_cell(curttype)) {
 		switch (t_subtype(curttype)) {
 			case bordersubtype:
-				return *bordpaintor;
+				return bordpaintor;
 			case connectionsubtype:
-				return *connpaintor;
+				return connpaintor;
 			case coatingsubtype:
-				return *coatpaintor;
+				return coatpaintor;
 			default:
 				terrain_subtype_warning("cursor adjust", curttype);
-				return &QD(arrow);
+				return nil;
 		}
 	}
-	return *(paintcursors[tooltype]);
+	return paintcursors[tooltype];
+}
+
+/* This is the top-level access to bring up the design window, can be called
+   anywhere, anytime. */
+
+void
+show_design_window()
+{
+	if (designwin == nil) {
+		create_design_window();
+	} else	 {
+		SelectTheWindow(designwin);
+		ActivateWindow(designwin, true);
+	}
 }
 
 /* Create the designer tool window. */
@@ -164,12 +213,14 @@ adjust_designer_cursor(Point mouse, RgnHandle region)
 void
 create_design_window()
 {
+	int i;
+
 	if (hasColorQD) {
-		designwin = GetNewCWindow(wDesign, NULL, (WindowPtr) -1L);
+		designwin = GetNewCWindow(wFloatFixed, NULL, NULL);
 	} else {
-		designwin = GetNewWindow(wDesign, NULL, (WindowPtr) -1L);
+		designwin = GetNewWindow(wFloatFixed, NULL, NULL);
 	}
-	add_window_menu_item("Design", designwin);  /* until this becomes a windoid */
+	SetWTitle(designwin, "\pDesign");
 	/* Make the current side be the one with the display. */
 	cursidenumber = side_number(dside);
 	build_terrain_type_menu();
@@ -185,8 +236,8 @@ create_design_window()
 	feature_add_button = GetNewControl(cFeatureAddButton, designwin);
 	feature_remove_button = GetNewControl(cFeatureRemoveButton, designwin);
 	feature_edit_button = GetNewControl(cFeatureEditButton, designwin);
-	SizeWindow(designwin, 2 * dtoolw - 1, (numtooltypes / 2) * dtoolh - 1, 1);
-	position_design_window();
+	/* Just put the window at the left edge of the screen. */
+	MoveWindow(designwin, 3, 280, true);
 	init_design_cursors();
 	enabledtooltype[notool] = TRUE;
 	enabledtooltype[terraintool] = TRUE;
@@ -199,34 +250,18 @@ create_design_window()
 	enabledtooltype[temperaturetool] = any_temp_variation;
 	enabledtooltype[cloudstool] = any_clouds;
 	enabledtooltype[windstool] = any_wind_variation;
-	enabledtooltype[viewtool] = !all_see_all;
-	ShowWindow(designwin);
-}
+	enabledtooltype[viewtool] = !g_see_all();
 
-/* Try to put the palette alongside the frontmost window. */
-
-void
-position_design_window()
-{
-	Point pt;
-	WindowPtr win;
-	GrafPtr oldport;
-
-	/* (should fix all this) */
-	if ((win = FrontWindow()) != nil) {
-		GetPort(&oldport);
-		SetPort(win);
-		SetPt(&pt, win->portRect.right + 3, win->portRect.top);
-		LocalToGlobal(&pt);
-		SetPort(oldport);
-	} else {
-		SetPt(&pt, 500, 50);
+	/* Count the number of tools in use and size window accordingly. */
+	for (i = 0; i < numtooltypes; ++i) {
+		if (enabledtooltype[i])
+		    ++ numtoolsused;
 	}
-	/* (should make sure is not off the edge of the screen) */
-	if (pt.h + 2 * dtoolw > 640 /* is off screen entirely */) {
-		SetPt(&pt, 640 - 2 * dtoolw - 2, 50);
-	}
-	MoveWindow(designwin, pt.h, pt.v, TRUE);
+	SizeWindow(designwin, dtoolw - 1, numtoolsused * dtoolh - 1, 1);
+
+	/* Make this a floating window! */
+	MakeFloat(designwin);
+	SelectTheWindow(designwin);
 }
 
 void
@@ -238,6 +273,8 @@ draw_design_window()
 	if (!active_display(dside) || designwin == nil) return;
 	GetPort(&oldport);
 	SetPort(designwin);
+	TextFont(small_font_id);
+	TextSize(12);
 	/* Draw each tool's palette item. */
 	for (i = 0; i < numtooltypes; ++i) {
 		draw_design_window_tool(i);
@@ -249,14 +286,22 @@ draw_design_window()
 void
 draw_design_window_tool(enum tooltype tool)
 {
-	int enabled = TRUE, paintable = TRUE;
+	int enabled = TRUE, paintable = TRUE, len;
 	char *toolname = NULL, *auxtoolname = NULL;
 	Rect tmprect, imagerect;
 	char imbuf[BUFSIZE];
 	Str255 tmpstr;
+	int i;
 
+	/* Don't draw unused tools. */
+	if (enabledtooltype[tool] == FALSE)
+		return;
 	SetRect(&tmprect, 0, 0, dtoolw, dtoolh);
-	OffsetRect(&tmprect, (tool / (numtooltypes/2)) * dtoolw, (tool % (numtooltypes/2)) * dtoolh);
+	/* Don't include unused tools in offset. */
+	for (i = 0; i < tool; ++i) {
+		if (enabledtooltype[i] == TRUE)
+		     OffsetRect(&tmprect, 0, dtoolh);
+	}
 	EraseRect(&tmprect);
 	/* Confine the image to a square subrect on the left side of the window. */
 	imagerect = tmprect;
@@ -272,7 +317,7 @@ draw_design_window_tool(enum tooltype tool)
 			/* Only do bg terrain type if painting cell terrain. */
 			if (t_is_cell(curttype)) {
 				auxtoolname = t_type_name(curbgttype);
-				/* bg type is always cell subtype. */
+				/* bg type is always a cell subtype. */
 				OffsetRect(&imagerect, 3, 3);
 				draw_terrain_sample(designwin, imagerect, curbgttype);
 				OffsetRect(&imagerect, -6, -6);
@@ -282,13 +327,21 @@ draw_design_window_tool(enum tooltype tool)
 		case unittool:
 			toolname = u_type_name(curutype);
 			InsetRect(&imagerect, (dtoolh - 32) / 2, (dtoolh - 32) / 2);
-		 	draw_unit_image(designwin, imagerect.left, imagerect.top, 32, 32,
-		 					curutype, cursidenumber, 0);
+		 	draw_unit_image(designwin, imagerect.left, imagerect.top, 32, 32, curutype, cursidenumber, 
+		 						(frontmap ? frontmap->sidecolors : default_sidecolors) , 0, 0,
+		 						(frontmap ? frontmap->draw_emblems : default_draw_emblems));
 			/* Gray out the unit if not allowed for the current side. */
 			paintable = type_allowed_on_side(curutype, side_n(cursidenumber));
 			break;
 		case peopletool:
 			toolname = shortest_side_title(side_n(cursidenumber), spbuf);
+			if (paintcontrolmode) {
+				strcpy(tmpbuf, "(");
+				strcat(tmpbuf, toolname);
+				strcat(tmpbuf, " ctrl)");
+				strcpy(spbuf, tmpbuf);
+				toolname = spbuf;
+			}
 			InsetRect(&imagerect, (dtoolh - 16) / 2, (dtoolh - 16) / 2);
 			draw_side_emblem(designwin, imagerect.left, imagerect.top, 16, 16,
 							 cursidenumber, shadow_emblem);
@@ -301,9 +354,17 @@ draw_design_window_tool(enum tooltype tool)
 				if (curfeature != NULL) {
 					toolname = curfeature->name;
 					auxtoolname = curfeature->typename;
+				} else if (curfid == 0) {
+					toolname = "(no feature)";
+					auxtoolname = " ";
+				} else {
+					sprintf(spbuf, "(#%d)", curfid);
+					toolname = spbuf;
+					auxtoolname = " ";
 				}
 			} else {
 				toolname = "Feature";
+				auxtoolname = " ";
 				enabled = FALSE;
 			}
 			break;
@@ -346,8 +407,7 @@ draw_design_window_tool(enum tooltype tool)
 			break;
 		case cloudstool:
 			if (enabledtooltype[cloudstool]) {
-				/* Black is not ideal here... */
-				FillRect(&imagerect, QDPat(black));
+				draw_terrain_sample(designwin, imagerect, curbgttype);
 				if (curcloudtype > 0) {
 					InsetRect(&imagerect, (dtoolh - hws[4]) / 2, (dtoolh - hhs[4]) / 2);
 					draw_clouds(imagerect.left, imagerect.top, 4, curcloudtype);
@@ -388,25 +448,34 @@ draw_design_window_tool(enum tooltype tool)
 			/* ??? */
 			break;
 	}
-	TextSize(12);
 	/* Draw a (short) text string in the image area. */
 	if (strlen(imbuf) > 0) {
-		/* (should center) */
 		MoveTo(tmprect.left + (dtoolh - StringWidth(tmpstr)) / 2, tmprect.bottom - 5);
 		DrawText(imbuf, 0, strlen(imbuf));
 	}
+	/* (should add to following text-drawing code to account for font changes) */
 	if (toolname != NULL) {
 		MoveTo(tmprect.left + dtoolh, tmprect.top + (auxtoolname != NULL ? dtoolh / 4 + 4
 														   : dtoolh / 2 + 5));
-		if (tool == featuretool)
-		  Move(- dtoolh, 0);
-		DrawText(toolname, 0, strlen(toolname));
+		/* Let feature name take up both image and tool name space. */
+		if (tool == featuretool) {
+			Move(2 - dtoolh, 0);
+			len = min(13, strlen(toolname));
+		} else {
+			len = min(8, strlen(toolname));
+		}
+		DrawText(toolname, 0, len);
 	}
 	if (auxtoolname != NULL) {
 		MoveTo(tmprect.left + dtoolh, tmprect.top + (dtoolh * 3) / 4);
-		if (tool == featuretool)
-		  Move(- dtoolh, 0);
-		DrawText(auxtoolname, 0, strlen(auxtoolname));
+		/* Let feature type take up both image and tool name space. */
+		if (tool == featuretool) {
+			Move(2 - dtoolh, 0);
+			len = min(5, strlen(auxtoolname));
+		} else {
+			len = min(8, strlen(auxtoolname));
+		}
+		DrawText(auxtoolname, 0, len);
 	}
 	if (!paintable) {
 		gray_out_rect(&tmprect);
@@ -448,7 +517,7 @@ draw_design_window_tool(enum tooltype tool)
 void
 do_mouse_down_design(Point mouse, int mods)
 {
-	int oldtool, poppedtool, newutype, newbgttype, toolchoice;
+	int oldtool, poppedtool, newutype, newbgttype, toolchoice, clicked, i;
 	Feature *feature;
 	Rect tmprect;
 	ControlHandle control;
@@ -457,7 +526,17 @@ do_mouse_down_design(Point mouse, int mods)
 
 	tmprect.left = 0;  tmprect.right = dtoolw;
 	oldtool = poppedtool = tooltype;
-	toolchoice = (mouse.v / dtoolh) + (mouse.h > dtoolw ? (numtooltypes / 2) : 0);
+	/* Find the box that was clicked. */
+	clicked = mouse.v / dtoolh;
+	toolchoice = 0;
+	/* Convert it into toolchoice. */ 
+	for(toolchoice = 0; toolchoice < numtooltypes; toolchoice++) {
+		if (toolchoice == clicked && enabledtooltype[toolchoice] == TRUE)
+			break;
+		/* Skip disabled tooltypes. */
+		 if (enabledtooltype[toolchoice] == FALSE)
+		 	++clicked;
+	}
 	FindControl(mouse, designwin, &control);
 	if (control == ttypepopup) {
 		TrackControl(control, mouse, (void *) -1);
@@ -477,7 +556,7 @@ do_mouse_down_design(Point mouse, int mods)
 		TrackControl(control, mouse, (void *) -1);
 		choice = LoWord(GetCtlValue(control));
 		if (choice > 0) {
-			cursidenumber = choice;
+			cursidenumber = choice - 1;
 			if (cursidenumber > numsides)
 			  cursidenumber = 0;
 		}
@@ -485,37 +564,41 @@ do_mouse_down_design(Point mouse, int mods)
 	} else if (control == mtypepopup) {
 		TrackControl(control, mouse, (void *) -1);
 		choice = LoWord(GetCtlValue(control));
-		if (choice > 0) curmtype = choice - 1;
+		if (choice > 0)
+		  curmtype = choice - 1;
 		poppedtool = materialtool;
 	} else if (control == featurepopup) {
 		TrackControl(control, mouse, (void *) -1);
 		choice = LoWord(GetCtlValue(control));
 		if (choice > 0) {
-			curfid = choice - 1; /* not reliable */
+			curfid = feature_menu_fids[choice - 1];
 			curfeature = find_feature(curfid);
 		}
 		poppedtool = featuretool;
 	} else if (control == feature_add_button) {
 		sprintf(spbuf, "%d", nextfid);
-		feature = create_feature("feature", copy_string(spbuf));
+		feature = net_create_feature("feature", copy_string(spbuf));
 		if (feature != NULL) {
 			curfeature = feature;
 			curfid = feature->id;
 			feature_rename_dialog(feature);
-			update_feature_menu(feature);
+			build_feature_menu();
 		}
 		poppedtool = featuretool;
 	} else if (control == feature_remove_button) {
-		destroy_feature(curfeature);
+		net_destroy_feature(curfeature);
+		/* Fall back to a known valid current feature. */
 		curfeature = NULL;
 		curfid = 0;
 		build_feature_menu();
 	} else if (control == feature_edit_button) {
 		feature_rename_dialog(curfeature);
+		build_feature_menu();
 		poppedtool = featuretool;
 	} else if (enabledtooltype[toolchoice]) {
 		/* Any other click selects the tool. */
-		if (toolchoice != brushsizetool) tooltype = toolchoice;
+		if (toolchoice != brushsizetool)
+		  tooltype = toolchoice;
 		/* Now handle any shortcuts. */
 		switch (toolchoice) {
 			case notool:
@@ -553,6 +636,8 @@ do_mouse_down_design(Point mouse, int mods)
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click cycle around all the sides. */
 					OPTION_CYCLE(cursidenumber, 0, numsides + 1, 1, mods);
+				} else if (mods & cmdKey) {
+					paintcontrolmode = !paintcontrolmode;
 				}
 				break;
 			case featuretool:
@@ -631,7 +716,7 @@ do_mouse_down_design(Point mouse, int mods)
 	if (tooltype == peopletool || poppedtool == peopletool) {
 		draw_design_window_tool(unittool);
 	}
-	/* (should only draw controls in relevant tools) */
+	/* (should only draw controls in relevant tools - restrict region?) */
 	DrawControls(designwin);
 }
 
@@ -654,13 +739,10 @@ mark_allowed_sides()
 	Side *side;
 
 	for_all_sides(side) {
-		EnableItem(sidemenu, side_number(side));
-		SetItemMark(sidemenu, side_number(side),
+		EnableItem(sidemenu, side_number(side) + 1);
+		SetItemMark(sidemenu, side_number(side) + 1,
 					(type_allowed_on_side(curutype, side) ? diamondMark : noMark));
 	}
-	EnableItem(sidemenu, numsides + 1);
-	SetItemMark(sidemenu, numsides + 1,
-				(type_allowed_on_side(curutype, NULL) ? diamondMark : noMark));
 }
 
 void
@@ -672,7 +754,8 @@ feature_rename_dialog(Feature *feature)
 	DialogPtr win;
 	short itemtype;  Handle itemhandle;  Rect itemrect;
 
-	if (feature == NULL) return;
+	if (feature == NULL)
+	  return;
 	win = GetNewDialog(dFeatureRename, NULL, (DialogPtr) -1L);
 	/* Seed the text items with the original names. */
 	newtypename = feature->typename;
@@ -689,23 +772,22 @@ feature_rename_dialog(Feature *feature)
 	SetIText(itemhandle, tmpstr);
 	ShowWindow(win);
 	while (!done) {
-		/* Deactivate the front window. */
-		activate_window(FrontWindow(), FALSE);
 		SetCursor(&QD(arrow));
 		ModalDialog(NULL, &ditem);
 		switch (ditem) {
 			case diRenameOK:
 				GetDItem(win, diFeatureRenameType, &itemtype, &itemhandle, &itemrect);
-				set_feature_type_name(feature, get_string_from_item(itemhandle));
+				net_set_feature_type_name(feature, get_string_from_item(itemhandle));
 				GetDItem(win, diFeatureRenameName, &itemtype, &itemhandle, &itemrect);
-				set_feature_name(feature, get_string_from_item(itemhandle));
+				net_set_feature_name(feature, get_string_from_item(itemhandle));
 				/* Fall into next case. */
 			case diRenameCancel:
 				done = TRUE;
 				break;
 		}
 	}
-	DisposDialog(win);
+	DisposeDialog(win);
+	update_all_map_windows();
 }
 
 /* Handling of mousedowns in the map when designing. */
@@ -714,7 +796,7 @@ void
 apply_designer_tool(Map *map, int h, int v, int mods)
 {
 	int x, y, dir;
-	int oldt, oldpeop, oldfid, oldtview, olduview;
+	int oldt, oldpeop, oldctrl, oldfid, oldtview, olduview;
 	Unit *unit;
 
 	m_nearest_boundary(map, h, v, &x, &y, &dir);
@@ -727,23 +809,23 @@ apply_designer_tool(Map *map, int h, int v, int mods)
 					   there. */
 					oldt = terrain_at(x, y);
 					painttype = (curttype == oldt ? curbgttype : curttype);
-					paint_cell(dside, x, y, curbrushradius, painttype);
+					net_paint_cell(dside, x, y, curbrushradius, painttype);
 					paint_on_drag(map, h, v, mods);
 					break;
 				case bordersubtype:
 					/* Toggle border on first mouse down. */
-					paint_border(dside, x, y, dir, curttype, -1);
+					net_paint_border(dside, x, y, dir, curttype, -1);
 					/* Dragging then adds or removes, depending on toggle's result. */
 					border_on_drag(map, h, v, mods, border_at(x, y, dir, curttype));
 					break;
 				case connectionsubtype:
 					/* Toggle connection on first mouse down. */
-					paint_connection(dside, x, y, dir, curttype, -1);
+					net_paint_connection(dside, x, y, dir, curttype, -1);
 					/* Dragging then adds or removes, depending on toggle's result. */
 					connect_on_drag(map, h, v, mods, connection_at(x, y, dir, curttype));
 					break;
 				case coatingsubtype:
-					paint_coating(dside, x, y, curbrushradius, curttype, curdepth);
+					net_paint_coating(dside, x, y, curbrushradius, curttype, curdepth);
 					paint_on_drag(map, h, v, mods);
 					break;
 				default:
@@ -757,7 +839,7 @@ apply_designer_tool(Map *map, int h, int v, int mods)
 				beep();
 				return;
 			}
-			unit = designer_create_unit(dside, curutype, cursidenumber, x, y);
+			unit = net_designer_create_unit(dside, curutype, cursidenumber, x, y);
 			if (unit != NULL) {
 				/* Make the new unit automatically be the current selection. */
 				unselect_all(map);
@@ -770,46 +852,54 @@ apply_designer_tool(Map *map, int h, int v, int mods)
 			/* No use for drag painting here, unlike most other tools. */
 			break;
 		case peopletool:
-			/* Paint people or clear, inverting from what is already here. */
-			oldpeop = people_side_at(x, y);
-			paintpeop = (cursidenumber == oldpeop ? NOBODY : cursidenumber);
-			paint_people(dside, x, y, curbrushradius, paintpeop);
+			if (paintcontrolmode) {
+				/* Paint side or clear, inverting from what is already here. */
+				oldctrl = control_side_at(x, y);
+				paintctrl = (cursidenumber == oldctrl ? NOCONTROL : cursidenumber);
+				net_paint_control(dside, x, y, curbrushradius, paintctrl);
+			} else {
+				/* Paint people or clear, inverting from what is already here. */
+				oldpeop = people_side_at(x, y);
+				paintpeop = (cursidenumber == oldpeop ? NOBODY : cursidenumber);
+				net_paint_people(dside, x, y, curbrushradius, paintpeop);
+			}
 			paint_on_drag(map, h, v, mods);
 			break;
 		case featuretool:
 			oldfid = raw_feature_at(x, y);
 			paintfid = (curfid == oldfid ? 0 : curfid);
-			paint_feature(dside, x, y, curbrushradius, paintfid);
+			net_paint_feature(dside, x, y, curbrushradius, paintfid);
 			paint_on_drag(map, h, v, mods);
 			break;
 		case materialtool:
-			paint_material(dside, x, y, curbrushradius, curmtype, curmamount);
+			net_paint_material(dside, x, y, curbrushradius, curmtype, curmamount);
 			paint_on_drag(map, h, v, mods);
 			break;
 		case elevationtool:
-			paint_elevation(dside, x, y, curbrushradius, curelevation);
+			net_paint_elevation(dside, x, y, curbrushradius, curelevation);
 			paint_on_drag(map, h, v, mods);
 			break;
 		case temperaturetool:
-			paint_temperature(dside, x, y, curbrushradius, curtemperature);
+			net_paint_temperature(dside, x, y, curbrushradius, curtemperature);
 			paint_on_drag(map, h, v, mods);
 			break;
 		case cloudstool:
-			paint_clouds(dside, x, y, curbrushradius, curcloudtype, curcloudbottom, curcloudheight);
+			net_paint_clouds(dside, x, y, curbrushradius, curcloudtype, curcloudbottom, curcloudheight);
 			paint_on_drag(map, h, v, mods);
 			break;
 		case windstool:
-			paint_winds(dside, x, y, curbrushradius, curwinddir, curwindforce);
+			net_paint_winds(dside, x, y, curbrushradius, curwinddir, curwindforce);
 			paint_on_drag(map, h, v, mods);
 			break;
 		case viewtool:
 			if (dside->terrview == NULL || dside->unitview == NULL)
 			  break;
 			oldtview = terrain_view(dside, x, y);
-			painttview = (curtview == oldtview ? UNSEEN : curtview);
+			painttview = (oldtview == UNSEEN ? buildtview(terrain_at(x, y)) : UNSEEN);
 			olduview = unit_view(dside, x, y);
 			paintuview = (curuview == olduview ? EMPTY : curuview);
-			paint_view(dside, x, y, curbrushradius, painttview, paintuview);
+			net_paint_view(dside, x, y, curbrushradius, painttview, paintuview);
+			/* (should paint other view layers also) */
 			paint_on_drag(map, h, v, mods);
 			break;
 		default:
@@ -839,36 +929,48 @@ paint_on_drag(Map *map, int h0, int v0, int mods)
 							/* This sort of drag-paint only works for area fillers,
 							   bords/conns use different algorithm. */
 							case cellsubtype:
-								paint_cell(dside, x, y, curbrushradius, painttype);
+								net_paint_cell(dside, x, y, curbrushradius, painttype);
 								break;
 							case coatingsubtype:
-								paint_coating(dside, x, y, curbrushradius, curttype, curdepth);
+								net_paint_coating(dside, x, y, curbrushradius, curttype, curdepth);
 								break;
 						}
 						break;
 					case peopletool:
-						paint_people(dside, x, y, curbrushradius, paintpeop);
+						if (paintcontrolmode)
+						  net_paint_control(dside, x, y, curbrushradius, paintctrl);
+						else
+						  net_paint_people(dside, x, y, curbrushradius, paintpeop);
 						break;
 					case materialtool:
-						paint_material(dside, x, y, curbrushradius, curmtype, curmamount);
+						net_paint_material(dside, x, y, curbrushradius, curmtype, curmamount);
 						break;
 					case featuretool:
-						paint_feature(dside, x, y, curbrushradius, paintfid);
+						net_paint_feature(dside, x, y, curbrushradius, paintfid);
 						break;
 					case elevationtool:
-						paint_elevation(dside, x, y, curbrushradius, curelevation);
+						net_paint_elevation(dside, x, y, curbrushradius, curelevation);
 						break;
 					case temperaturetool:
-						paint_temperature(dside, x, y, curbrushradius, curtemperature);
+						net_paint_temperature(dside, x, y, curbrushradius, curtemperature);
 						break;
 					case cloudstool:
-						paint_clouds(dside, x, y, curbrushradius, curcloudtype, curcloudbottom, curcloudheight);
+						net_paint_clouds(dside, x, y, curbrushradius, curcloudtype, curcloudbottom, curcloudheight);
 						break;
 					case windstool:
-						paint_winds(dside, x, y, curbrushradius, curwinddir, curwindforce);
+						net_paint_winds(dside, x, y, curbrushradius, curwinddir, curwindforce);
 						break;
 					case viewtool:
-						paint_view(dside, x, y, curbrushradius, curtview, curuview);
+						/* If we're painting "seen" area, always set it to be the terrain
+						   etc actually at the given location. */
+						if (painttview != UNSEEN)
+						  painttview = buildtview(terrain_at(x, y));
+						Dprintf("painttview is %d\n", painttview);
+						if (paintuview != EMPTY)
+						  paintuview = unit_view(dside, x, y);
+						Dprintf("paintuview is %d\n", paintuview);
+						net_paint_view(dside, x, y, curbrushradius, painttview, paintuview);
+						/* (should paint other view layers also) */
 						break;
 					default:
 						break;
@@ -894,7 +996,7 @@ border_on_drag(Map *map, int h0, int v0, int mods, int paintmode)
 			pt1 = newmouse;
 			m_nearest_boundary(map, pt1.h, pt1.v, &x, &y, &dir);
 			if (inside_area(x, y)) {
-				paint_border(dside, x, y, dir, curttype, paintmode);
+				net_paint_border(dside, x, y, dir, curttype, paintmode);
 			}
 		}
 	}
@@ -916,7 +1018,7 @@ connect_on_drag(Map *map, int h0, int v0, int mods, int paintmode)
 			pt1 = newmouse;
 			m_nearest_boundary(map, pt1.h, pt1.v, &x, &y, &dir);
 			if (inside_area(x, y)) {
-				paint_connection(dside, x, y, dir, curttype, paintmode);
+				net_paint_connection(dside, x, y, dir, curttype, paintmode);
 			}
 		}
 	}
@@ -952,8 +1054,6 @@ designer_save_dialog()
 	GetDItem(win, diDesignerSaveCompress, &itemtype, &itemhandle, &itemrect);
 	SetCtlValue((ControlHandle) itemhandle, module->compress_layers);
 	while (!done) {
-		/* Deactivate the front window. */
-		activate_window(FrontWindow(), FALSE);
 		SetCursor(&QD(arrow));
 		ModalDialog(NULL, &ditem);
 		switch (ditem) {
@@ -984,7 +1084,7 @@ designer_save_dialog()
 				SetPt(&pnt, 100, 100);
 				sprintf(spbuf, "%s.g", namebuf);
 				c2p(spbuf, tmpstr);
-				SFPutFile(pnt, "\p", tmpstr, /*(DlgHookProcPtr)*/ nil, &reply);
+				SFPutFile(pnt, "\pSave game data as", tmpstr, /*(DlgHookProcPtr)*/ nil, &reply);
 				if (reply.good) {
 					/* Make the location of the file be the current volume. */
 					SetVol(reply.fName, reply.vRefNum);
@@ -1042,7 +1142,8 @@ designer_save_dialog()
 				break;
 		}
 	}
-	DisposDialog(win);
+	DisposeDialog(win);
+	update_all_map_windows();
 }
 
 /* A special dialog that allows for saving a world of different dimensions than is
@@ -1059,8 +1160,6 @@ designer_reshape_dialog(Module *module)
 	
 	win = GetNewDialog(dDesignerReshape, NULL, (DialogPtr) -1L);
 	while (!done) {
-		/* Deactivate the front window. */
-		activate_window(FrontWindow(), FALSE);
   		put_number_into_ditem(diDesignerReshapeOrigWidth, area.width);
   		put_number_into_ditem(diDesignerReshapeOrigHeight, area.height);
   		put_number_into_ditem(diDesignerReshapeOrigWorld, world.circumference);
@@ -1098,7 +1197,8 @@ designer_reshape_dialog(Module *module)
 				break;
 		}
 	}
-	DisposDialog(win);
+	DisposeDialog(win);
+	update_all_map_windows();
 }
 
 #endif /* DESIGNERS */
