@@ -1,5 +1,5 @@
 /* Low-level drawing for the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@ any later version.  See the file COPYING.  */
 #define wind_dir(w) ((w) & 0x07)
 #define wind_force(w) ((w) >> 3)
 #include "macconq.h"
-extern int emblem_position PARAMS ((Image *uimg, char *ename, Image *eimg, int sw, int sh,
-				    int *exxp, int *eyyp, int *ewp, int *ehp));
 
 extern short any_los;
 
@@ -31,23 +29,23 @@ short pref_color_unit_images;
 
 short pref_solid_color_terrain;
 
-PolyHandle polygons[NUMPOWERS];
-int lastpolyx[NUMPOWERS], lastpolyy[NUMPOWERS];
+PolyHandle *polygons;
+int *lastpolyx, *lastpolyy;
 
-RgnHandle cellrgns[NUMPOWERS];
-int lastcellrgnx[NUMPOWERS], lastcellrgny[NUMPOWERS];
-RgnHandle gridcellrgns[NUMPOWERS];
-int lastgridcellrgnx[NUMPOWERS], lastgridcellrgny[NUMPOWERS];
+RgnHandle *cellrgns;
+int *lastcellrgnx, *lastcellrgny;
+RgnHandle *gridcellrgns;
+int *lastgridcellrgnx, *lastgridcellrgny;
 
-RgnHandle cellrgns30[NUMPOWERS];
-int lastcellrgnx30[NUMPOWERS], lastcellrgny30[NUMPOWERS];
-RgnHandle gridcellrgns30[NUMPOWERS];
-int lastgridcellrgnx30[NUMPOWERS], lastgridcellrgny30[NUMPOWERS];
+RgnHandle *cellrgns30;
+int *lastcellrgnx30, *lastcellrgny30;
+RgnHandle *gridcellrgns30;
+int *lastgridcellrgnx30, *lastgridcellrgny30;
 
-RgnHandle cellrgns15[NUMPOWERS];
-int lastcellrgnx15[NUMPOWERS], lastcellrgny15[NUMPOWERS];
-RgnHandle gridcellrgns15[NUMPOWERS];
-int lastgridcellrgnx15[NUMPOWERS], lastgridcellrgny15[NUMPOWERS];
+RgnHandle *cellrgns15;
+int *lastcellrgnx15, *lastcellrgny15;
+RgnHandle *gridcellrgns15;
+int *lastgridcellrgnx15, *lastgridcellrgny15;
 
 int numwindsicns = 0;
 
@@ -61,283 +59,292 @@ extern Image **best_terrain_images = NULL;
 
 static Image **best_unseen_images = NULL;
 
+static void draw_unit_mask(BitMap mm, Rect srcrect, Rect destrect, int wid1, int wid2);
+static void draw_unit_mask_layer(BitMap mm, Rect srcrect, Rect imagerect, int d, int mode);
+
 /* 	Draw an image of the given type of unit in the given rectangle, adding an emblem if requested. */
 
 void
-draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int mod, int emblem)
+draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int sidecolors, int selected, int mod, int emblem)
 {
-	int ex, ey, ew, eh, bx, by, bw, bh, overlap, desperate, sx2, sy2;
+	int ex, ey, ew, eh, bx, by, bw, bh;
+	int iconmasks = FALSE;
+	int usebox = FALSE;
 	char *ename;
 	Rect srcrect, imagerect;
 	CIconHandle cicnhandle;
-	Image *uimg;
 	MacImage *macuimg;
+	Image *uimg;
 	Map *curmap;
-	List *curlist;
 	BitMap portbits;			/* Target BitMap for CopyBits */
-	BitMap bm, mm;				/* icon BitMap & MaskMap */
-	PixMap pm;					/* icon PixMap */
-	
+	BitMap bm, mm;			/* icon BitMap & MaskMap */
+	PixMap pm;			/* icon PixMap */
+
 	/* Filter out very small images. */
 	if (sw <= 1)
-	  return; 
+	    return; 
 
-	/* Find current map or list. */
-	/* (inefficient, should pass in from above) */
-	curmap = map_from_window(win);
-	curlist = list_from_window(win);
-	
 	/* Default colors used in the absence of defined side colors. */
 	RGBForeColor(&forecolor);
 	RGBBackColor(&maskcolor);
-	
-	if (curmap && curmap->sidecolors) {
-		/* Set main icon color & mask color when plotting to maps. */
+
+	/* Optionally use side colors. */
+	if (sidecolors) {
 		if (main_icon_color[e])
-		  RGBForeColor(&(get_sideColor(e, main_icon_color[e])));
+			RGBForeColor(&(get_sideColor(e, main_icon_color[e])));
 		if (icon_mask_color[e])
-		  RGBBackColor(&(get_sideColor(e, icon_mask_color[e])));
-
-	} else if (curlist && curlist->sidecolors) {
-		/* Set main icon color when plotting to lists. */
-		if (main_icon_color[e])
-		  RGBForeColor(&(get_sideColor(e, main_icon_color[e])));
-	}	
-	/* Set destination rectangle. */
-	SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
-
-	/* Plot squares if the picture is too small. */
-	if (sw <= 4) {
-		FillRect(&imagerect, QDPat(black));
-		/* Optionally use a second color to plot the right half of the square */
-		if (curmap->sidecolors && half_icon_color[e] && split_icon[e]) {
-			RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
-			/* Restrict plotting to right half of image. */
-			imagerect.left = imagerect.left + sw / 2;
-			FillRect(&imagerect, QDPat(black));
-		}
-		/* Restore colors before returning! */
-		ForeColor(blackColor);
-		BackColor(whiteColor);
-		return;
+			RGBBackColor(&(get_sideColor(e, icon_mask_color[e])));
+	}	    	
+	/* Optionally set the iconmasks flag. */
+	curmap = map_from_window(win);
+	if (curmap && curmap->iconmasks
+	    || (!curmap && default_iconmasks)) {
+		iconmasks = TRUE;
 	}
-	desperate = FALSE;
+	/* Get the best unit image from family. */
+	uimg = best_image(uimages[u], sw, sh); 
 
-	/* Get best unit image from family */
-	uimg = best_image(uimages[u], sw, sh);
-	if (uimg && uimg->hook) {
+	/* Use unit image if it exists and unit size > 4. */
+	if (sw > 4  && uimg  && uimg->hook) {
 		macuimg = (MacImage *) uimg->hook;
 
-		if (uimg->w < sw || uimg->h < sh) {
-			sx2 = sx + (sw - uimg->w) / 2;  sy2 = sy + (sh - uimg->h) / 2;
-			/* Only change the size of the rectangle being drawn if it's
-			   smaller than what was passed in. */
-			if (uimg->w < sw) {
-			    sx = sx2;
-			    sw = uimg->w;
-			}
-			if (uimg->h < sh) {
-			    sy = sy2;
-			    sh = uimg->h;
-			}
-			SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
+		/* Plot incomplete units in gray. */
+		if (mod) {
+			RGBForeColor(&graycolor);
+			BackColor(whiteColor);
 		}
-	
+		/* Only change the size of the rectangle being drawn if it's
+		   smaller than what was passed in. */
+		if (uimg->w < sw) {
+		    sx += (sw - uimg->w) / 2;  
+		    sw = uimg->w;
+		}
+		if (uimg->h < sh) {
+		    sy += (sh - uimg->h) / 2;
+		    sh = uimg->h;
+		}
+		SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
 		portbits = QD(thePort)->portBits;
 
-		/* Use pict resource if available */
+		/* Use pict resource if available (obsolete?). */
 		if (macuimg->monopict != nil) {
 			DrawPicture(macuimg->monopict, &imagerect);
 
-		/* Use cicn if drawing in color (also in B/W if no ICON or SICN exists) */
+		/* Use cicn if drawing in color (also in B/W if no ICON or SICN exists). */
 		} else if (macuimg->colricon != nil
 				   && (minscreendepth > 1
 					   || (macuimg->monoicon == nil && macuimg->monosicn == nil))) {
 
-			/* Set srcerect and get the cicnhandle */
+			/* Set srcerect and get the cicnhandle. */
 			SetRect(&srcrect, 0, 0, uimg->w, uimg->h);
 			cicnhandle = (CIconHandle) macuimg->colricon;
 
-			/* Get pixmap, bitmap & maskmap */
+			/* Get pixmap, bitmap & maskmap. */
 			pm = (*(cicnhandle))->iconPMap;
 			bm = (*(cicnhandle))->iconBMap;
 			mm = (*(cicnhandle))->iconMask;
 
-			/* cicns in maps */
-			if (curmap) {
-
-				/* Always use the MaskMap as mask if iconmasks is on */
-				if (curmap->iconmasks) {
-					/* First plot the MaskMap srcBic using mask color */
-					CopyBits(&mm, &portbits, &srcrect, &imagerect, srcBic, 0);
-				}
-
-				/* Plot the PixMap if color images are preferred and side colors are off */
-				if (pref_color_unit_images &! curmap->sidecolors) {
-					/* Use white backColor and black forecolor to avoid color distortion */
-					BackColor(whiteColor);	
-					ForeColor(blackColor);
-					/* Optionally use the BitMap as mask here. This makes it possible to use */
-					/* a single "unified" cicn instead of the the "standard" and "color" cicns. */
-					/* It also allows usemasks to control B/W "color" cicns, which otherwise */
-					/* are plotted with a white mask (if available) even if usemasks is off. */
-					if (unified_cicns) {
-						 CopyBits(&bm, &portbits, &srcrect, &imagerect, srcBic, 0);
-					} else {
-						CopyBits(&mm, &portbits, &srcrect, &imagerect, srcBic, 0);
-					}
-					/* Then plot the PixMap itself in its native (non-side) color */
-					CopyBits((BitMap*) &pm, &portbits, &srcrect, &imagerect, srcOr, 0);
-
-				/* Plot the BitMap in all other cases */
-				} else {
-					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
-
-					/* Optionally use a second side color to plot the right half of the icon */
-					if (curmap->sidecolors && half_icon_color[e] && split_icon[e]) {
-						RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
-						/* Restrict plotting to right half of image */
-						srcrect.left 	= srcrect.right / 2;
-						imagerect.left 	= imagerect.left + sw/2;
-						CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
-					}
-				}
-			} else if (curlist) {
-				/* cicns in lists (no need for masks) */
-
-				/* Plot the PixMap if color images are preferred and side colors are off */
-				if (pref_color_unit_images &! curlist->sidecolors) {
-					/* Use black forecolor to avoid color distortion */
-					 ForeColor(blackColor);
-					 CopyBits((BitMap*) &pm, &portbits, &srcrect, &imagerect, srcOr, 0);
-
-				} else {
-					/* Plot the BitMap in all other cases  */
-					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0); 			
-					
-					/* Optionally use a second side color to plot the right half of the icon */
-					if (curlist->sidecolors && half_icon_color[e] && split_icon[e]) {
-						RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
-						/* Restrict plotting to right half of image */
-						srcrect.left 	= srcrect.right/2;
-						imagerect.left 	= imagerect.left + sw/2;
-						CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
-					}
-				}
+			/* Always use mask if iconmasks is on. */
+			if (iconmasks)
+			    CopyBits(&mm, &portbits, &srcrect, &imagerect, srcBic, 0);
 			
+			/* Plot the PixMap if preferred, and always for incomplete units. */
+			if (pref_color_unit_images || mod) {
+				/* Draw selection mask if asked to. */
+				if (selected && curmap->selectionmasks)
+				      draw_unit_mask(mm, srcrect, imagerect, 1, 1);
+				/* Draw white mask under native color pixmap. */
+				if (!sidecolors)
+				    BackColor(whiteColor);	
+				/* Optionally use the BitMap as mask here. This makes it possible to use
+				a single "unified" cicn instead of the the "standard" and "color" cicns. It 
+				also allows usemasks to control B/W "color" cicns, which otherwise are 
+				plotted with a white mask (if available) even if usemasks is off. */
+				if (unified_cicns)
+					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcBic, 0);
+				else	CopyBits(&mm, &portbits, &srcrect, &imagerect, srcBic, 0);
+				/* Then plot the PixMap itself. */
+				CopyBits((BitMap*) &pm, &portbits, &srcrect, &imagerect, srcOr, 0);
+				/* Optionally use a second side color to plot the right half of the icon. */
+				if (sidecolors && half_icon_color[e] && split_icon[e] &! mod) {
+					RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
+					/* Restrict plotting to right half of image. */
+					srcrect.left = srcrect.right / 2;
+					imagerect.left += sw/2;
+					CopyBits((BitMap*) &pm, &portbits, &srcrect, &imagerect, srcOr, 0);
+				}
+
+			/* Plot the BitMap in all other cases. */
 			} else {
-				/* cicns in help window etc (no need for masks or side colors) */
-				/* Plot the PixMap if color images are preferred */
-				if (pref_color_unit_images) {
-					 ForeColor(blackColor);					
-					 CopyBits((BitMap*)&pm, &portbits, &srcrect, &imagerect, srcOr, 0);
-				} else {
-					/* Plot the BitMap in all other cases */
+				if (selected && curmap->selectionmasks)
+				      draw_unit_mask(mm, srcrect, imagerect, 1, 1);
+				CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
+				/* Optionally use a second side color to plot the right half of the icon. */
+				if (sidecolors && half_icon_color[e] && split_icon[e]) {
+					RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
+					/* Restrict plotting to right half of image. */
+					srcrect.left = srcrect.right / 2;
+					imagerect.left += sw/2;
 					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
 				}
 			}
-						
-		/* Use ICON if no pict or cicn exists (or when cicn does but drawing is in B/W) */
+
+		/* Use ICON if no pict or cicn exists (or when cicn does but drawing is in B/W). */
 		} else if (macuimg->monoicon != nil) {
 			SetRect(&srcrect, 0, 0, 32, 32);
 			bm.rowBytes = 4;
 			bm.bounds = srcrect;
 
-			/* Plot the mask ICON if iconmasks is on and plotting is to map */
-			if (curmap && curmap->iconmasks) {
+			/* Plot the mask ICON if iconmasks is on. */
+			if (iconmasks) {
 				if (macuimg->maskicon != nil) {
 					bm.baseAddr = *(macuimg->maskicon);
 					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcBic, 0);
 				} else {
-					/* Use mask box if no mask ICON exists */
+					/* Use mask box if no mask ICON exists. */
 					FillRect(&imagerect, QDPat(white));
 				}
 			}
 
-			/* Now plot the real ICON */ 
+			/* Now plot the real ICON. */ 
 			bm.baseAddr = *(macuimg->monoicon);
+				if (selected && curmap->selectionmasks)
+				draw_unit_mask(bm, srcrect, imagerect, 1, 1);
+			/* Plot incomplete units in gray. */
+			if (mod) {
+				RGBForeColor(&graycolor);
+				BackColor(whiteColor);
+			}
 			CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
-			/* Optionally use a second color to plot the right half of the SICN */
-			if (((curmap && curmap->sidecolors) || (curlist && curlist->sidecolors)) 
-				&& half_icon_color[e] && split_icon[e]) {
+			/* Optionally use a second color to plot the right half of the ICON. */
+			if (sidecolors && half_icon_color[e] && split_icon[e] &! mod) {
 				RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
-				/* Restrict plotting to right half of image */
-				srcrect.left 	= srcrect.right/2;
-				imagerect.left 	= imagerect.left + sw/2;
+				/* Restrict plotting to right half of image. */
+				srcrect.left = srcrect.right/2;
+				imagerect.left += sw/2;
 				CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
 			}
 
-		/* Use SICN if no pict, cicn or ICON exist */
+		/* Use SICN if no pict, cicn or ICON exist. */
 		} else if (macuimg->monosicn != nil) {
 			SetRect(&srcrect, 0, 0, 16, 16);
 			bm.rowBytes = 2;
 			bm.bounds = srcrect;
 
-			/* Plot the mask SICN if iconmasks is on and plotting is to map */
-			if (curmap && curmap->iconmasks) {
+			/* Plot the mask SICN if iconmasks is on and plotting is to map. */
+			if (iconmasks) {
 				if (macuimg->masksicn != nil) {
 					bm.baseAddr = *(macuimg->masksicn);
 					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcBic, 0);
-				/* Use mask box if no mask SICN exists */
+				/* Use mask box if no mask SICN exists. */
 				} else {
 					FillRect(&imagerect, QDPat(white));
 				}
 			}
-
-			/* Now plot the real SICN */
+			/* Now plot the real SICN. */
 			bm.baseAddr = *(macuimg->monosicn);
+			if (selected && curmap->selectionmasks)
+				draw_unit_mask(bm, srcrect, imagerect, 1, 1);
+			/* Plot incomplete units in gray. */
+			if (mod) {
+				RGBForeColor(&graycolor);
+				BackColor(whiteColor);
+			}
 			CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
-			/* Optionally use a second color to plot the right half of the SICN */
-			if (((curmap && curmap->sidecolors) || (curlist && curlist->sidecolors)) 
-				&& half_icon_color[e] && split_icon[e]) {
+			/* Optionally use a second color to plot the right half of the SICN. */
+			if (sidecolors && half_icon_color[e] && split_icon[e] &! mod) {
 				RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
 				/* Restrict plotting to right half of image. */
-				srcrect.left 	= srcrect.right / 2;
-				imagerect.left 	= imagerect.left + sw / 2;
+				srcrect.left = srcrect.right / 2;
+				imagerect.left += sw / 2;
 				CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
 			}
 
-		/* Use box if no pict, cicn, ICON or SICN was found */
-		} else {
-			desperate = TRUE;
-		}
-	} else {
-		desperate = TRUE;
-	}
-	/* If nothing is working, draw a simple black box. */
-	if (desperate) {
+		/* Use a default box if no pict, cicn, ICON or SICN was found. */
+		} else usebox = TRUE;
+	/* and also for very small images or if the image does not exist. */
+	} else usebox = TRUE;
+
+	if (usebox) {
+		SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
 		FillRect(&imagerect, QDPat(black));
-		/* Optionally use a second color to plot the right half of the square */
-		if (((curmap && curmap->sidecolors) || (curlist && curlist->sidecolors)) 
-			&& half_icon_color[e] && split_icon[e]) {
+		/* Optionally use a second color to plot the right half of the square. */
+		if (sidecolors && half_icon_color[e] && split_icon[e]) {
 			RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
-			/* Restrict plotting to right half of image */
-			imagerect.left = imagerect.left + sw/2;
+			/* Restrict plotting to right half of image. */
+			imagerect.left += sw/2;
 			FillRect(&imagerect, QDPat(black));
 		}
+		/* Gray out units under construction. */
+		if (mod) {
+			SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
+			gray_out_rect(&imagerect);
+		}
+		/* Draw selection masks if asked to. */
+		if (selected && curmap->selectionmasks) {
+			SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
+			InsetRect(&imagerect, -1, -1);
+			PenPat(QDPat(white));
+			FrameRect(&imagerect);
+			InsetRect(&imagerect, -1, -1);
+			PenPat(QDPat(black));
+			FrameRect(&imagerect);
+		}
 	}
-
-	/* Gray out units under construction. */
-	if (mod != 0) {
-		gray_out_rect(&imagerect);
-	}
-
 	/* Restore colors before proceeding. */
 	BackColor(whiteColor);							
 	ForeColor(blackColor);							
-
 	/* Now draw a side emblem if asked for. */
-	if (between(0, e, numsides) && emblem) {
+	if (between(0, e, numsides) && emblem && sw > 4 ) {
 		ename = (side_n(e) ? side_n(e)->emblemname : NULL);
-		if (emblem_position(uimg, ename, /*eimg*/ NULL, sw, sh, &ex, &ey, &ew, &eh)) {
+		if (emblem_position(uimg, ename, eimages[e], sw, sh, &ex, &ey, &ew, &eh)) {
 			/* Do the drawing proper. */
 			draw_side_emblem(win, sx + ex, sy + ey, ew, eh, e, plain_emblem);
 		}
 	}
 }
 
+void
+draw_unit_mask(BitMap mm, Rect srcrect, Rect imagerect, int wid1, int wid2)
+{
+	int i;
+	
+	/* First draw the outer layer using the fore color. */ 
+	for (i = wid1 + 1; i <= wid1 + wid2; i++) {
+		draw_unit_mask_layer(mm, srcrect, imagerect, i, srcOr);
+	}
+	/* Then draw the inner layer using the mask color. */
+	for (i = 1; i <= wid1; i++) {
+		draw_unit_mask_layer(mm, srcrect, imagerect, i, srcBic);
+	}
+}
+
+void
+draw_unit_mask_layer(BitMap mm, Rect srcrect, Rect imagerect, int d, int mode)
+{
+	BitMap portbits = QD(thePort)->portBits;
+
+	OffsetRect(&imagerect,   d, d);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* NE */
+	OffsetRect(&imagerect, -d, 0);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* E */
+	OffsetRect(&imagerect, -d, 0);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* SE */
+	OffsetRect(&imagerect, 0, -d);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* S	*/
+	OffsetRect(&imagerect, 0, -d);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* SW */
+	OffsetRect(&imagerect,   d, 0);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* W */
+	OffsetRect(&imagerect,   d, 0);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* NW */
+	OffsetRect(&imagerect,  0, d);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* N */
+	OffsetRect(&imagerect, -d, 0);
+}
+
 /* Draw a given side id's emblem. Uses the current GrafPort.  No iconmasks or sidecolors,
-   since emblems are vary rarely colorizable silhouettes. */
+   since emblems are very rarely colorizable silhouettes. */
 
 void
 draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style)
@@ -485,14 +492,14 @@ calc_best_terrain_images()
 	best_terrain_images = (Image **) xmalloc(NUMPOWERS * numttypes * sizeof(Image *));
 	for_all_terrain_types(t) {
 		for (p = 0; p < NUMPOWERS; ++p) {
-			timg = best_image(timages[t], hws[p], hcs[p]);
+			timg = best_image(timages[t], hws[p], hhs[p]);
 			best_terrain_images[p * numttypes + t] = timg;
 		}
 	}
 	best_unseen_images = (Image **) xmalloc(NUMPOWERS * sizeof(Image *));
 	if (unseen_image != NULL) {
 		for (p = 0; p < NUMPOWERS; ++p) {
-			timg = best_image(unseen_image, hws[p], hcs[p]);
+			timg = best_image(unseen_image, hws[p], hhs[p]);
 			best_unseen_images[p] = timg;
 		}
 	}
@@ -518,18 +525,51 @@ draw_cell_block(int sx, int sy, int n, int power, int t, int over, int angle, in
 	}
 	if (best_terrain_images == NULL)
 	  calc_best_terrain_images();
+	mactimg = NULL;
+	/* Use background color when erasing outside the map. */
+	if (t == BACKTTYPE) {
+		if (hasColorQD) {
+			RGBForeColor(&backcolor);
+			PaintRect(&rect);
+			RGBForeColor(&blackcolor);
+		} else {
+			switch (bggray) {
+				case blackgray:
+					PenPat(QDPat(black));  break;
+				case darkgray:
+					PenPat(QDPat(dkGray));  break;
+				case mediumgray:
+					PenPat(QDPat(gray));  break;
+				case lightgray:
+					PenPat(QDPat(ltGray));  break;
+				case whitegray:
+					PenPat(QDPat(white));  break;
+			}
+			PaintRect(&rect);
+			PenNormal();
+		}	
+		return;
+	} else
 	if (t == NONTTYPE)
 	  timg = best_unseen_images[power];
-	else
-	  timg = best_terrain_images[power * numttypes + t];
-	if (timg && timg->hook) {
-		mactimg = (MacImage *) timg->hook;
+	else {
+		timg = best_terrain_images[power * numttypes + t];
+		if (hasColorQD
+			&& (dosolid || timg == NULL || timg->hook == NULL)
+			&& tcolors[t] != NULL
+			&& minscreendepth > 1) {
+			mactimg = tcolors[t];
+		}
+	}
+	if (timg && timg->hook && mactimg == NULL)
+	  mactimg = (MacImage *) timg->hook;
+	if (mactimg) {
 		if (hasColorQD) {
-			if (dosolid && tcolors[t] != NULL && minscreendepth > 1) {
+			if (dosolid && minscreendepth > 1) {
 				paintcolor = TRUE;
 			} else if (mactimg->colrpat != nil && (minscreendepth > 1 || !mactimg->patdefined)) {
 				FillCRect(&rect, mactimg->colrpat);
-			} else if (tcolors[t] != NULL && minscreendepth > 1) {
+			} else if (minscreendepth > 1) {
 				paintcolor = TRUE;
 			} else if (mactimg->patdefined) {
 				FillRect(&rect, IMG_PAT(mactimg));
@@ -537,22 +577,15 @@ draw_cell_block(int sx, int sy, int n, int power, int t, int over, int angle, in
 		} else {
 			FillRect(&rect, IMG_PAT(mactimg));
 		}
-	} else {
-		/* No image; use a color instead. */
-		if (hasColorQD) {
-			if (tcolors[t] != NULL && minscreendepth > 1) {
-				paintcolor = TRUE;
-			} else {
-				/* can never happen? */
-			}
-		} else {
-			/* can never happen? */
-		}
 	}
 	if (hasColorQD && paintcolor) {
-		cellcolor.red   = tcolors[t]->color.red;
-		cellcolor.green = tcolors[t]->color.green;
-		cellcolor.blue  = tcolors[t]->color.blue;
+		if (t == NONTTYPE)
+			cellcolor = unseencolor;
+		else {	
+			cellcolor.red   = mactimg->color.red;
+			cellcolor.green = mactimg->color.green;
+			cellcolor.blue  = mactimg->color.blue;
+		}
 		RGBForeColor(&cellcolor);
 		PaintRect(&rect);
 		/* Restore the previous color. */
@@ -576,10 +609,11 @@ draw_cell_block(int sx, int sy, int n, int power, int t, int over, int angle, in
 /* Draw the given terrain type into a hexagonal region of the given size and position. */
 
 void
-draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogrid, int dosolid)
+draw_hex_region(int sx, int sy, int power, int t, int over, int angle,
+				int dogrid, int dosolid, int var)
 {
 	int paintcolor = FALSE;
-	Image *timg;
+	Image *timg, *subimg;
 	MacImage *mactimg;
 	RGBColor hexcolor, oldcolor;
 	RgnHandle rgn;
@@ -607,21 +641,72 @@ draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogri
  		lastgridcellrgnx[power] = sx;  lastgridcellrgny[power] = sy;
 		rgn = (dogrid ? gridcellrgns[power] : cellrgns[power]);
 	}
+	mactimg = NULL;
+
+	/* Use background color when erasing outside the map. */
+	if (t == BACKTTYPE) {
+		if (hasColorQD) {
+			RGBForeColor(&backcolor);
+			PaintRgn(rgn);
+			RGBForeColor(&blackcolor);
+		} else {
+			switch (bggray) {
+				case blackgray:
+					PenPat(QDPat(black));  break;
+				case darkgray:
+					PenPat(QDPat(dkGray));  break;
+				case mediumgray:
+					PenPat(QDPat(gray));  break;
+				case lightgray:
+					PenPat(QDPat(ltGray));  break;
+				case whitegray:
+					PenPat(QDPat(white));  break;
+			}
+			PaintRgn(rgn);
+			PenNormal();
+		}	
+		return;
+	} else
+
 	if (t == NONTTYPE)
 	  timg = best_unseen_images[power];
-	else
-	  timg = best_terrain_images[power * numttypes + t];
-	if (hasColorQD
-		&& (dosolid || timg == NULL || timg->hook == NULL)
-		&& tcolors[t] != NULL
-		&& minscreendepth > 1) {
-		paintcolor = TRUE;
-	} else if (timg && timg->hook) {
-		mactimg = (MacImage *) timg->hook;
+	else {
+		timg = best_terrain_images[power * numttypes + t];
+		if (hasColorQD
+			&& (dosolid || timg == NULL || timg->hook == NULL)
+			&& tcolors[t] != NULL
+			&& minscreendepth > 1) {
+			mactimg = tcolors[t];
+		}
+	}
+	subimg = timg;
+	if (timg->numsubimages > 0 && timg->subimages)
+	  subimg = timg->subimages[var % timg->numsubimages];
+	if (subimg && subimg->hook && mactimg == NULL)
+	  mactimg = (MacImage *) subimg->hook;
+	if (mactimg) {
 		if (hasColorQD) {
-			if (mactimg->colrpat != nil && (minscreendepth > 1 || !mactimg->patdefined)) {
+			if (dosolid && minscreendepth > 1) {
+				paintcolor = TRUE;
+			} else if (mactimg->colricon != nil && (minscreendepth > 1 || !mactimg->patdefined)) {
+				Rect srcrect, imagerect;
+				CIconHandle cicnhandle;
+				BitMap portbits;	/* Target BitMap for CopyBits */
+				BitMap mm;			/* icon BitMap & MaskMap */
+				PixMap pm;			/* icon PixMap */
+
+				SetRect(&srcrect, 0, 0, hws[power], hhs[power]);
+				SetRect(&imagerect, sx, sy, sx + hws[power], sy + hhs[power]);
+				portbits = QD(thePort)->portBits;
+				cicnhandle = (CIconHandle) mactimg->colricon;
+				/* Get pixmap & maskmap from cicn. */
+				pm = (*(cicnhandle))->iconPMap;
+				mm = (*(cicnhandle))->iconMask;
+				/* Draw it, using the mask as the copy mask. */
+				CopyMask((BitMap *) &pm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
+			} else if (mactimg->colrpat != nil && (minscreendepth > 1 || !mactimg->patdefined)) {
 				FillCRgn(rgn, mactimg->colrpat);
-			} else if (tcolors[t] != NULL && minscreendepth > 1) {
+			} else if (minscreendepth > 1) {
 				paintcolor = TRUE;
 			} else if (mactimg->patdefined) {
 				/* Fall back on the b/w pattern. */
@@ -630,14 +715,16 @@ draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogri
 		} else {
 			FillRgn(rgn, IMG_PAT(mactimg));
 		}
-	} else {
-		/* We have nothing to draw with. */
 	}
-	if (hasColorQD && paintcolor) {
+	if (paintcolor) {
 		/* Paint a solid color region. */
-		hexcolor.red   = tcolors[t]->color.red;
-		hexcolor.green = tcolors[t]->color.green;
-		hexcolor.blue  = tcolors[t]->color.blue;
+		if (t == NONTTYPE)
+			hexcolor = unseencolor;
+		else {	
+			hexcolor.red   = mactimg->color.red;
+			hexcolor.green = mactimg->color.green;
+			hexcolor.blue  = mactimg->color.blue;
+		}
 		RGBForeColor(&hexcolor);
 		PaintRgn(rgn);
 		/* Restore the previous color. */
@@ -658,8 +745,6 @@ draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogri
 	}
 }
 
-// --- MOVED HERE FROM MACMAP.C ---------------------------------
-
 /* Given a magnification power, compute the clipping regions to be used at
    that power. */
 
@@ -759,14 +844,12 @@ make_cell_clip(int power)
 	SectRgn(gridcellrgns15[power], tmprgn, gridcellrgns15[power]);
 }
 
-// --------------------------------------------------------------
-
 /* Draw a set of borders for the given position. */
 
 void
 draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid)
 {
-	int wid = bwid[power], wid2, dir, tweakedcolor = FALSE;
+	int wid = bwid[power], wid2, dir, paintcolor = FALSE, tweakedcolor = FALSE;
 	int sx1, sy1, sx2, sy2;
 	Image *timg;
 	MacImage *mactimg;
@@ -790,24 +873,23 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 	PenSize(wid, wid);
 	/* Decide on the line color/pattern to use. */
 	timg = best_image(timages[t], wid, wid);
-	if (timg && timg->hook) {
-		mactimg = (MacImage *) timg->hook;
+	mactimg = NULL;
+	if (hasColorQD
+		&& (dosolid || timg == NULL || timg->hook == NULL)
+		&& tcolors[t] != NULL
+		&& minscreendepth > 1) {
+		mactimg = tcolors[t];
+	}
+	if (timg && timg->hook && mactimg == NULL)
+	  mactimg = (MacImage *) timg->hook;
+	if (mactimg) {
 		if (hasColorQD) {
-			if (dosolid && tcolors[t] != NULL && minscreendepth > 1) {
-				cellcolor.red   = tcolors[t]->color.red;
-				cellcolor.green = tcolors[t]->color.green;
-				cellcolor.blue  = tcolors[t]->color.blue;
-				RGBForeColor(&cellcolor);
-				tweakedcolor = TRUE;
+			if (dosolid && minscreendepth > 1) {
+				paintcolor = TRUE;
 			} else if (mactimg->colrpat && (maxscreendepth > 1 || !mactimg->patdefined)) {
 				PenPixPat(mactimg->colrpat);
-			} else if (tcolors[t] != NULL && minscreendepth > 1) {
-				/* A solid color is preferable to a b/w pattern. */
-				cellcolor.red   = tcolors[t]->color.red;
-				cellcolor.green = tcolors[t]->color.green;
-				cellcolor.blue  = tcolors[t]->color.blue;
-				RGBForeColor(&cellcolor);
-				tweakedcolor = TRUE;
+			} else if (minscreendepth > 1) {
+				paintcolor = TRUE;
 			} else {
 				PenPat(IMG_PAT(mactimg));
 			}
@@ -815,30 +897,26 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 			PenPat(IMG_PAT(mactimg));
 		}
 	} else {
-		if (hasColorQD) {
-			if (tcolors[t] != NULL && minscreendepth > 1) {
-				/* A solid color is preferable to gray. */
-				cellcolor.red   = tcolors[t]->color.red;
-				cellcolor.green = tcolors[t]->color.green;
-				cellcolor.blue  = tcolors[t]->color.blue;
-				RGBForeColor(&cellcolor);
-				tweakedcolor = TRUE;
-			} else {
-				PenPat(QDPat(dkGray));
-			}
+		if (hasColorQD && minscreendepth > 1) {
+			paintcolor = TRUE;
 		} else {
 			PenPat(QDPat(dkGray));
 		}
 	}
+	if (paintcolor) {
+		cellcolor.red   = mactimg->color.red;
+		cellcolor.green = mactimg->color.green;
+		cellcolor.blue  = mactimg->color.blue;
+		RGBForeColor(&cellcolor);
+		tweakedcolor = TRUE;
+	}
 	for_all_directions(dir) {
-
-// --- SPEEDS UP DRAWING & PREVENTS DOUBLE LINES AT CLIFF TOP & BOTTOM BUG ----------
-
+#if 0 /* although this is a tempting optimization, it doesn't actually work;
+		 individual cell updates during game get messed up, not to mention
+		 designer popup. */
 		if (dir == EAST || dir == SOUTHWEST || dir == SOUTHEAST)
 			continue;
-
-// ----------------------------------------------------------------------
-
+#endif
 		if (bitmask & (1 << dir)) {
 			/* Actually draw the line. */
 			sx1 = bsx[power][dir];  sy1 = bsy[power][dir];
@@ -867,14 +945,10 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 			PenMode(notPatOr);
 		}
 		for_all_directions(dir) {
-
-// --- SPEEDS UP DRAWING & PREVENTS DOUBLE LINES AT CLIFF TOP & BOTTOM BUG ----------
-
+#if 0
 			if (dir == EAST || dir == SOUTHWEST || dir == SOUTHEAST)
 				continue;
-
-// ----------------------------------------------------------------------
-
+#endif
 			if (bitmask & (1 << dir)) {
 				/* Actually draw the line. */
 				sx1 = bsx[power][dir];  sy1 = bsy[power][dir];
@@ -906,8 +980,8 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 void
 draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid)
 {
-	int dir, wid = cwid[power], lh, ly, xoff, yoff, tweakedcolor = FALSE;
-	Image *timg;
+	int dir, wid = cwid[power], lh, ly, xoff, yoff, paintcolor = FALSE, tweakedcolor = FALSE;
+	Image *timg, *subimg;
 	MacImage *mactimg;
 	RGBColor cellcolor, oldcolor;				
 
@@ -925,26 +999,58 @@ draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int po
 		CopyBits(&(connbitmaps[4]), winbits, &srcrect, &destrect, srcOr, nil);
 		return;
 	}
-	PenSize(wid, wid);
+	if (best_terrain_images == NULL)
+	  calc_best_terrain_images();
+	timg = best_terrain_images[power * numttypes + t];
+	/* If we have an actual connection image with all the subimages, draw it. */
+	if (timg != NULL
+		&& timg->isconnection
+		&& timg->subimages != NULL) {
+		subimg = timg->subimages[bitmask];
+		if (subimg && subimg->hook)
+		  mactimg = (MacImage *) subimg->hook;
+		if (mactimg
+			&& hasColorQD
+			&& mactimg->colricon != nil
+			&& (minscreendepth > 1 || !mactimg->patdefined)) {
+			Rect srcrect, imagerect;
+			CIconHandle cicnhandle;
+			BitMap portbits;	/* Target BitMap for CopyBits */
+			BitMap mm;			/* icon BitMap & MaskMap */
+			PixMap pm;			/* icon PixMap */
+
+			SetRect(&srcrect, 0, 0, hws[power], hhs[power]);
+			SetRect(&imagerect, sx, sy, sx + hws[power], sy + hhs[power]);
+			portbits = QD(thePort)->portBits;
+			cicnhandle = (CIconHandle) mactimg->colricon;
+			/* Get pixmap & maskmap from cicn. */
+			pm = (*(cicnhandle))->iconPMap;
+			mm = (*(cicnhandle))->iconMask;
+			/* Draw it, using the mask as the copy mask. */
+			CopyMask((BitMap *) &pm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
+		}
+		return;
+	}
+	/* Otherwise try drawing a connection the old way, as a patterned line. */
 	timg = best_image(timages[t], wid, wid);
-	if (timg && timg->hook) {
-		mactimg = (MacImage *) timg->hook;
+	PenSize(wid, wid);
+	mactimg = NULL;
+	if (hasColorQD
+		&& (dosolid || timg == NULL || timg->hook == NULL)
+		&& tcolors[t] != NULL
+		&& minscreendepth > 1) {
+		mactimg = tcolors[t];
+	}
+	if (timg && timg->hook && mactimg == NULL)
+	  mactimg = (MacImage *) timg->hook;
+	if (mactimg) {
 		if (hasColorQD) {
-			if (dosolid && tcolors[t] != NULL && minscreendepth > 1) {
-				cellcolor.red   = tcolors[t]->color.red;
-				cellcolor.green = tcolors[t]->color.green;
-				cellcolor.blue  = tcolors[t]->color.blue;
-				RGBForeColor(&cellcolor);
-				tweakedcolor = TRUE;
+			if (dosolid && minscreendepth > 1) {
+				paintcolor = TRUE;
 			} else if (mactimg->colrpat && (maxscreendepth > 1 || !mactimg->patdefined)) {
 				PenPixPat(mactimg->colrpat);
-			} else if (tcolors[t] != NULL && minscreendepth > 1) {
-				/* A solid color is preferable to a b/w pattern. */
-				cellcolor.red   = tcolors[t]->color.red;
-				cellcolor.green = tcolors[t]->color.green;
-				cellcolor.blue  = tcolors[t]->color.blue;
-				RGBForeColor(&cellcolor);
-				tweakedcolor = TRUE;
+			} else if (minscreendepth > 1) {
+				paintcolor = TRUE;
 			} else {
 				PenPat(IMG_PAT(mactimg));
 			}
@@ -952,21 +1058,19 @@ draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int po
 			PenPat(IMG_PAT(mactimg));
 		}
 	} else {
-		if (hasColorQD) {
-			if (tcolors[t] != NULL && minscreendepth > 1) {
-				/* A solid color is preferable to gray. */
-				cellcolor.red   = tcolors[t]->color.red;
-				cellcolor.green = tcolors[t]->color.green;
-				cellcolor.blue  = tcolors[t]->color.blue;
-				RGBForeColor(&cellcolor);
-				tweakedcolor = TRUE;
-			} else {
-				PenPat(QDPat(gray));
-			}
+		if (hasColorQD && minscreendepth > 1) {
+			paintcolor = TRUE;
 		} else {
-			PenPat(QDPat(gray));
+			PenPat(QDPat(dkGray));
 		}
 	}
+	if (paintcolor) {
+		cellcolor.red   = mactimg->color.red;
+		cellcolor.green = mactimg->color.green;
+		cellcolor.blue  = mactimg->color.blue;
+		RGBForeColor(&cellcolor);
+		tweakedcolor = TRUE;
+	}
 	lh = hhs[power];
 	if (angle == 30)
 	  lh /= 4;
@@ -1028,16 +1132,16 @@ draw_terrain_sample(WindowPtr win, Rect tmprect, int t)
 
 	switch (t_subtype(t)) {
 		case cellsubtype:
-			draw_hex_region(tmprect.left, tmprect.top, 4, t, 0, 90, FALSE, dosolid);
+			draw_hex_region(tmprect.left, tmprect.top, 4, t, 0, 90, FALSE, dosolid, 0);
 			break;
 		case bordersubtype:
-			draw_border_line_multiple(win, tmprect.left, tmprect.top, -1, 4, t, 90, 0, dosolid);
+			draw_border_line_multiple(win, tmprect.left, tmprect.top, 63, 4, t, 90, 0, dosolid);
 			break;
 		case connectionsubtype:
-			draw_connection_line_multiple(win, tmprect.left, tmprect.top, -1, 4, t, 90, 0, dosolid);
+			draw_connection_line_multiple(win, tmprect.left, tmprect.top, 63, 4, t, 90, 0, dosolid);
 			break;
 		case coatingsubtype:
-			draw_hex_region(tmprect.left, tmprect.top, 4, t, 0, 90, FALSE, dosolid);
+			draw_hex_region(tmprect.left, tmprect.top, 4, t, 0, 90, FALSE, dosolid, 0);
 			/* Make it a 50% pattern. (should make more obvious somehow?) */
 			gray_out_rect(&tmprect);
 			break;
@@ -1310,7 +1414,7 @@ draw_legend_text(int sx, int sy, int sh, char *legend, int just, int masks, int
 	/* Make sure small font is used. */
 	TextFont(small_font_id);
 
-	sy += 4;	/* Fix for name erasing */	
+	sy += 3;	/* Fix for name erasing */	
 
 	/* Use fixed optimized fonts if asked to do so. */
 	if (optimize) {
@@ -1335,6 +1439,10 @@ draw_legend_text(int sx, int sy, int sh, char *legend, int just, int masks, int
 				TextSize(24);
 				TextFace(bold);
 				break;
+			case 256:
+				TextSize(36);
+				TextFace(bold);
+				break;
 			default:
 				TextSize(9);
 				TextFace(normal);
