@@ -1,5 +1,5 @@
 /* Image families for the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@ any later version.  See the file COPYING.  */
 #endif
 #include <Resources.h>
 #include <Quickdraw.h>
+#include <Icons.h>
 #endif /* THINK_C */
 
 #include "macimf.h"
@@ -45,9 +46,11 @@ extern void mac_add_ppat(ImageFamily *imf, Handle handle);
 
 static MacImage *init_mac_image(Image *img);
 static void mac_interp_image(ImageFamily *imf, Image *img, int force);
+static void mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int force);
 static void mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump);
-static void mac_copy_bytes(char *data, int numbytes, Handle desthandle, int jump, int rowbytes);
-static CTabHandle interp_ctab(Obj *palette);
+static void mac_copy_bytes(char *data, int numbytes, Handle desthandle, int macrowbytes, int rowbytes);
+static void mac_init_cicn_1(Image *img);
+static CTabHandle interp_ctab(Image *img);
 static CTabHandle synth_ctab(void);
 static void convert_ppat(Image *img, PixPatHandle colrpat);
 static void convert_cicn(Image *img, CIconHandle colricon, int *hasmono, int *hasmask);
@@ -63,6 +66,10 @@ init_mac_image(Image *img)
 	for (j = 0; j < 8; ++j)
 	  SET_IMG_PAT (macimg, j, '\F');
 	macimg->generic = img;
+//	
+	macimg->colricon = nil;
+	macimg->cicn_inited = FALSE;
+	
 	return macimg;
 }
 
@@ -92,7 +99,7 @@ mac_load_imf(ImageFamily *imf)
     Handle imfhandle, pathandle, sicnhandle, iconhandle, maskhandle, handle;
     PicHandle pichandle;
     Rect bounds;
-    Str255 namestr, maskstr, im32x32str, im16x16str, im1x1str;
+    Str255 namestr, maskstr, im32x32str, im16x16str, im8x8str, im1x1str;
 	short resid;  ResType restype;  Str255 resname;
 
 	/* Can't do anything without a name for the image family. */
@@ -108,6 +115,9 @@ mac_load_imf(ImageFamily *imf)
 	/* The name of the 16x16 cicn/ppat is always formed by appending " 16x16". */
 	sprintf(tmpstrbuf, "%s 16x16", imf->name);
 	c2p(tmpstrbuf, im16x16str);
+	/* The name of the 8x8 cicn is always formed by appending " 8x8". */
+	sprintf(tmpstrbuf, "%s 8x8", imf->name);
+	c2p(tmpstrbuf, im8x8str);
 	/* The name of the solid color is always formed by appending " 1x1". */
 	sprintf(tmpstrbuf, "%s 1x1", imf->name);
 	c2p(tmpstrbuf, im1x1str);
@@ -140,6 +150,8 @@ mac_load_imf(ImageFamily *imf)
 		mac_add_cicn(imf, handle);
 		handle = GetNamedResource('cicn', im16x16str);
 		mac_add_cicn(imf, handle);
+		handle = GetNamedResource('cicn', im8x8str);
+		mac_add_cicn(imf, handle);
 	} else {
 		/* (should at least try to get the mono part, without using CQD traps) */
 	}
@@ -267,8 +279,6 @@ mac_add_cicn(ImageFamily *imf, Handle handle)
 		}
 	}
 
-// --- HEAP FRAGMENTATION FIX -----------------------------------------
-
 	/* Always unlock this handle or the heap will be messed up beyond all recognition!
 	Moreover, if get_mac_image returns an already loaded cicn, the new cicnhandle is 
 	never used, as macimg->colricon still points to the old cicnhandle at the top of the 
@@ -277,14 +287,6 @@ mac_add_cicn(ImageFamily *imf, Handle handle)
 	HUnlock((Handle) cicnhandle);
 	if (!newcicn)
 		DisposeCIcon(cicnhandle);
-
-//	/* We want to unlock the cicn now, and lock it high later on;
-//	   but not if it was already processed. */
-//	if (newcicn)
-//	  HUnlock((Handle) cicnhandle);
-	
-// --------------------------------------------------------------
-
 }
 
 void
@@ -385,6 +387,39 @@ int force;
 
 void
 mac_interp_image(ImageFamily *imf, Image *img, int force)
+{
+    int i, subi;
+    Image *subimg;
+
+    /* Iterate through any subimages. */
+    if (img->isborder && img->subimages != NULL) {
+		for (i = 0; i < 16; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+		img->numsubimages = 16;
+    } else if (img->isconnection && img->subimages != NULL) {
+		for (i = 0; i < 64; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+		img->numsubimages = 64;
+    } else if (img->istransition && img->subimages != NULL) {
+		for (i = 0; i < 4 * 4; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+		img->numsubimages = 4 * 4;
+    } else if (img->numsubimages > 0 && img->subimages != NULL) {
+		for (i = 0; i < img->numsubimages; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+    } else {
+		mac_interp_image_1(imf, img, img, 0, force);
+    }
+}
+
+/* Interpret a single image/subimage. */
+
+static void
+mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int force)
 {
 	int w, h, i, numbytes, bitrowbytes, actualw, actualh, pixelsize, rowbytes;
 	int macrowbytes;
@@ -399,20 +434,20 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 	actualw = img->actualw;  actualh = img->actualh;
 	pixelsize = img->pixelsize;
 	rowbytes = computed_rowbytes(actualw, pixelsize);
-	macimg = get_mac_image(img);
+	macimg = get_mac_image(subimg);
 	/* Mono icons and masks are very similar; digest both here. */
 	monodone = colrdone = maskdone = FALSE;
-	if (img->monodata != lispnil && (img->rawmonodata == NULL || force)) {
+	if (subimg->monodata != lispnil && (subimg->rawmonodata == NULL || force)) {
 		numbytes = h * computed_rowbytes(w, 1);
-		img->rawmonodata = xmalloc(numbytes);
-		interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
+		subimg->rawmonodata = xmalloc(numbytes);
+		interp_bytes(subimg->monodata, numbytes, subimg->rawmonodata, 0);
 	}
-	if (w == 8 && h == 8 && img->rawmonodata != NULL && img->istile) {
+	if (w == 8 && h == 8 && subimg->rawmonodata != NULL && img->istile) {
 		/* Monochrome pattern. */
 		datahandle = NewHandle(8);
 		memset(*datahandle, 0, 8);
 		/* Read exactly 8 bytes. */
-		mac_copy_bytes(img->rawmonodata, 8, datahandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmonodata, 8, datahandle, rowbytes, rowbytes);
 		/* Fill in the monopat. */
 		for (i = 0; i < 8; ++i)
 		  SET_IMG_PAT(macimg, i, (*datahandle)[i]);
@@ -420,44 +455,44 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 		/* Patterns have no masks, but defeat subsequent mask hacking. */
 		monodone = maskdone = TRUE;
 	}
-	if (w == 16 && h == 16 && img->rawmonodata != NULL) {
+	if (w == 16 && h == 16 && subimg->rawmonodata != NULL) {
 		/* Shape is appropriate for a small icon, make one. */
 		sicnhandle = NewHandle(32);
 		memset(*sicnhandle, 0, 32);
 		/* Read exactly 32 bytes. */
-		mac_copy_bytes(img->rawmonodata, 32, sicnhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmonodata, 32, sicnhandle, rowbytes, rowbytes);
 		macimg->monosicn = sicnhandle;
 		monodone = TRUE;
 	}
-	if (w == 32 && h == 32 && img->rawmonodata != NULL) {
+	if (w == 32 && h == 32 && subimg->rawmonodata != NULL) {
 		/* Shape is appropriate for a standard icon, make one. */
 		iconhandle = NewHandle(128);
 		memset(*iconhandle, 0, 128);
 		/* Read exactly 128 bytes. */
-		mac_copy_bytes(img->rawmonodata, 128, iconhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmonodata, 128, iconhandle, rowbytes, rowbytes);
 		macimg->monoicon = iconhandle;
 		monodone = TRUE;
 	}
-	if (img->maskdata != lispnil && (img->rawmaskdata == NULL || force)) {
+	if (subimg->maskdata != lispnil && (subimg->rawmaskdata == NULL || force)) {
 		numbytes = h * computed_rowbytes(w, 1);
-		img->rawmaskdata = xmalloc(numbytes);
-		interp_bytes(img->maskdata, numbytes, img->rawmaskdata, 0);
+		subimg->rawmaskdata = xmalloc(numbytes);
+		interp_bytes(subimg->maskdata, numbytes, subimg->rawmaskdata, 0);
 	}
-	if (w == 16 && h == 16 && img->rawmaskdata != NULL) {
+	if (w == 16 && h == 16 && subimg->rawmaskdata != NULL) {
 		/* Shape is appropriate for a small icon mask, make one. */
 		sicnhandle = NewHandle(32);
 		memset(*sicnhandle, 0, 32);
 		/* Read exactly 32 bytes. */
-		mac_copy_bytes(img->rawmaskdata, 32, sicnhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmaskdata, 32, sicnhandle, rowbytes, rowbytes);
 		macimg->masksicn = sicnhandle;
 		maskdone = TRUE;
 	}
-	if (w == 32 && h == 32 && img->rawmaskdata != NULL) {
+	if (w == 32 && h == 32 && subimg->rawmaskdata != NULL) {
 		/* Shape is appropriate for a standard icon, make one. */
 		iconhandle = NewHandle(128);
 		memset(*iconhandle, 0, 128);
 		/* Read exactly 128 bytes. */
-		mac_copy_bytes(img->rawmaskdata, 128, iconhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmaskdata, 128, iconhandle, rowbytes, rowbytes);
 		macimg->maskicon = iconhandle;
 		maskdone = TRUE;
 	}
@@ -472,7 +507,7 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 		(*pmhandle)->pixelSize = 1;
 		(*pmhandle)->cmpCount = 1;
 		(*pmhandle)->cmpSize = pixelsize;
-		(*pmhandle)->pmTable = interp_ctab(img->palette);
+		(*pmhandle)->pmTable = interp_ctab(img);
 		datahandle = NewHandle(8);
 		(*ppathandle)->patData = datahandle;
 		HLock(datahandle);
@@ -482,49 +517,33 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 		macimg->colrpat = ppathandle;
 		colrdone = maskdone = TRUE;
 	}
-	if (img->colrdata != lispnil && (img->rawcolrdata == NULL || force)) {
+	if (subimg->colrdata != lispnil && (subimg->rawcolrdata == NULL || force)) {
 		numbytes = h * computed_rowbytes(w, img->pixelsize);
-		img->rawcolrdata = xmalloc(numbytes);
-		interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+		subimg->rawcolrdata = xmalloc(numbytes);
+		interp_bytes(subimg->colrdata, numbytes, subimg->rawcolrdata, 0);
 	}
+    if (subimg->rawcolrdata == NULL && img->filedata != lispnil) {
+		make_image_from_file_image(imf, img, subimg, subi);
+    }
 	/* A sufficiently small tiling color image can be a color pattern. */
-	if (hasColorQD && w <= 64 && h <= 64 && img->istile && img->rawcolrdata != NULL) {
-		macimg->colrpat = mac_create_ppat(img);
+	if (hasColorQD && w <= 64 && h <= 64 && img->istile && subimg->rawcolrdata != NULL) {
+		macimg->colrpat = mac_create_ppat(subimg);
 		colrdone = maskdone = TRUE;
 	}
 	/* Finally, a color icon covers all other cases. */
-	if (hasColorQD && w <= 64 && h <= 64 && !img->istile
-	    && (img->rawcolrdata != NULL
+	if (hasColorQD && w <= 88 && h <= 96 && !img->istile
+	    && (subimg->rawcolrdata != NULL
 	        || (macimg->monosicn == nil
 	            && macimg->masksicn == nil
 	            && macimg->monoicon == nil
 	            && macimg->maskicon == nil))) {
-	    macimg->colricon = (Handle) mac_create_cicn(img);
-	    macimg->cicn_inited = FALSE;
-		mac_init_cicn(img);
+		mac_init_cicn(subimg);
 		colrdone = TRUE;
 	}
+	/* This will mostly happen for images that are too large. */
 	if (!monodone && !colrdone) {
 		run_warning("%dx%d image of \"%s\" could not be interpreted", w, h, imf->name);
 	}
-#if 0 /* broken */
-    /* If we have a color pattern of random size, and an 8x8 mono pattern, then we should
-       glue the mono pattern into the color pattern's mono slot.  This requires searching
-       all pairs of images in the family. */
-	for (img = imf->images; img != NULL; img = img->next) {
-		Image *img2;
-		MacImage *macimg2;
-		macimg = get_mac_image(img);
-		if (macimg->colrpat != nil) {
-			for (img2 = imf->images; img2 != NULL; img2 = img2->next) {
-				macimg2 = get_mac_image(img2);
-				if (macimg2->patdefined) {
-					memcpy((char *) &(*(macimg->colrpat)->pat1Data), (char *) &(macimg2->monopat), 8);
-				}
-			}
-		}
-	}
-#endif
 }
 
 PixPatHandle
@@ -554,11 +573,11 @@ mac_create_ppat(Image *img)
 	(*pmhandle)->pixelSize = pixelsize;
 	(*pmhandle)->cmpCount = 1;
 	(*pmhandle)->cmpSize = pixelsize;
-	(*pmhandle)->pmTable = interp_ctab(img->palette);
+	(*pmhandle)->pmTable = interp_ctab(img);
 	numbytes = actualh * rowbytes;
 	datahandle = NewHandle(numbytes);
 	(*ppathandle)->patData = datahandle;
-	mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+	mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes, rowbytes);
 	(*ppathandle)->patXValid = -1;
 	return ppathandle;
 }
@@ -572,8 +591,8 @@ mac_create_cicn(Image *img)
 	int pixelsize, rowbytes, macrowbytes;
 	MacImage *macimg;
 	Handle sicnhandle, iconhandle, datahandle, monohandle, maskhandle;
-    CIconHandle cicnhandle;
-    PixMapHandle pmhandle;
+	CIconHandle cicnhandle;
+	PixMapHandle pmhandle;
 
 	w = img->w;  h = img->h;
 	actualw = img->actualw;  actualh = img->actualh;
@@ -588,7 +607,9 @@ mac_create_cicn(Image *img)
 	macimg = get_mac_image(img);
 	/* Make a full color icon. */
 	/* Allocate enough space for the icon and its data/mask bitmaps. */
-	cicnhandle = (CIconHandle) NewHandle(sizeof(CIcon) + 2 * actualh * macbitrowbytes);
+	/* Use xmalloc to avoid heap gymnastics. */
+	cicnhandle = (CIconHandle) NewEmptyHandle();
+	*cicnhandle = (CIcon *) xmalloc(sizeof(CIcon) + 2 * actualh * macbitrowbytes);
 	if (cicnhandle == nil)
 	  run_error("out of memory");
 	HLock((Handle) cicnhandle);
@@ -605,7 +626,7 @@ mac_create_cicn(Image *img)
 	(*cicnhandle)->iconPMap.cmpCount = 1;
 	(*cicnhandle)->iconPMap.cmpSize = pixelsize;
 	(*cicnhandle)->iconPMap.planeBytes = 0;
-	(*cicnhandle)->iconPMap.pmTable = interp_ctab(img->palette);
+	(*cicnhandle)->iconPMap.pmTable = interp_ctab(img);
 	(*cicnhandle)->iconPMap.pmReserved = 0;
 	/* Configure the monochrome icon. */
 	SetRect(&((*cicnhandle)->iconBMap.bounds), 0, 0, actualw, actualh);
@@ -616,7 +637,9 @@ mac_create_cicn(Image *img)
 	(*cicnhandle)->iconMask.rowBytes = macbitrowbytes;
 	(*cicnhandle)->iconMask.baseAddr = NULL;
 	numbytes = actualh * macrowbytes;
-	datahandle = NewHandle(numbytes);
+	/* Use xmalloc to avoid heap gymnastics. */
+	datahandle = NewEmptyHandle();
+	*datahandle = xmalloc(numbytes);
 	(*cicnhandle)->iconData = datahandle;
 	/* Fill up the datahandle with the color data, or else use mono/mask data
 	   if the color data is missing. */
@@ -626,14 +649,14 @@ mac_create_cicn(Image *img)
 			img->rawcolrdata = xmalloc(xnumbytes);
 			interp_bytes(img->colrdata, xnumbytes, img->rawcolrdata, 0);
 		}
-		mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+		mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes, rowbytes);
 	} else if (img->monodata != lispnil || img->rawmonodata != NULL) {
 		if (img->rawmonodata == NULL) {
 			int xnumbytes = h * computed_rowbytes(w, 1);
 			img->rawmonodata = xmalloc(xnumbytes);
 			interp_bytes(img->monodata, xnumbytes, img->rawmonodata, 0);
 		}
-		mac_copy_bytes(img->rawmonodata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+		mac_copy_bytes(img->rawmonodata, numbytes, datahandle, macrowbytes, rowbytes);
 		/* Make an ersatz color table. */
 		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
 	} else if (img->maskdata != lispnil || img->rawmaskdata != NULL) {
@@ -642,7 +665,7 @@ mac_create_cicn(Image *img)
 			img->rawmaskdata = xmalloc(xnumbytes);
 			interp_bytes(img->maskdata, xnumbytes, img->rawmaskdata, 0);
 		}
-		mac_copy_bytes(img->rawmaskdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+		mac_copy_bytes(img->rawmaskdata, numbytes, datahandle, macrowbytes, rowbytes);
 		/* Make an ersatz color table. */
 		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
 	} else {
@@ -672,11 +695,32 @@ mac_create_cicn(Image *img)
 			img->rawmonodata = xmalloc(xnumbytes);
 			interp_bytes(img->monodata, xnumbytes, img->rawmonodata, 0);
 		}
-		mac_copy_bytes(img->rawmonodata, numbytes, monohandle, macbitrowbytes - bitrowbytes, bitrowbytes);
+		mac_copy_bytes(img->rawmonodata, numbytes, monohandle, macbitrowbytes, bitrowbytes);
 		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *monohandle, numbytes);
 		HUnlock(monohandle);
+	/* Use mask data if no mono data exists. */
+	} else if (macimg->maskicon != nil) {
+		HLock(macimg->maskicon);
+		memcpy((char *) (*cicnhandle)->iconMaskData + numbytes, *(macimg->maskicon), numbytes);
+		HUnlock(macimg->maskicon);
+	} else if (macimg->masksicn != nil) {
+		HLock(macimg->masksicn);
+		memcpy((char *) (*cicnhandle)->iconMaskData + numbytes, *(macimg->masksicn), numbytes);
+		HUnlock(macimg->masksicn);
+	} else if (img->maskdata != lispnil || img->rawmaskdata != NULL) {
+		monohandle = NewHandle(numbytes);
+		HLock(monohandle);
+		memset(*monohandle, 0, numbytes);
+		if (img->rawmaskdata == NULL) {
+			int xnumbytes = h * computed_rowbytes(w, 1);
+			img->rawmaskdata = xmalloc(xnumbytes);
+			interp_bytes(img->maskdata, xnumbytes, img->rawmaskdata, 0);
+		}
+		mac_copy_bytes(img->rawmaskdata, numbytes, monohandle, macbitrowbytes, bitrowbytes);
+		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *monohandle, numbytes);
+		HUnlock(monohandle);
+	/* No mono or mask data, make it all 1s. */
 	} else {
-		/* No mono data, make it all 1s. */
 		monohandle = NewHandle(numbytes);
 		HLock(monohandle);
 		memset(*monohandle, 0xff, numbytes);
@@ -703,11 +747,32 @@ mac_create_cicn(Image *img)
 			img->rawmaskdata = xmalloc(xnumbytes);
 			interp_bytes(img->maskdata, xnumbytes, img->rawmaskdata, 0);
 		}
-		mac_copy_bytes(img->rawmaskdata, numbytes, maskhandle, macbitrowbytes - bitrowbytes, bitrowbytes);
+		mac_copy_bytes(img->rawmaskdata, numbytes, maskhandle, macbitrowbytes, bitrowbytes);
 		memcpy((char *) (*cicnhandle)->iconMaskData, *maskhandle, numbytes);
 		HUnlock(maskhandle);
+	/* Use mono data if no mask data exists. */
+	} else if (macimg->monoicon != nil) {
+		HLock(macimg->monoicon);
+		memcpy((char *) (*cicnhandle)->iconMaskData, *(macimg->monoicon), numbytes);
+		HUnlock(macimg->monoicon);
+	} else if (macimg->monosicn != nil) {
+		HLock(macimg->monosicn);
+		memcpy((char *) (*cicnhandle)->iconMaskData, *(macimg->monosicn), numbytes);
+		HUnlock(macimg->monosicn);
+	} else if (img->monodata != lispnil || img->rawmonodata != NULL) {
+		maskhandle = NewHandle(numbytes);
+		HLock(maskhandle);
+		memset(*maskhandle, 0, numbytes);
+		if (img->rawmonodata == NULL) {
+			int xnumbytes = h * computed_rowbytes(w, 1);
+			img->rawmonodata = xmalloc(xnumbytes);
+			interp_bytes(img->monodata, xnumbytes, img->rawmonodata, 0);
+		}
+		mac_copy_bytes(img->rawmonodata, numbytes, maskhandle, macbitrowbytes, bitrowbytes);
+		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *maskhandle, numbytes);
+		HUnlock(maskhandle);
+	/* No mask or mono data, make it be the entire area. */
 	} else {
-		/* No mask data, make it be the entire area. */
 		maskhandle = NewHandle(numbytes);
 		HLock(maskhandle);
 		memset(*maskhandle, 0xff, numbytes);
@@ -726,126 +791,139 @@ mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump)
     HUnlock(desthandle);
 }
 
+/* Mac bitmap/pixmap data usually has a an even number of bytes per row, while
+   generic image data can use the smallest number, whether odd or even.  So
+   when we copy from generic to Mac-specific, we may need to insert an empty
+   byte at the end of each row. */
+
 static void
-mac_copy_bytes(char *data, int numbytes, Handle desthandle, int jump, int rowbytes)
+mac_copy_bytes(char *data, int numbytes, Handle desthandle, int macrowbytes, int rowbytes)
 {
 	int i, j;
 
     HLock(desthandle);
-    if (jump == 0) {
+    if (macrowbytes == rowbytes) {
 		memcpy(*desthandle, data, numbytes);
     } else {
 		j = 0;
 		for (i = 0; i < numbytes; ++i) {
-			(*desthandle)[i] = data[j++];
-			if (jump == 1) {
-				++i;
+			if ((i % macrowbytes) >= rowbytes) {
 				(*desthandle)[i] = 0;
+			} else {
+				(*desthandle)[i] = data[j++];
 			}
 		}
     }
     HUnlock(desthandle);
 }
 
-/* This function makes dummy calls to PlotCIcon.  The dummy call is absolutely
-   essential if one wishes to speed up and/or modify plotting by direct access
-   of the cicn PixMap and BitMap, as is now done in draw_unit_image and draw_side_emblem. 
-   The reason for this is that PlotCIcon not only plots the cicn, but also sets up
-   a pointer to the PixMap data at offset zero. If this has not been done at least
-   once, CopyBits cannot handle the cicn. Moreover, since this is a pointer and not
-   a handle, it gets trashed whenever the cicn or PixMap data is moved in memory.
-   This is why a dummy call to PlotCIcon is needed before every access of the PixMap.
-   To avoid this, the cicn, PixMap and CTable are now locked in memory right after
-   the first (and only) PlotCIcon call. To prevent fragmentation, the handles are
-   also moved to the top of the heap zone before this is done. */
-
 void
 mac_init_cicn(Image *img)
 {
-	CIconHandle cicnhandle;
-	MacImage *macimg;
+	int subi;
 
-	macimg = (MacImage *) img->hook;
-	if (macimg->colricon != NULL && !macimg->cicn_inited) {
-		cicnhandle = (CIconHandle) macimg->colricon;
+	if (img->numsubimages > 0) {
+		for (subi = 0; subi < img->numsubimages; ++subi) {
+			mac_init_cicn_1(img->subimages[subi]);
+		}
+	} else {
+		mac_init_cicn_1(img);
+	}
+}
 
-		/* Hang 'em High! */
-		HLockHi((Handle) cicnhandle);
-		HLockHi((Handle) (*cicnhandle)->iconData);
-		HLockHi((Handle) (*cicnhandle)->iconPMap.pmTable);
+	/* This function makes dummy calls to PlotCIcon.  This is absolutely
+	essential if one wishes to speed up and/or modify plotting by direct access
+	of the cicn PixMap and BitMap, as is now done everywhere in the code. 
+	The reason for this is that PlotCIcon not only plots the cicn, but also sets up
+	a pointer to the PixMap data at offset zero. If this has not been done at least
+	once, CopyBits cannot handle the cicn. */
 
-		/* Initialize PixMap pointer by this dummy call. */
-		PlotCIcon(NULL, cicnhandle);
-
-		/* Lock down stuff that got unlocked by PlotCIcon. */
-		HLock((Handle) (*cicnhandle)->iconData);
-		HLock((Handle) cicnhandle);
+void
+mac_init_cicn_1(Image *img)
+{
+	MacImage *macimg = (MacImage *) img->hook;
+	
+	/* First make sure we have a colricon! */
+	if (macimg->colricon == NULL) {
+		macimg->colricon = (Handle) mac_create_cicn(img);
+		macimg->cicn_inited = FALSE;
+	}
+	/* Init the PixMap pointer by a dummy PlotCIcon call. */
+	if (macimg->cicn_inited == FALSE) {
+		PlotCIcon(NULL, (CIconHandle) macimg->colricon);
 		macimg->cicn_inited = TRUE;
 	}
 }
 
-/* Given a list of color table entries, create and return a color table. */
+/* Given an image, create and return a color table, using either its Lisp palette
+   or the raw palette, whichever has colors in it. */
 
 static CTabHandle
-interp_ctab(Obj *palette)
+interp_ctab(Image *img)
 {
 	int len, i, r, g, b;
 	char *colorname;
-	Obj *head, *rest, *color;
+	Obj *palette, *head, *rest, *color;
 	ColorSpec *ctdata;
 	CTabHandle ctabhandle;
 
+	palette = img->palette;
 	if (!consp(palette)) {
 		/* (should look up named palette) */
 	}
-	len = length(palette);
-	if (len == 0)
+	if (palette == lispnil && img->rawpalette == NULL)
 	  return nil;
-	ctabhandle = (CTabHandle) NewHandle(8 + len * 8);
+	len = (palette != lispnil ? length(palette) : img->numcolors);
+	/* Use xmalloc to avoid heap gymnastics. */
+	ctabhandle = (CTabHandle) NewEmptyHandle();
+	*ctabhandle = (ColorTable *) xmalloc(8 + len * 8);
 	HLock((Handle) ctabhandle);
 	(*ctabhandle)->ctFlags = 0;
 	(*ctabhandle)->ctSeed = GetCTSeed();
 	(*ctabhandle)->ctSize = len - 1;
 	ctdata = (ColorSpec *) &((*ctabhandle)->ctTable);
-	for (i = 0, rest = palette; i < len; ++i, rest = cdr(rest)) {
-		head = car(rest);
-		ctdata[i].value = c_number(car(head));
-		color = cdr(head);
-		if (symbolp(car(color)) || stringp(car(color))) {
-			colorname = c_string(car(color));
-			if (strcmp(colorname, "white") == 0) {
-				ctdata[i].rgb.red   = 65535;
-				ctdata[i].rgb.green = 65535;
-				ctdata[i].rgb.blue  = 65535;
-			} else if (strcmp(colorname, "black") == 0) {
-				ctdata[i].rgb.red   = 0;
-				ctdata[i].rgb.green = 0;
-				ctdata[i].rgb.blue  = 0;
-			} else if (strcmp(colorname, "red") == 0) {
-				ctdata[i].rgb.red   = 65535;
-				ctdata[i].rgb.green = 0;
-				ctdata[i].rgb.blue  = 0;
-			} else if (strcmp(colorname, "green") == 0) {
-				ctdata[i].rgb.red   = 0;
-				ctdata[i].rgb.green = 65535;
-				ctdata[i].rgb.blue  = 0;
-			} else if (strcmp(colorname, "blue") == 0) {
-				ctdata[i].rgb.red   = 0;
-				ctdata[i].rgb.green = 0;
-				ctdata[i].rgb.blue  = 65535;
+	if (palette != lispnil) {
+		for (i = 0, rest = palette; i < len; ++i, rest = cdr(rest)) {
+			head = car(rest);
+			ctdata[i].value = c_number(car(head));
+			color = cdr(head);
+			if (symbolp(car(color)) || stringp(car(color))) {
+				colorname = c_string(car(color));
+				/* Look for predefined color names. */
+				if (strcmp(colorname, "white") == 0) {
+					r = 65535;  g = 65535;  b = 65535;
+				} else if (strcmp(colorname, "black") == 0) {
+					r = 0;  g = 0;  b = 0;
+				} else if (strcmp(colorname, "red") == 0) {
+					r = 65535;  g = 0;  b = 0;
+				} else if (strcmp(colorname, "green") == 0) {
+					r = 0;  g = 65535;  b = 0;
+				} else if (strcmp(colorname, "blue") == 0) {
+					r = 0;  g = 0;  b = 65535;
+				} else {
+					init_warning("No color named \"%s\" found, substituting gray",
+								 colorname);
+					r = g = b = i * 5000;
+				}
+			} else if (numberp(car(color))) {
+				/* The usual case is a triple of numbers. */
+				r = c_number(car(color));
+				g = c_number(cadr(color));
+				b = c_number(car(cddr(color)));
 			} else {
-				init_warning("No color named \"%s\" found, substituting gray",
-							 colorname);
-				ctdata[i].rgb.red   = i * 5000;
-				ctdata[i].rgb.green = i * 5000;
-				ctdata[i].rgb.blue  = i * 5000;
+				init_warning("palette color is not a name or set of numbers, substituting black");
+				r = g = b = 0;
 			}
-		} else if (numberp(car(color))) {
-			ctdata[i].rgb.red   = c_number(car(color));
-			ctdata[i].rgb.green = c_number(cadr(color));
-			ctdata[i].rgb.blue  = c_number(car(cddr(color)));
-		} else {
-			init_warning("palette color is not a name or set of numbers, ignoring");
+			ctdata[i].rgb.red   = r;
+			ctdata[i].rgb.green = g;
+			ctdata[i].rgb.blue  = b;
+		}
+	} else {
+		for (i = 0; i < len; ++i) {
+			ctdata[i].value = img->rawpalette[i * 4];
+			ctdata[i].rgb.red   = img->rawpalette[i * 4 + 1];
+			ctdata[i].rgb.green = img->rawpalette[i * 4 + 2];
+			ctdata[i].rgb.blue  = img->rawpalette[i * 4 + 3];
 		}
 	}
 	HUnlock((Handle) ctabhandle);
