@@ -8,7 +8,6 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 extern int ok_to_exit;
-#include "kpublic.h"
 #include "macconq.h"
 extern void recalc_spacing(void);
 extern void invert_map(Map *map, int x, int y, int r);
@@ -50,12 +49,10 @@ int default_draw_unit_numbers = FALSE;
 int default_draw_feature_names = FALSE;
 
 /* True if should display latitude and longitude of locations. */
-
-int default_draw_latlong = FALSE;
+int default_draw_meridians = TRUE;
 
 /* Spacing, in minutes, of latitude/longitude lines. */
-
-int default_latlong_interval = 600;
+int default_meridian_interval = 300;
 
 int
 open_preferences()
@@ -195,10 +192,10 @@ interp_mac_ui_data(Obj *uispec)
 				default_draw_grid = numval;
 			} else if (strcmp(name, "default-draw-names") == 0) {
 				default_draw_names = numval;
-			} else if (strcmp(name, "default-draw-latlong") == 0) {
-				default_draw_latlong = numval;
-			} else if (strcmp(name, "default-latlong-interval") == 0) {
-				default_latlong_interval = numval;
+			} else if (strcmp(name, "default-draw-meridians") == 0) {
+				default_draw_meridians = numval;
+			} else if (strcmp(name, "default-meridian-interval") == 0) {
+				default_meridian_interval = numval;
 			} else if (strcmp(name, "color-unit-images") == 0) {
 				pref_color_unit_images = numval;
 			} else if (strcmp(name, "solid-color-terrain") == 0) {
@@ -229,12 +226,26 @@ interp_mac_ui_data(Obj *uispec)
 			  default_shorelines = numval;
 			else if (strcmp(name, "simple-borders") == 0)
 			  default_simple_borders = numval;
+			else if (strcmp(name, "draw-ai") == 0)
+			  default_draw_ai = numval;
+			else if (strcmp(name, "depth-alert") == 0)
+			  alert_256_colors = numval;
 			else if (strcmp(name, "optimize-fonts") == 0)
 			  default_optimize_fonts = numval;
+			else if (strcmp(name, "draw-emblems") == 0)
+			  default_draw_emblems = numval;
+			else if (strcmp(name, "blinking-curunit") == 0)
+			  default_blinking_curunit = numval;
+			else if (strcmp(name, "selectionmasks") == 0)
+			  default_selectionmasks = numval;
 			else if (strcmp(name, "draw-topline") == 0)
 			  default_draw_topline = numval;
 			else if (strcmp(name, "draw-topunit") == 0)
 			  default_draw_topunit = numval;
+			else if (strcmp(name, "draw-scrollbars") == 0)
+			  default_draw_scrollbars = numval;
+			else if (strcmp(name, "draw-cpanel") == 0)
+			  default_draw_cpanel = numval;
 			else if (strcmp(name, "drawothermaps") == 0)
 			  default_drawothermaps = numval;
 			else if (strcmp(name, "erase-names") == 0)
@@ -299,16 +310,17 @@ interp_mac_ui_data(Obj *uispec)
 				  default_colorscheme[14] = strval;
 			   	else if (strcmp(name, "side15-colors") == 0)
 				  default_colorscheme[15] = strval;
-
+			   	else if (strcmp(name, "backcolor") == 0)
+				  default_featColorName[0] = strval;
 			   	else if (strcmp(name, "forecolor") == 0)
 				  default_featColorName[1] = strval;
 			   	else if (strcmp(name, "maskcolor") == 0)
 				  default_featColorName[2] = strval;
-			   	else if (strcmp(name, "gridcolor") == 0)
-				  default_featColorName[3] = strval;
-			   	else if (strcmp(name, "unseencolor") == 0)
-				  default_featColorName[4] = strval;
 			   	else if (strcmp(name, "textcolor") == 0)
+				  default_featColorName[3] = strval;
+			   	else if (strcmp(name, "gridcolor") == 0)
+				  default_featColorName[4] = strval;
+			   	else if (strcmp(name, "unseencolor") == 0)
 				  default_featColorName[5] = strval;
 			   	else if (strcmp(name, "shorecolor") == 0)
 				  default_featColorName[6] = strval;
@@ -316,6 +328,10 @@ interp_mac_ui_data(Obj *uispec)
 				  default_featColorName[7] = strval;
 			   	else if (strcmp(name, "frontcolor") == 0)
 				  default_featColorName[8] = strval;
+			   	else if (strcmp(name, "meridiancolor") == 0)
+				  default_featColorName[9] = strval;
+			   	else if (strcmp(name, "contourcolor") == 0)
+				  default_featColorName[10] = strval;
 			} else {
 				/* Note unrecognized properties, but don't bother the user. */
 				Dprintf("Preference binding `%s' unrecognized\n", name);
@@ -336,8 +352,8 @@ ui_update_state()
 	push_binding(&state, intern_symbol("checkpoint-interval"), new_number(checkpoint_interval));
 	push_binding(&state, intern_symbol("default-draw-grid"), new_number(default_draw_grid));
 	push_binding(&state, intern_symbol("default-draw-names"), new_number(default_draw_names));
-	push_binding(&state, intern_symbol("default-draw-latlong"), new_number(default_draw_latlong));
-	push_binding(&state, intern_symbol("default-latlong-interval"), new_number(default_latlong_interval));
+	push_binding(&state, intern_symbol("default-draw-meridians"), new_number(default_draw_meridians));
+	push_binding(&state, intern_symbol("default-meridian-interval"), new_number(default_meridian_interval));
 	push_binding(&state, intern_symbol("color-unit-images"), new_number(pref_color_unit_images));
 	push_binding(&state, intern_symbol("solid-color-terrain"), new_number(pref_solid_color_terrain));
 	push_binding(&state, intern_symbol("small-font-size"), new_number(small_font_size));
@@ -353,9 +369,16 @@ ui_update_state()
 	push_binding(&state, intern_symbol("featurenames"), new_number(default_featurenames));
 	push_binding(&state, intern_symbol("shorelines"), new_number(default_shorelines));
 	push_binding(&state, intern_symbol("simple-borders"), new_number(default_simple_borders));
+	push_binding(&state, intern_symbol("draw-ai"), new_number(default_draw_ai));
+	push_binding(&state, intern_symbol("depth-alert"), new_number(alert_256_colors));
 	push_binding(&state, intern_symbol("optimize-fonts"), new_number(default_optimize_fonts));
+	push_binding(&state, intern_symbol("draw-emblems"), new_number(default_draw_emblems));
+	push_binding(&state, intern_symbol("blinking-curunit"), new_number(default_blinking_curunit));
+	push_binding(&state, intern_symbol("selectionmasks"), new_number(default_selectionmasks));
 	push_binding(&state, intern_symbol("draw-topline"), new_number(default_draw_topline));
 	push_binding(&state, intern_symbol("draw-topunit"), new_number(default_draw_topunit));
+	push_binding(&state, intern_symbol("draw-scrollbars"), new_number(default_draw_scrollbars));
+	push_binding(&state, intern_symbol("draw-cpanel"), new_number(default_draw_cpanel));
 	push_binding(&state, intern_symbol("drawothermaps"), new_number(default_drawothermaps));
 	push_binding(&state, intern_symbol("erase-names"), new_number(default_erase_names));
 	push_binding(&state, intern_symbol("x-buffer"), new_number(default_max_bufx));
@@ -390,15 +413,17 @@ ui_update_state()
 		push_binding(&state, intern_symbol("side13-colors"), new_string(default_colorscheme[13]));
 		push_binding(&state, intern_symbol("side14-colors"), new_string(default_colorscheme[14]));
 		push_binding(&state, intern_symbol("side15-colors"), new_string(default_colorscheme[15]));
-
+		push_binding(&state, intern_symbol("backcolor"), new_string(default_featColorName[0]));
 		push_binding(&state, intern_symbol("forecolor"), new_string(default_featColorName[1]));
 		push_binding(&state, intern_symbol("maskcolor"), new_string(default_featColorName[2]));
-		push_binding(&state, intern_symbol("gridcolor"), new_string(default_featColorName[3]));
-		push_binding(&state, intern_symbol("unseencolor"), new_string(default_featColorName[4]));
-		push_binding(&state, intern_symbol("textcolor"), new_string(default_featColorName[5]));
+		push_binding(&state, intern_symbol("textcolor"), new_string(default_featColorName[3]));
+		push_binding(&state, intern_symbol("gridcolor"), new_string(default_featColorName[4]));
+		push_binding(&state, intern_symbol("unseencolor"), new_string(default_featColorName[5]));
 		push_binding(&state, intern_symbol("shorecolor"), new_string(default_featColorName[6]));
 		push_binding(&state, intern_symbol("featurecolor"), new_string(default_featColorName[7]));
 		push_binding(&state, intern_symbol("frontcolor"), new_string(default_featColorName[8]));
+		push_binding(&state, intern_symbol("meridiancolor"), new_string(default_featColorName[9]));
+		push_binding(&state, intern_symbol("contourcolor"), new_string(default_featColorName[10]));
 	}
 	dside->uidata = replace_at_key(dside->uidata, "mac", state);
 }
@@ -541,12 +566,12 @@ set_preferences()
 						default_draw_names = GetCtlValue((ControlHandle) itemhandle);
 						GetDItem(win, diPrefsSolidColorTerrain, &itemtype, &itemhandle, &itemrect);
 						pref_solid_color_terrain = GetCtlValue((ControlHandle) itemhandle);
-						GetDItem(win, diPrefsLatLong, &itemtype, &itemhandle, &itemrect);
-						default_draw_latlong = GetCtlValue((ControlHandle) itemhandle);
+						GetDItem(win, diPrefsMeridians, &itemtype, &itemhandle, &itemrect);
+						default_draw_meridians = GetCtlValue((ControlHandle) itemhandle);
 						get_number_from_ditem(diPrefsIncr, new_incr);
 						/* Can't use directly in get_number_from_ditem macro, can't
 						   guarantee size of variable being written into. */
-						default_latlong_interval = new_incr;
+						default_meridian_interval = new_incr;
 
 						GetDItem(win, diPrefsSideColors, &itemtype, &itemhandle, &itemrect);
 						default_sidecolors = GetCtlValue((ControlHandle) itemhandle);
@@ -566,6 +591,9 @@ set_preferences()
 						GetDItem(win, diPrefsFeatureNames, &itemtype, &itemhandle, &itemrect);
 						default_featurenames = GetCtlValue((ControlHandle) itemhandle);
 
+						GetDItem(win, diPrefsAIRegions, &itemtype, &itemhandle, &itemrect);
+						default_draw_ai = GetCtlValue((ControlHandle) itemhandle);
+
 						GetDItem(win, diPrefsSimpleBorders, &itemtype, &itemhandle, &itemrect);
 						default_simple_borders = GetCtlValue((ControlHandle) itemhandle);
 
@@ -581,6 +609,12 @@ set_preferences()
 						GetDItem(win, diPrefsUnitInfo, &itemtype, &itemhandle, &itemrect);
 						default_draw_topunit = GetCtlValue((ControlHandle) itemhandle);
 
+						GetDItem(win, diPrefsScrollBars, &itemtype, &itemhandle, &itemrect);
+						default_draw_scrollbars = GetCtlValue((ControlHandle) itemhandle);
+
+						GetDItem(win, diPrefsControlPanel, &itemtype, &itemhandle, &itemrect);
+						default_draw_cpanel = GetCtlValue((ControlHandle) itemhandle);
+
 						GetDItem(win, diPrefsOtherMaps, &itemtype, &itemhandle, &itemrect);
 						default_drawothermaps = GetCtlValue((ControlHandle) itemhandle);
 
@@ -672,6 +706,9 @@ set_preferences()
 						/* Dont allow negative values */
 						default_max_bufy = max(*itemnum, 0);
 
+						GetDItem(win, diPrefs256colors, &itemtype, &itemhandle, &itemrect);
+						alert_256_colors = GetCtlValue((ControlHandle) itemhandle);
+
 						break;
 
 					case miPreferencesTopicsImageSets:
@@ -685,6 +722,21 @@ set_preferences()
 						GetDItem(win, diPrefsUnifiedCicns, &itemtype, &itemhandle, &itemrect);
 						unified_cicns = GetCtlValue((ControlHandle) itemhandle);
 
+						GetDItem(win, diPrefsDrawEmblems, &itemtype, &itemhandle, &itemrect);
+						default_draw_emblems = GetCtlValue((ControlHandle) itemhandle);
+
+						GetDItem(win, diPrefsBlinkingUnit, &itemtype, &itemhandle, &itemrect);
+						default_blinking_curunit = GetCtlValue((ControlHandle) itemhandle);
+
+						GetDItem(win, diPrefsAnimatedBox, &itemtype, &itemhandle, &itemrect);
+						default_blinking_curunit = !GetCtlValue((ControlHandle) itemhandle);
+
+						GetDItem(win, diPrefsSelectionMasks, &itemtype, &itemhandle, &itemrect);
+						default_selectionmasks = GetCtlValue((ControlHandle) itemhandle);
+
+						GetDItem(win, diPrefsSelectionBoxes, &itemtype, &itemhandle, &itemrect);
+						default_selectionmasks = !GetCtlValue((ControlHandle) itemhandle);
+
 						break;
 
 					case miPreferencesTopicsColorSchemes:
@@ -775,7 +827,7 @@ set_preferences()
 							case diPrefsGrid:
 							case diPrefsNames:
 							case diPrefsSolidColorTerrain:
-							case diPrefsLatLong:
+							case diPrefsMeridians:
 							case diPrefsSideColors:
 							case diPrefsIconMasks:
 							case diPrefsBoxMasks:
@@ -783,8 +835,11 @@ set_preferences()
 							case diPrefsFeatureBorders:
 							case diPrefsFeatureNames:
 							case diPrefsShoreLines:
+							case diPrefsAIRegions:
 							case diPrefsTopLine:
 							case diPrefsUnitInfo:
+							case diPrefsScrollBars:
+							case diPrefsControlPanel:
 							case diPrefsOtherMaps:
 							case diPrefsEraseNames:
 							/* Toggle checkboxes */
@@ -930,6 +985,11 @@ set_preferences()
 						break;
 					case miPreferencesTopicsOffscreen:
 						switch (ditem) {
+							/* Toggle the checkbox. */
+							case diPrefs256colors:
+								GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, !GetCtlValue((ControlHandle) itemhandle));
+								break;
 							default:
 								break;
 						}
@@ -957,6 +1017,40 @@ set_preferences()
 								SetCtlValue((ControlHandle) itemhandle, !GetCtlValue((ControlHandle) itemhandle));
 								break;
 
+							/* Toggle checkbox */
+							case diPrefsDrawEmblems:
+								GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, !GetCtlValue((ControlHandle) itemhandle));
+								break;
+
+							/* Switch radiobuttons */
+							case diPrefsBlinkingUnit:
+								GetDItem(win, diPrefsBlinkingUnit, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, TRUE);
+								GetDItem(win, diPrefsAnimatedBox, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, FALSE);
+								break;
+							case diPrefsAnimatedBox:
+								GetDItem(win, diPrefsBlinkingUnit, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, FALSE);
+								GetDItem(win, diPrefsAnimatedBox, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, TRUE);
+								break;
+
+							/* Switch radiobuttons */
+							case diPrefsSelectionMasks:
+								GetDItem(win, diPrefsSelectionMasks, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, TRUE);
+								GetDItem(win, diPrefsSelectionBoxes, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, FALSE);
+								break;
+							case diPrefsSelectionBoxes:
+								GetDItem(win, diPrefsSelectionMasks, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, FALSE);
+								GetDItem(win, diPrefsSelectionBoxes, &itemtype, &itemhandle, &itemrect);
+								SetCtlValue((ControlHandle) itemhandle, TRUE);
+								break;
+
 							default:
 								break;
 						}
@@ -973,13 +1067,15 @@ set_preferences()
 							/* Activate set side colors dialog */
 							case diPrefsSidesPopup:
 								GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
-								set_default_side_colors(GetCtlValue((ControlHandle) itemhandle) - 1);
+								if (GetCtlValue((ControlHandle) itemhandle) > 2)
+									set_default_side_colors(GetCtlValue((ControlHandle) itemhandle) - 3);
 								break;
 
 							/* Activate set feature colors dialog */
 							case diPrefsFeaturesPopup:
 								GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
-								set_default_feature_color(GetCtlValue((ControlHandle) itemhandle));
+								if (GetCtlValue((ControlHandle) itemhandle) > 2)
+									set_default_feature_color(GetCtlValue((ControlHandle) itemhandle) - 3);
 								break;
 
 							/* Main icon color radio buttons */
@@ -1118,25 +1214,8 @@ set_preferences()
 	DisposeDialog(win);
 	update_all_map_windows();
 	/* Reflect some kinds of changes immediately. */
-	if (pref_color_unit_images != oldcolor) {
-		/* Get rid of old images and collect new ones. */
-		/* (should be more effective and GC the images themselves too) */
-
-// --- CRASHING BUG FIX -------------------------------------------
-
-		/* dispose_unit_cicns accesses uimages, so don't free until after that call. */
-		
- 		dispose_unit_cicns();
- 		free(uimages);
-
-//		free(uimages);
-// 		dispose_unit_cicns();
- 
-// ------------------------------------------------------------
-
-	  	init_unit_images();
-	}
-
+	if (pref_color_unit_images != oldcolor)
+	  init_unit_images();
 	if (fonts_changed)
 	  recalc_spacing();
 	if (pref_color_unit_images != oldcolor
@@ -1189,9 +1268,9 @@ add_preferences_topic_items(DialogPtr win)
 			SetCtlValue((ControlHandle) itemhandle, default_draw_names);
 			GetDItem(win, diPrefsSolidColorTerrain, &itemtype, &itemhandle, &itemrect);
 			SetCtlValue((ControlHandle) itemhandle, pref_solid_color_terrain);
-			GetDItem(win, diPrefsLatLong, &itemtype, &itemhandle, &itemrect);
-			SetCtlValue((ControlHandle) itemhandle, default_draw_latlong);
-			put_number_into_ditem(diPrefsIncr, default_latlong_interval);
+			GetDItem(win, diPrefsMeridians, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, default_draw_meridians);
+			put_number_into_ditem(diPrefsIncr, default_meridian_interval);
 			GetDItem(win, diPrefsSideColors, &itemtype, &itemhandle, &itemrect);
 			SetCtlValue((ControlHandle) itemhandle, default_sidecolors);
 			GetDItem(win, diPrefsIconMasks, &itemtype, &itemhandle, &itemrect);
@@ -1210,10 +1289,16 @@ add_preferences_topic_items(DialogPtr win)
 			SetCtlValue((ControlHandle) itemhandle, !default_simple_borders);
 			GetDItem(win, diPrefsShoreLines, &itemtype, &itemhandle, &itemrect);
 			SetCtlValue((ControlHandle) itemhandle, default_shorelines);
+			GetDItem(win, diPrefsAIRegions, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, default_draw_ai);
 			GetDItem(win, diPrefsTopLine, &itemtype, &itemhandle, &itemrect);
 			SetCtlValue((ControlHandle) itemhandle, default_draw_topline);
 			GetDItem(win, diPrefsUnitInfo, &itemtype, &itemhandle, &itemrect);
 			SetCtlValue((ControlHandle) itemhandle, default_draw_topunit);
+			GetDItem(win, diPrefsScrollBars, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, default_draw_scrollbars);
+			GetDItem(win, diPrefsControlPanel, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, default_draw_cpanel);
 			GetDItem(win, diPrefsOtherMaps, &itemtype, &itemhandle, &itemrect);
 			SetCtlValue((ControlHandle) itemhandle, default_drawothermaps);
 			GetDItem(win, diPrefsEraseNames, &itemtype, &itemhandle, &itemrect);
@@ -1284,7 +1369,8 @@ add_preferences_topic_items(DialogPtr win)
 			GetDItem(win, diPrefsVertiBuff, &itemtype, &itemhandle, &itemrect);
 			NumToString(default_max_bufy, tmpstr);
 			SetIText(itemhandle, tmpstr);
-			
+			GetDItem(win, diPrefs256colors, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, alert_256_colors);
 			break;
 		case miPreferencesTopicsImageSets:
 			ditl = GetResource('DITL', dsPreferencesImageSets);
@@ -1300,6 +1386,21 @@ add_preferences_topic_items(DialogPtr win)
 			GetDItem(win, diPrefsUnifiedCicns, &itemtype, &itemhandle, &itemrect);
 			SetCtlValue((ControlHandle) itemhandle, unified_cicns);
 			
+			GetDItem(win, diPrefsDrawEmblems, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, default_draw_emblems);
+
+			GetDItem(win, diPrefsBlinkingUnit, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, default_blinking_curunit);
+
+			GetDItem(win, diPrefsAnimatedBox, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, !default_blinking_curunit);
+
+			GetDItem(win, diPrefsSelectionMasks, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, default_selectionmasks);
+
+			GetDItem(win, diPrefsSelectionBoxes, &itemtype, &itemhandle, &itemrect);
+			SetCtlValue((ControlHandle) itemhandle, !default_selectionmasks);
+
 			break;
 		case miPreferencesTopicsColorSchemes:
 			ditl = GetResource('DITL', dsPreferencesColorSchemes);
