@@ -1,25 +1,24 @@
 /* Row drawing functions for the Mac interface to Xconq.
-
-	Contains draw_row and all the functions that it calls.
-
-	Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
+/*	Contains draw_row and all the functions that it calls. */
+
 #include "conq.h"
 #include "macconq.h"
 
 extern int any_thickness;
-extern int contour_interval;
+
+extern int ai_region_at(Side *side, int x, int y);
 
 int tmpdrawlighting;
 int tmpdrawcoverage;
 
 static void draw_terrain_row(Map *map, int x0, int y0, int len);
-static int cell_update(Map *map, int x, int y);
 static void draw_cliffs(Map *map, int x, int y);
 static void draw_contours(Map *map, int x, int y);
 extern void draw_gridlines(Map *map, int x, int y);
@@ -28,42 +27,54 @@ static void draw_borders(Map *map, int x, int y, int b);
 static void draw_connections(Map *map, int x, int y, int c);
 static void draw_feature_boundary_simple(Map *map, int x, int y);
 static void draw_feature_boundary(Map *map, int x, int y);
-static void draw_feature_name(Map *map, int x, int y);
 static void draw_people_row(Map *map, int x, int y);
 static void draw_materials(Map *map, int x, int y);
 static void draw_units(Map *map, int x, int y);
 static void draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh);
 static void draw_ai_region(Map *map, int x, int y);
+static void draw_region(Map *map, int x, int y);
 
-/* These three macros are used to handle cases where an item stretches across the */
-/* edge of a wrapped world. It is then necessary to draw it twice, once at each end. */
-/* The macros should be applied to all drawing functions that take an sx value, rect */
-/* or poly as a parameter. */
+/* These three macros are used to handle cases where an item stretches across the
+   edge of a wrapped world. It is then necessary to draw it twice, once at each end.
+   The macros should be applied to all drawing functions that take an sx value, rect
+   or poly as a parameter. */
 
-/* Repeat (function) once using sx shifted one area width to the right. */
+/* Repeat (function) both if sx < 0 and if sx is within one hex from sxmax. */
 #define UNWRAP_SX(map, sx, function) { \
 	(function); \
-	if (area.xwrap && (sx) < 0) { \
+	if (area.xwrap && (sx) > (map)->vp->sxmax - (map)->vp->hw) { \
+		(sx) -= (map)->vp->sxmax; \
+		(function); \
+		(sx) += (map)->vp->sxmax; \
+	} else if (area.xwrap && (sx) < 0) { \
 		(sx) += (map)->vp->sxmax; \
 		(function); \
 		(sx) -= (map)->vp->sxmax; \
 	} \
 }
 
-/* Repeat (function) once using rect offset one area width to the right. */
+/* Repeat (function) both if rect.left < 0 and if rect.right > sxmax. */
 #define UNWRAP_RECT(map, rect, function) { \
 	(function); \
-	if (area.xwrap && (rect).left < 0) { \
+	if (area.xwrap && (rect).right > (map)->vp->sxmax) { \
+		OffsetRect(&(rect), -(map)->vp->sxmax, 0); \
+		(function); \
+		OffsetRect(&(rect), (map)->vp->sxmax, 0); \
+	} else if (area.xwrap && (rect).left < 0) { \
 		OffsetRect(&(rect), (map)->vp->sxmax, 0); \
 		(function); \
 		OffsetRect(&(rect), -(map)->vp->sxmax, 0); \
 	} \
 }
 
-/* Repeat (function) once using poly offset one area width to the right. */
+/* Repeat (function) both if polyBBox.left < 0 and if polyBBox.right > sxmax. */
 #define UNWRAP_POLY(map, poly, function) { \
 	(function); \
-	if (area.xwrap && (*(*(poly))).polyBBox.left < 0) { \
+	if (area.xwrap && (*(*(poly))).polyBBox.right > (map)->vp->sxmax) { \
+		OffsetPoly((poly), -(map)->vp->sxmax, 0); \
+		(function); \
+		OffsetPoly((poly), (map)->vp->sxmax, 0); \
+	} else if (area.xwrap && (*(*(poly))).polyBBox.left < 0) { \
 		OffsetPoly((poly), (map)->vp->sxmax, 0); \
 		(function); \
 		OffsetPoly((poly), -(map)->vp->sxmax, 0); \
@@ -83,16 +94,21 @@ static void draw_ai_region(Map *map, int x, int y);
           ? (cover(dside, x, y) == 0 ? -1 : 0)  \
           : 0)))
 
+/* Use BACKTTYPE to draw the window background. */
+
 #define cell_terrain(map, x, y)  \
-  ((map->see_all || dside->designer) \
+   ((!in_area(x, y)) ? BACKTTYPE :  \
+   ((map->see_all || dside->designer) \
    ? terrain_at(x, y)  \
    : (terrain_visible(dside, x, y)  \
 	  ? vterrain(terrain_view(dside, x, y))  \
-	  : NONTTYPE))
+	  : NONTTYPE)))
+
+/* Always redraw cells outside the area, that is the window background. */
 
 #define cell_style(map, x, y, power)  \
-  ((map->see_all || terrain_visible(dside, x, y) || unseen_image != NULL)  \
-   ? (power >= 4 ? usepolygons : useblocks)  \
+  ((map->see_all || terrain_visible(dside, x, y) || unseen_image != NULL || !in_area(x, y))  \
+   ? (power >= 3 ? usepolygons : useblocks)  \
    : dontdraw);
 
 /* x0 may be negative here. */
@@ -141,14 +157,8 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 			}
 		}
 		/* Draw grid explicitly if asked to and the quick method using smaller cells doesn't work. */
-		if (map->vp->draw_grid 
-			&& 	( map->erase_names 					/* Leaks into the grid. */
-				|| !grid_matches_unseen 				/* Can't use quick method. */
-				|| ang != 90						/* Can't use quick method.  */
-		    		|| map->vp->draw_plans 				/* Leaks into the grid. */	
-		    		|| map->vp->draw_ai 					/* Leaks into the grid. */
-		    		|| map->vp->draw_elevations)) {			/* Leaks into the grid. */
-			  		draw_gridlines(map, x, y);
+		if (map->vp->draw_grid && map->leaky_grid) {
+			draw_gridlines(map, x, y);
 		}
 		/* Draw shorelines if desired. */
 		if (map->shorelines) { 
@@ -157,9 +167,8 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 		/* Draw connections and borders. */
 		if (any_aux_terrain_defined()) {
 			if (bords_to_draw(map)) {
-				for_all_terrain_types(t) {
+				for_all_border_types(t) {
 					if (map->vp->draw_aux_terrain[t] 
-						&& t_is_border(t) 
 						&& aux_terrain_defined(t)) {
 							draw_borders(map, x, y, t);
 					}
@@ -167,9 +176,8 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 			}
 			/* Connections should always be drawn on top of borders. */
 			if (conns_to_draw(map)) {
-				for_all_terrain_types(t) {
+				for_all_connection_types(t) {
 					if (map->vp->draw_aux_terrain[t] 
-						&& t_is_connection(t) 
 						&& aux_terrain_defined(t)) {
 							draw_connections(map, x, y, t);
 					}
@@ -203,17 +211,15 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 		if (!inside_area(x, y))
 		    continue;
 
-		/* Draw feature boundaries and names. */
-		if (features_defined()) {
-			if (map->featureborders && map->vp->hw > 4) {
-				if (map->simple_borders){ 
-					 draw_feature_boundary_simple(map, x, y);
-				} else draw_feature_boundary(map, x, y);
-			}
-			if (map->featurenames && map->vp->hh > 3) {
-				draw_feature_name(map, x, y);
+		/* Draw feature boundaries. */
+		if (features_defined() && map->featureborders && map->vp->hw > 4) {
+			if (map->simple_borders) { 
+				draw_feature_boundary_simple(map, x, y);
+			} else {
+				draw_feature_boundary(map, x, y);
 			}
 		}
+
 		/* Draw country borders and people rows. */
 		if ((people_sides_defined() || control_sides_defined()) && bwid2[pwr] > 0) {
 			draw_people_row(map, x, y);
@@ -230,8 +236,11 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 		if (map->vp->draw_ai) {
 			draw_ai_region(map, x, y);
 		}
+		if (DebugG) {
+			draw_region(map, x, y);
+		}
 		/* If debugging, draw coverage on top of everything else. */
-		if (DebugG && !all_see_all && map->vp->hw >= 16) {
+		if (DebugG && !dside->see_all && map->vp->hw >= 16) {
 			UNWRAP_SX(map, sx, 
 				draw_coverage(sx, sy, pwr, cover(dside, xw, y), alt_cover(dside, xw, y)));
 		}
@@ -284,27 +293,20 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 		}
 	}
 	/* Draw grid explicitly if asked to and the quick method using smaller cells doesn't work. */
-	if (map->vp->draw_grid 
-		&& 	(map->erase_names 					/* Leaks into the grid. */
-			|| !grid_matches_unseen 				/* Can't use quick method. */
-			|| ang != 90						/* Can't use quick method.  */
-	    		|| map->vp->draw_plans 				/* Leaks into the grid. */	
-	    		|| map->vp->draw_ai 					/* Leaks into the grid. */
-	    		|| map->vp->draw_elevations)) {			/* Leaks into the grid. */
-				for (x = x0; x < x0 + len + 1; ++x)	
-		  			draw_gridlines(map, x, y);		/* Draw one extra cell here! */
+	if (map->vp->draw_grid && map->leaky_grid) {
+		for (x = x0; x < x0 + len + 1; ++x)		/* Draw one extra cell here! */
+		  draw_gridlines(map, x, y);
 	}
 	/* Draw shorelines if desired. */
 	if (map->shorelines) {
-		for (x = x0; x < x0 + len + 1; ++x)
-			draw_shorelines(map, x, y);			/* Draw one extra cell here! */
+		for (x = x0; x < x0 + len + 1; ++x)		/* Draw one extra cell here! */
+		  draw_shorelines(map, x, y);
 	}
 	/* Draw connections and borders. */
 	if (any_aux_terrain_defined()) {
 		if (bords_to_draw(map)) {
-			for_all_terrain_types(t) {
+			for_all_border_types(t) {
 				if (map->vp->draw_aux_terrain[t] 
-					&& t_is_border(t) 
 					&& aux_terrain_defined(t)) {
 						for (x = x0; x < x0 + len; ++x)
 							draw_borders(map, x, y, t);
@@ -313,9 +315,8 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 		}
 		/* Connections should always be drawn on top of borders. */
 		if (conns_to_draw(map)) {
-			for_all_terrain_types(t) {
+			for_all_connection_types(t) {
 				if (map->vp->draw_aux_terrain[t] 
-					&& t_is_connection(t) 
 					&& aux_terrain_defined(t)) {
 						for (x = x0; x < x0 + len; ++x)
 							draw_connections(map, x, y, t);
@@ -373,42 +374,43 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 	if (len <= 0)
 	  return;
 
-	/* Draw feature boundaries and names. */
-	if (features_defined()) {
-		if (map->featureborders && map->vp->hw > 4) {
-			for (x = x0; x < x0 + len; ++x) {
-				if (map->simple_borders) {
-					 draw_feature_boundary_simple(map, x, y);
-				} else draw_feature_boundary(map, x, y);
+	/* Draw feature boundaries. */
+	if (features_defined() && map->featureborders && map->vp->hw > 4) {
+		for (x = x0; x < x0 + len; ++x) {
+			if (map->simple_borders) {
+				draw_feature_boundary_simple(map, x, y);
+			} else {
+				draw_feature_boundary(map, x, y);
 			}
 		}
-		if (map->featurenames && map->vp->hh > 3) {
-			for (x = x0; x < x0 + len; ++x)
-				draw_feature_name(map, x, y);
-		}
 	}
+
 	/* Draw cell materials. */
 	if (any_cell_materials_defined() && map->vp->num_materials_to_draw > 0 && map->vp->hh > 20) {
 		for (x = x0; x < x0 + len; ++x)
-			draw_materials(map, x, y);
+		  draw_materials(map, x, y);
 	}
 	/* Draw people rows and country borders. */
 	if ((people_sides_defined() || control_sides_defined()) && bwid2[pwr] > 0) {
 		for (x = x0; x < x0 + len; ++x)
-			draw_people_row(map, x, y);
+		  draw_people_row(map, x, y);
 	}
 	/* Draw units. */
 	if (map->vp->draw_units && map->vp->hw > 2) {
 		for (x = x0; x < x0 + len; ++x)
-			draw_units(map, x, y);
+		  draw_units(map, x, y);
 	}
 	/* Draw ai regions. */
 	if (map->vp->draw_ai) {
 		for (x = x0; x < x0 + len; ++x)
-			draw_ai_region(map, x, y);
+		  draw_ai_region(map, x, y);
+	}
+	if (DebugG) {
+		for (x = x0; x < x0 + len; ++x)
+		  draw_region(map, x, y);
 	}
 	/* If debugging, draw coverage on top of everything else. */
-	if (DebugG && !all_see_all && map->vp->hw >= 16) {
+	if (DebugG && !dside->see_all && map->vp->hw >= 16) {
 		for (x = x0; x < x0 + len; ++x) {
 			xw = wrapx(x);	
 			xform(map, x, y, &sx, &sy);		/* No need to test for in_area here. */
@@ -421,56 +423,51 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 #endif
 
 /* Draw an entire row of terrain, possibly with a single rectangle fill. */
- /* x0 may be negative. */
+/* x0 may be negative. */
 
 void
 draw_terrain_row(Map *map, int x0, int y, int len)
 {
-	int x0w, x, xw, x1, x1w, sx, sy, i;
-	int pwr = map->vp->power, ang = map->vp->angle;
-	int dogrid = map->vp->draw_grid & grid_matches_unseen; 
-	int dofill = map->vp->draw_cell_pats;
-	int dosolid = map->solid_color_terrain;
-	int inarea, seginarea, style, segstyle, terr, segterr, over, segover, update, segupdate;
+	int 	x0w = wrapx(x0), x, xw, x1 = x0, sx, sy, i = 0, quasirand;
+	int 	style, segstyle, terr, segterr, over, segover;
+	int 	dosolid = map->solid_color_terrain;
+	int 	dofill = map->vp->draw_cell_pats;
+	int 	pwr = map->vp->power;
+	int	ang = map->vp->angle;
+	int 	dogrid = FALSE;
 
+	/* Don't use quick grid method if it would leak. */ 
+	if (map->vp->draw_grid &! map->leaky_grid) 
+		dogrid = TRUE;
+
+	/* This stuff is used in the cell_overlay macro. */
 	tmpdrawlighting = map->vp->draw_lighting;
 	tmpdrawcoverage = (!dside->see_all && map->vp->draw_cover);
-	x0w = wrapx(x0);
-	i = 0;
-	x1 = x0;
-	x1w = wrapx(x1);
-	seginarea = in_area(x0w, y);
+
 	segstyle = cell_style(map, x0w, y, pwr);
 	segterr = cell_terrain(map, x0w, y);
 	segover = cell_overlay(x0w, y);
-	segupdate = (seginarea ? cell_update(map, x0w, y) : FALSE);
-	/* Note that in this loop we do NOT wrap x for drawing, just for
-	   looking at layer data. */
+
+	/* Note that in this loop we do NOT wrap x for drawing, just for looking at layer data. */
 	for (x = x0; x < x0 + len + 1; ++x) {
 		xw = wrapx(x);
-		inarea = in_area(xw, y);
 		style = cell_style(map, xw, y, pwr);
 		terr = cell_terrain(map, xw, y);
 		over = cell_overlay(xw, y);
-		update = (inarea ? cell_update(map, xw, y) : FALSE);
+
 		/* Decide if the run is over and we need to dump some output. */
 		if (x == x0 + len
-			|| x == area.width
-			|| inarea != seginarea
 			|| style != segstyle
 			|| terr != segterr
 			|| over != segover
-			|| update != segupdate
-/*			|| segstyle == useblocks  fixes, but poor performance */
 			|| segstyle == usepictures
 			|| segstyle == usepolygons
 			|| ang != 90) {
-			/* don't draw anything that would match the window's bg */
-#if 0 /* use this for heavy-duty debugging only */
-			DGprintf("Seg is %d,%d len=%d inarea=%d style=%d terr=%d over=%d update=%d\n",
-					 x1, y, i, seginarea, segstyle, segterr, segover, segupdate);
-#endif
-			if (seginarea && segupdate && segstyle != dontdraw) {
+
+			if (segstyle != dontdraw) {
+				/* Never use grid on the window background. */
+				if (segterr == BACKTTYPE) 
+					dogrid = FALSE;
 				xform(map, x1, y, &sx, &sy);
 				switch (segstyle) {
 					case useblocks:
@@ -480,41 +477,22 @@ draw_terrain_row(Map *map, int x0, int y, int len)
 					case usepictures:
 						break;
 					case usepolygons:
+						quasirand = (11 * x1 + (13 * y) + 7) * 41;
 						UNWRAP_SX(map, sx, 
-							draw_hex_region(sx, sy, pwr, segterr, segover, ang, dogrid, dosolid));
+							draw_hex_region(sx, sy, pwr, segterr, segover, ang, dogrid, dosolid, quasirand));
 				}
 			}
 			/* Set up for the next segment. */
 			i = 0;
 			x1 = x;
-			x1w = wrapx(x1);
-			seginarea = inarea;
 			segstyle = style;
 			segterr = terr;
 			segover = over;
-			segupdate = update;
 		}
 		++i;
 	}
 }
 
-/* Is this just to see if a cell is visible? */
-
-int
-cell_update(Map *map, int x, int y)
-{
-#if 0
-	int sx, sy;
-	Rect tmprect;
-
-	xform(map, x, y, &sx, &sy);
-	SetRect(&tmprect, sx, sy, sx + map->vp->hw, sy + map->vp->hh);
-	return (RectInRgn(&tmprect, map->window->visRgn));
-#else
-	return TRUE;
-#endif
-}
-
 static void
 draw_cliffs(Map *map, int x, int y)
 {
@@ -524,6 +502,9 @@ draw_cliffs(Map *map, int x, int y)
 	RGBColor hexcolor, oldcolor;
 
 	xw = wrapx(x);
+	/* Prevents crashes when trying to get thickness of BACKTTYPE. */
+	if (!in_area(xw, y))
+		return;
 	t = cell_terrain(map, xw, y);
 	elev = (elevations_defined() ? elev_at(xw, y) : 0) + t_thickness(t);
 	drawit = FALSE;
@@ -640,8 +621,7 @@ draw_contours(Map *map, int x, int y)
 	int i, sx, sy, numlines;
 	LineSegment *lines;
 
-	init_contour_lines();
-	if (contour_interval < 1)
+	if (map->vp->contour_interval < 1)
 	  return;
 	xform(map, x, y, &sx, &sy);
 	contour_lines_at(map->vp, wrapx(x), y, sx, sy, &lines, &numlines);
@@ -659,6 +639,13 @@ draw_contours(Map *map, int x, int y)
 			MoveTo(lines[i].sx1, lines[i].sy1); 
 			LineTo(lines[i].sx2, lines[i].sy2);
 		}
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
+			lines[i].sx1 -= map->vp->sxmax;
+			lines[i].sx2 -= map->vp->sxmax;
+			MoveTo(lines[i].sx1, lines[i].sy1); 
+			LineTo(lines[i].sx2, lines[i].sy2);
+		}
 	}
 	ForeColor(blackColor);
 }
@@ -679,6 +666,7 @@ draw_gridlines(Map *map, int x, int y)
 	/* (should use generic drawing style test) */
 	if (pwr < 4)
 	  return;
+
 	xw = wrapx(x);
 	/* Dont draw outside the area (causes crashes) */
 	if (!in_area(xw, y))
@@ -738,6 +726,13 @@ draw_gridlines(Map *map, int x, int y)
 			LineTo(sx + dx2, sy + dy2 - 1);
 			sx -= map->vp->sxmax;
 		}
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
+			sx -= map->vp->sxmax;
+			MoveTo(sx + dx1, sy + dy1 - 1);
+			LineTo(sx + dx2, sy + dy2 - 1);
+			sx += map->vp->sxmax;
+		}
 	}
 	PenNormal();
 	ForeColor(blackColor);
@@ -752,7 +747,7 @@ draw_shorelines(Map *map, int x, int y)
 	int	t, elev, offset = 0, cliffs = FALSE;
 	int	pwr = map->vp->power;
 	int	ang = map->vp->angle;
-	int	wid = bwid2[pwr];
+	int	wid = bwid[pwr];
 	int	wid2 = wid / 2;
 	int	xw = wrapx(x);
 
@@ -783,6 +778,7 @@ draw_shorelines(Map *map, int x, int y)
 	for_all_directions(dir) {
 		/* Used to kill xform offset. */
 		int adjust = 0;
+
 		/* Never draw SE and SW shores of a given hex */
 		if (dir == SOUTHEAST || dir == SOUTHWEST)
 		  continue;
@@ -832,7 +828,7 @@ draw_shorelines(Map *map, int x, int y)
 		if (dir == NORTHWEST || dir == NORTHEAST) {
 			if (!t_liquid(terrain_at(xw, y)))
 			  dx1 += 1; 
-			else if (pwr > 3)
+			else if (pwr > 2)
 			  dy1 -= 1;
 		}
 
@@ -843,10 +839,11 @@ draw_shorelines(Map *map, int x, int y)
 		    dy2 /= 2;
 		if (ang == 15) 
 		    dy2 /= 4;
-		if (dir == NORTHWEST || dir == NORTHEAST) {
+		if (dir == NORTHWEST /*|| dir == NORTHEAST*/) {
 			if (!t_liquid(terrain_at(xw, y)))
 			  dx2 += 1; 
-			else if (pwr > 3)
+			else if (pwr > 2)
+			if (pwr > 2)
 			  dy2 -= 1;
 		}
 
@@ -861,6 +858,13 @@ draw_shorelines(Map *map, int x, int y)
 			LineTo(sx + dx2 - wid2, sy + dy2 - 1 + adjust);
 			sx -= map->vp->sxmax;
 		}
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
+			sx -= map->vp->sxmax;
+			MoveTo(sx + dx1 - wid2, sy + dy1 - 1 + adjust);
+			LineTo(sx + dx2 - wid2, sy + dy2 - 1 + adjust);
+			sx += map->vp->sxmax;
+		}
 	}
 	PenNormal();
 	ForeColor(blackColor);
@@ -880,7 +884,7 @@ draw_borders(Map *map, int x, int y, int b)
 	}
 	if (bitmask != 0) {
 		tmpdrawlighting = map->vp->draw_lighting;
-		tmpdrawcoverage = (!all_see_all && map->vp->draw_cover);
+		tmpdrawcoverage = (!dside->see_all && map->vp->draw_cover);
 		over = cell_overlay(xw, y);
 		xform(map, x, y, &sx, &sy);
 		/* (should compute and pass in overlay) */
@@ -898,7 +902,7 @@ draw_connections(Map *map, int x, int y, int c)
 {
 	int xw = wrapx(x), dir, sx, sy, bitmask = 0, over;
 	
-	if (!(map->see_all || terrain_visible(dside, xw, y)) || !any_connections_at(xw, y, c))
+	if (!(map->see_all || terrain_visible(dside, xw, y)) || !any_connections_at(xw, y, c) || !tt_drawable(c, terrain_at(xw, y)))
 	  return;
 	for_all_directions(dir) {
 		if (connection_at(xw, y, dir, c)) {
@@ -907,7 +911,7 @@ draw_connections(Map *map, int x, int y, int c)
 	}
 	if (bitmask != 0) {
 		tmpdrawlighting = map->vp->draw_lighting;
-		tmpdrawcoverage = (!all_see_all && map->vp->draw_cover);
+		tmpdrawcoverage = (!dside->see_all && map->vp->draw_cover);
 		over = cell_overlay(xw, y);
 		xform(map, x, y, &sx, &sy);
 		/* (should compute and pass in overlay) */
@@ -975,6 +979,13 @@ draw_feature_boundary_simple(Map *map, int x, int y)
 			LineTo(sx + dx2 - wid2, sy + dy2 - wid2);
 			sx -= map->vp->sxmax;
 		}
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
+			sx -= map->vp->sxmax;
+			MoveTo(sx + dx1 - wid2, sy + dy1 - wid2);
+			LineTo(sx + dx2 - wid2, sy + dy2 - wid2);
+			sx += map->vp->sxmax;
+		}
 	}
 	PenNormal();
 	ForeColor(blackColor);
@@ -1012,46 +1023,6 @@ draw_feature_boundary(Map *map, int x, int y)
 	}
 }
 
-/* Draw feature name. */
-/* (could precompute what the string will lap over and move or truncate str),
-     should be deterministic for each mag, so redraw doesn't scramble */
-
-void
-draw_feature_name(Map *map, int x, int y)
-{
-	int xw, sx, sy;
-	char *str, buf[BUFSIZE];
-	Feature *feature;
-	
-	xw = wrapx(x);
-	
-	/* Draw the name of a terrain feature. */
-	/* (should limit to one cell of feature, preferably centering on quasi-centroid) */	
-	if (terrain_visible(dside, xw, y)) {
-		feature = feature_at(xw, y);
-		if (feature != NULL) {
-			if (feature->size > 0) {
-				if ((feature->x == xw && feature->y == y)
-					|| (feature->x == 0 && feature->y == 0)
-					|| 0 /* center far away */) {
-					str = feature_desc(feature, buf);
-					if (str != NULL) {
-						xform(map, x, y, &sx, &sy);
-						if (map->featurenames) {
-							RGBForeColor(&featurecolor);
-							/* use sy instead of sy + hh/2 to prevent drawing on top of unit names */
-							UNWRAP_SX(map, sx, 
-								draw_legend_text(sx + map->vp->hw/2, sy, 2 * map->vp->uh, 
-											   str, 0, map->textmasks, map->optimize_fonts));
-							ForeColor(blackColor);
-						}
-					}
-				}
-			}
-		}
-	}
-}
-
 /* Indicate what kind of people are living in the given row. */
 
 static void
@@ -1198,7 +1169,7 @@ draw_units(Map *map, int x, int y)
 				sx += (map->vp->hw - sw) / 2;  sy += (map->vp->hh - sw) / 2;
 				if (unit->occupant != NULL
 					&& sw >= 8
-	    			&& (unit->side == dside || all_see_all || u_see_occupants(unit->type))) {
+	    			&& (unit->side == dside || dside->see_all || u_see_occupants(unit->type))) {
 					/* Draw a "grouping box", in white, but with no occs drawn. */
 					SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
 					s = side_number(unit->side);
@@ -1216,8 +1187,15 @@ draw_units(Map *map, int x, int y)
 						FrameRect(&tmprect));
 	    		}
 				UNWRAP_SX(map, sx, 
-					draw_unit_image(map->window, sx, sy, sw, sh, unit->type, 
-								   side_number(unit->side), !completed(unit), TRUE));
+					draw_unit_image(map->window, sx, sy, sw, sh, unit->type, side_number(unit->side), 
+								   map->sidecolors, 0, !completed(unit), map->draw_emblems));
+
+				/* Draw unit size on top of advanced unit. */
+				if (u_advanced(unit->type) && map->drawsizes) {
+					UNWRAP_SX(map, sx, 
+						draw_unit_size(unit, map->window, sx, sy, sw, sh));
+				}
+
 				/* Indicate if more than one stacked here.  Since this may cover up the unit's
 				   picture, adjust it down as far as possible while remaining inside the
 				   "cell minus grid" area. */
@@ -1230,12 +1208,6 @@ draw_units(Map *map, int x, int y)
 						DrawPicture(dotdotdotpicture, &tmprect));
 				}
 				if (map->vp->draw_names) {
-					/* Draw city names in uniform size and position */
-					if (!mobile(unit->type)) {
-					UNWRAP_SX(map, sx, 
-						draw_unit_name(unit, sx, sy, map->vp->uw, map->vp->uh, 
-									  map->textmasks, map->optimize_fonts)); 
-					} else
 					UNWRAP_SX(map, sx, 
 						draw_unit_name(unit, sx, sy, sw, sh, 
 									  map->textmasks, map->optimize_fonts));
@@ -1258,7 +1230,8 @@ draw_units(Map *map, int x, int y)
 			/* Adjust to unit part of cell. */
 			sx += (map->vp->hw - sw) / 2;  sy += (map->vp->hh - sh) / 2;
 			UNWRAP_SX(map, sx, 
-				draw_unit_image(map->window, sx, sy, sw, sh, u, s, 0, TRUE));
+				draw_unit_image(map->window, sx, sy, sw, sh, u, s, 
+							   map->sidecolors, 0, 0, map->draw_emblems));
 		}
 	}
 }
@@ -1269,7 +1242,7 @@ static void
 draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh)
 {
 	int u = unit->type, s = side_number(unit->side), sx2, sy2, sw2, sh2;
-	int emblem = TRUE;
+	int emblem = map->draw_emblems;
 	Unit *occ;
 	Rect tmprect;
 	
@@ -1278,16 +1251,15 @@ draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh)
 	   will also be visible. */
 	if (unit->transport && unit->side == unit->transport->side)
 	  emblem = FALSE;
-	if (unit->occupant != NULL
-		&& sw > 8
-	    && (dside->see_all
-		    || unit->side == dside
-		    || u_see_occupants(unit->type)
-		    || side_owns_occupant(dside, unit))) {
-
+	if (sw > 8 && occupants_visible(dside, map->vp, unit)) {
 		/* Draw a sort of "grouping box", in white. */
-		/* Make box 1 pixel smaller to avoid leakage of its lower left corner into the grid. */
-		SetRect(&tmprect, sx, sy, sx + sw - 1, sy + sh - 1);
+		SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
+		 /*Make box 1 pixel less tall to avoid leakage of its lower left corner into the grid.
+		 The grouping box can also be 1 pixel wider without problems. This makes things
+		 look much better since the unit images now fit entirely within the box. */
+		tmprect.bottom -= 1;
+		tmprect.right += 1;
+		/* Set mask color for the grouping box. */
 		if (map->boxmasks) {
 			if (map->sidecolors && icon_mask_color[s]) {
 				RGBBackColor(&(get_sideColor(s, icon_mask_color[s])));
@@ -1304,7 +1276,14 @@ draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh)
 		/* Draw the transport in the UL quarter of the box. */
 		m_xform_occupant(map, unit, unit, sx, sy, sw, sh, &sx2, &sy2, &sw2, &sh2);
 		UNWRAP_SX(map, sx2, 
-			draw_unit_image(map->window, sx2, sy2, sw2, sh2, u, s, !completed(unit), TRUE));
+			draw_unit_image(map->window, sx2, sy2, sw2, sh2, u, s, map->sidecolors, 0, 
+												!completed(unit), emblem));
+
+		/* Draw unit size on top of advanced unit. */
+		if (u_advanced(unit->type) && map->drawsizes) 
+			UNWRAP_SX(map, sx2, 
+				draw_unit_size(unit, map->window, sx2, sy2, sw2, sh2));
+
 		if (map->vp->draw_names) {
 			/* Draw city names in uniform size and position */
 			if (!mobile(unit->type)) {
@@ -1321,12 +1300,23 @@ draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh)
 		/* Draw all the occupants, in the bottom half of the box. */
 		for_all_occupants(unit, occ) {
 			m_xform_occupant(map, unit, occ, sx, sy, sw, sh, &sx2, &sy2, &sw2, &sh2);
-			/* Accomodate the smaller box by moving occs 2 pixels up. */
-			draw_unit_and_occs(map, occ, sx2, sy2 - 2, sw2, sh2);	/* RECURSION! */
+			draw_unit_and_occs(map, occ, sx2, sy2, sw2, sh2);	/* RECURSION! */
 		}
 	} else { /* Just draw the one unit. */
-	    UNWRAP_SX(map, sx, 
-		draw_unit_image(map->window, sx, sy, sw, sh, u, s, !completed(unit), emblem));
+	    
+		/* Tweak sy so that occs are moved inside the smaller grouping box. */
+		if (unit->transport)
+		    sy -= 2;
+		UNWRAP_SX(map, sx, 
+			draw_unit_image(map->window, sx, sy, sw, sh, u, s, map->sidecolors, 0, 
+											!completed(unit), emblem));
+
+		/* Draw unit size on top of advanced unit. */
+		if (u_advanced(unit->type) && map->drawsizes) { 
+	    		UNWRAP_SX(map, sx, 
+				draw_unit_size(unit, map->window, sx, sy, sw, sh));
+		}
+
 		if (map->vp->draw_names) {
 			/* Draw city names in uniform size and position */
 			if (!mobile(unit->type)) {
@@ -1367,3 +1357,66 @@ draw_ai_region(Map *map, int x, int y)
 		}
 	}
 }
+
+
+void draw_region_borders(WindowPtr win, int sx, int sy, int bitmask, int power);
+
+void
+draw_region_borders(WindowPtr win, int sx, int sy, int bitmask, int power)
+{
+	int wid, wid2, dir;
+	Rect tmprect;
+
+	if (bwid[power] > 0) {
+		wid = 2;
+		wid2 = 1;
+		PenSize(wid, wid);
+		for_all_directions(dir) {
+			if (bitmask & (1 << dir)) {
+				MoveTo(sx + bsx[power][dir] - wid2 + 1, sy + bsy[power][dir] - wid2 + 1);
+				LineTo(sx + bsx[power][dir+1] - wid2 + 1, sy + bsy[power][dir+1] - wid2 + 1);
+			}
+		}
+		PenMode(notPatCopy);
+		for_all_directions(dir) {
+			if (bitmask & (1 << dir)) {
+				MoveTo(sx + bsx[power][dir] - wid2, sy + bsy[power][dir] - wid2);
+				LineTo(sx + bsx[power][dir+1] - wid2, sy + bsy[power][dir+1] - wid2);
+			}
+		}
+		PenNormal();
+	} else {
+		SetRect(&tmprect, sx, sy, sx + hws[power], sy + hhs[power]);
+		OffsetRect(&tmprect, 1, 1);
+		FillRect(&tmprect, QDPat(black));
+		OffsetRect(&tmprect, -1, -1);
+		FillRect(&tmprect, QDPat(white));
+	}
+}
+
+static void
+draw_region(Map *map, int x, int y)
+{
+	int xw, sx, sy, dir, x1, y1, bitmask = 0;
+	struct a_region *reg, *reg1;
+
+	xw = wrapx(x);
+	reg = aref(area.regions, xw, y);
+	/* Decide which edges are borders of the theater. */
+	for_all_directions(dir) {
+		/* Don't do anything about edge cells. */
+		if (interior_point_in_dir(xw, y, dir, &x1, &y1)) {
+			reg1 = aref(area.regions, wrapx(x1), y1);
+			if (reg != reg1) {
+				bitmask |= 1 << dir;
+			}
+		}
+	}
+	if (bitmask != 0) {
+		xform(map, x, y, &sx, &sy);
+		if (bitmask != 0) {
+		    UNWRAP_SX(map, sx, 
+			draw_region_borders(map->window, sx, sy, bitmask, map->vp->power));
+		}
+	}
+}
