@@ -1,5 +1,5 @@
 /* Map handling for the X11 interface to Xconq.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999, 2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,79 +10,78 @@ any later version.  See the file COPYING.  */
    windows. */
 
 #include "conq.h"
-extern int num_features PARAMS ((void));
 #include "kpublic.h"
-#include "xconq.h"
-void update_unit_controls PARAMS ((Side *side, Map *map));
+#include "xtconq.h"
+void update_unit_controls(Side *side, Map *map);
 
-static void create_map_controls PARAMS ((Side *side, Map *map));
-static void interp_key_command PARAMS ((Side *side, Map *map));
-static void handle_map_click PARAMS ((Side *side, Map *map, int sx, int sy));
-static void help_unit_type PARAMS ((Side *side, Map *map));
-static void help_terrain_type PARAMS ((Side *side, Map *map));
-static void mode_callback PARAMS ((Widget w, XtPointer client_data,
-				  XtPointer call_data));
-static void command_callback PARAMS ((Widget w, XtPointer client_data,
-				     XtPointer call_data));
-static void view_flag_callback PARAMS ((Widget w, XtPointer client_data,
-				       XtPointer call_data));
-static void zoom_callback PARAMS ((Widget w, XtPointer client_data,
-				  XtPointer call_data));
-static void map_help_callback PARAMS ((Widget w, XtPointer client_data,
-				      XtPointer call_data));
-static void unit_type_list_callback PARAMS ((Widget w, XtPointer client_data,
-					    XtPointer call_data));
-static void panner_callback PARAMS ((Widget w, XtPointer client_data,
-				    XtPointer call_data));
-static void porthole_callback PARAMS ((Widget w, XtPointer client_data,
-				      XtPointer call_data));
-static void panner_resize_handler PARAMS ((Widget w, XtPointer client_data,
-					  XEvent *event, Boolean *cont));
-static void MA_keypress PARAMS ((Widget w, XEvent *event, String *params,
-				Cardinal *num_params));
-static void MA_mapexpose PARAMS ((Widget w, XEvent *event, String *params,
-				 Cardinal *num_params));
-static void MA_setcoord PARAMS ((Widget w, XEvent *event, String *params,
-				Cardinal *num_params));
-static void MA_movelook PARAMS ((Widget w, XEvent *event, String *params,
-				Cardinal *num_params));
-static void MA_moveunit PARAMS ((Widget w, XEvent *event, String *params,
-				Cardinal *num_params));
-static void MA_distance PARAMS ((Widget w, XEvent *event, String *params,
-				Cardinal *num_params));
-static void MA_motionsetcoord PARAMS ((Widget w, XEvent *event, String *params,
-				      Cardinal *num_params));
-static void MA_toolaction PARAMS ((Widget w, XEvent *event, String *params,
-				  Cardinal *num_params));
-static void MA_toolselection PARAMS ((Widget w, XEvent *event, String *params,
-				     Cardinal *num_params));
-static void MA_movelook_ul PARAMS ((Widget wdgt, XEvent *event, String *params,
-				   Cardinal *num_params));
-static void MA_movelook_l  PARAMS ((Widget wdgt, XEvent *event, String *params,
-				   Cardinal *num_params));
-static void MA_movelook_dl PARAMS ((Widget wdgt, XEvent *event, String *params,
-				   Cardinal *num_params));
-static void MA_movelook_ur PARAMS ((Widget wdgt, XEvent *event, String *params,
-				   Cardinal *num_params));
-static void MA_movelook_r  PARAMS ((Widget wdgt, XEvent *event, String *params,
-				   Cardinal *num_params));
-static void MA_movelook_dr PARAMS ((Widget wdgt, XEvent *event, String *params,
-				   Cardinal *num_params));
-static void MA_center PARAMS ((Widget wdgt, XEvent *event, String *params,
-			      Cardinal *num_params));
-static void movelook_one_cell PARAMS ((Widget w, int xdelta, int ydelta));
-static void popup_ctrlpanel PARAMS ((Side *side, Map *map));
-static void popdown_ctrlpanel PARAMS ((Side *side, Map *map));
-static void create_ctrlpanel PARAMS ((Side *side, Map *map));
-static void ctrlpanel_cancel_callback PARAMS ((Widget w, XtPointer client_data,
-					      XtPointer call_data));
-static void ctrlpanel_revert_callback PARAMS ((Widget w, XtPointer client_data,
-					      XtPointer call_data));
-static void ctrlpanel_apply_callback PARAMS ((Widget w, XtPointer client_data,
-					     XtPointer call_data));
-static void ctrlpanel_done_callback PARAMS ((Widget w, XtPointer client_data,
-					    XtPointer call_data));
-static int set_from_controls PARAMS ((Side *side, Map *map));
+static void create_map_controls(Side *side, Map *map);
+static void interp_key_command(Side *side, Map *map);
+static void handle_map_click(Side *side, Map *map, int sx, int sy);
+static void help_unit_type(Side *side, Map *map);
+static void help_terrain_type(Side *side, Map *map);
+static void mode_callback(Widget w, XtPointer client_data,
+			  XtPointer call_data);
+static void command_callback(Widget w, XtPointer client_data,
+			     XtPointer call_data);
+static void view_flag_callback(Widget w, XtPointer client_data,
+			       XtPointer call_data);
+static void zoom_callback(Widget w, XtPointer client_data,
+			  XtPointer call_data);
+static void map_help_callback(Widget w, XtPointer client_data,
+			      XtPointer call_data);
+static void unit_type_list_callback(Widget w, XtPointer client_data,
+				    XtPointer call_data);
+static void panner_callback(Widget w, XtPointer client_data,
+			    XtPointer call_data);
+static void porthole_callback(Widget w, XtPointer client_data,
+			      XtPointer call_data);
+static void panner_resize_handler(Widget w, XtPointer client_data,
+				  XEvent *event, Boolean *cont);
+static void MA_keypress(Widget w, XEvent *event, String *params,
+			Cardinal *num_params);
+static void MA_mapexpose(Widget w, XEvent *event, String *params,
+			 Cardinal *num_params);
+static void MA_setcoord(Widget w, XEvent *event, String *params,
+			Cardinal *num_params);
+static void MA_movelook(Widget w, XEvent *event, String *params,
+			Cardinal *num_params);
+static void MA_moveunit(Widget w, XEvent *event, String *params,
+			Cardinal *num_params);
+static void MA_distance(Widget w, XEvent *event, String *params,
+			Cardinal *num_params);
+static void MA_motionsetcoord(Widget w, XEvent *event, String *params,
+			      Cardinal *num_params);
+static void MA_toolaction(Widget w, XEvent *event, String *params,
+			  Cardinal *num_params);
+static void MA_toolselection(Widget w, XEvent *event, String *params,
+			     Cardinal *num_params);
+static void MA_movelook_ul(Widget wdgt, XEvent *event, String *params,
+			   Cardinal *num_params);
+static void MA_movelook_l (Widget wdgt, XEvent *event, String *params,
+			   Cardinal *num_params);
+static void MA_movelook_dl(Widget wdgt, XEvent *event, String *params,
+			   Cardinal *num_params);
+static void MA_movelook_ur(Widget wdgt, XEvent *event, String *params,
+			   Cardinal *num_params);
+static void MA_movelook_r (Widget wdgt, XEvent *event, String *params,
+			   Cardinal *num_params);
+static void MA_movelook_dr(Widget wdgt, XEvent *event, String *params,
+			   Cardinal *num_params);
+static void MA_center(Widget wdgt, XEvent *event, String *params,
+		      Cardinal *num_params);
+static void movelook_one_cell(Widget w, int xdelta, int ydelta);
+static void popup_ctrlpanel(Side *side, Map *map);
+static void popdown_ctrlpanel(Side *side, Map *map);
+static void create_ctrlpanel(Side *side, Map *map);
+static void ctrlpanel_cancel_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data);
+static void ctrlpanel_revert_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data);
+static void ctrlpanel_apply_callback(Widget w, XtPointer client_data,
+				     XtPointer call_data);
+static void ctrlpanel_done_callback(Widget w, XtPointer client_data,
+				    XtPointer call_data);
+static int set_from_controls(Side *side, Map *map);
 
 XtActionsRec map_actions_table[] = {
     { "keypress", MA_keypress },
@@ -115,9 +114,7 @@ add_map_actions()
    of the different subareas. */
 
 static void
-subdivide_map(side, map)
-Side *side;
-Map *map;
+subdivide_map(Side *side, Map *map)
 {
     /* Subdivide the window into right and left halves. */
     if (map->leftfrac == 0)
@@ -233,20 +230,17 @@ char *geospec;
     map->vp->draw_people = FALSE;
     map->vp->draw_control = FALSE;
     map->vp->draw_elevations = FALSE;
-    map->drawfeatureboundaries = FALSE;
-    map->drawfeaturenames = FALSE;
-    /* cache the highest feature number; don't forget to update this 
-       if new featured are added (presently not implemented)  */
-    side->ui->numfeatures = num_features();
+    map->vp->draw_feature_boundaries = FALSE;
+    map->vp->draw_feature_names = FALSE;
     place_legends(side);
     for_all_material_types(m)
       map->vp->draw_materials[m] = FALSE;
     map->vp->draw_temperature = FALSE;
     map->vp->draw_clouds = FALSE;
     map->vp->draw_winds = FALSE;
-    map->seeall = all_see_all;
+    map->seeall = side->show_all;
     map->vp->draw_cover = FALSE;
-    map->vp->latlong_interval = side->ui->default_latlong_interval;
+    map->vp->meridian_interval = side->ui->default_meridian_interval;
     /* Start the map off in generic move mode normally. */
     map->curtool = movetool;
     map->inpch = '\0';
@@ -642,7 +636,7 @@ Map *map;
       XtVaCreateManagedWidget("all", toggleWidgetClass, map->controlform,
 			      XtNlabel, "All",
 			      XtNresize, False,
-			      XtNsensitive, !all_see_all,
+			      XtNsensitive, side->may_set_show_all,
 			      XtNwidth, wid,
 			      XtNfromVert, map->controls[ZOOM_OUT],
 			      XtNvertDistance, 10,
@@ -1191,7 +1185,7 @@ XEvent *evt;
 {
     char buf[4], ch;
     int nchar, cancelled;
-    void (*fn) PARAMS ((Side *sidex, Map *mapx, int cancelledx));
+    void (*fn)(Side *sidex, Map *mapx, int cancelledx);
 
     nchar = XLookupString(&(evt->xkey), buf, 4, NULL, NULL);
     if (nchar > 0) {
@@ -1255,7 +1249,10 @@ Map *map;
     } else {
 	clear_prompt(side, map);
 	map->frombutton = FALSE;
-	execute_command(side, map);
+	side->ui->curmap = map;
+	side->ui->beepcount = 0;
+	side->prefixarg = map->prefixarg;
+	execute_command(side, map->inpch);
 	/* Reset the argument for next command, unless the current one is still
 	 * waiting for further input. */
 	if (map->modalhandler == NULL)
@@ -1341,7 +1338,7 @@ Side *side;
 Map *map;
 char *prompt;
 int *possibles;
-void (*handler)();
+void (*handler)(Side *side, Map *map, int cancel);
 {
     int u, numtypes = 0;
 
@@ -1464,15 +1461,14 @@ Side *side;
 Map *map;
 char *prompt;
 int *possibles;
-void (*handler) PARAMS ((Side *side, Map *map, int cancelled));
+void (*handler)(Side *side, Map *map, int cancelled);
 {
     int numtypes = 0, t;
 
     for_all_terrain_types(t) {
 	if (possibles == NULL || possibles[t]) {
 	    map->tvec[numtypes] = t;
-	    map->tstr[numtypes] =
-	      (!empty_string(t_char(t)) ? t_char(t)[0] : (t - 'a'));
+	    map->tstr[numtypes] = terrchars[t];
 	    ++numtypes;
 	}
     }
@@ -1547,7 +1543,7 @@ ask_position(side, map, prompt, handler)
 Side *side;
 Map *map;
 char *prompt;
-void (*handler) PARAMS ((Side *side, Map *map, int cancelled));
+void (*handler)(Side *side, Map *map, int cancelled);
 {
     strcpy(map->prompt, prompt);
     map->answer[0] = '\0';
@@ -1586,7 +1582,7 @@ Side *side;
 Map *map;
 char *question;
 int dflt;
-void (*handler) PARAMS ((Side *side, Map *map, int cancelled));
+void (*handler)(Side *side, Map *map, int cancelled);
 {
     sprintf(map->prompt, "%s [%s]", question, (dflt ? "yn" : "ny"));
     map->answer[0] = '\0';
@@ -1619,7 +1615,7 @@ ask_string(side, map, prompt, dflt, handler)
 Side *side;
 Map *map;
 char *prompt, *dflt;
-void (*handler) PARAMS ((Side *side, Map *map, int cancelled));
+void (*handler)(Side *side, Map *map, int cancelled);
 {
     sprintf(map->prompt, "%s", prompt);
     /* Default must be non-NULL. */
@@ -1666,7 +1662,7 @@ Side *side;
 Map *map;
 char *prompt;
 Side *dfltside;
-void (*handler) PARAMS ((Side *side, Map *map, int cancel));
+void (*handler)(Side *side, Map *map, int cancel);
 {
     char *dfltstr;
 
@@ -1683,7 +1679,7 @@ Side *side;
 Map *map;
 Side **side2p;
 {
-    char ch = map->inpch, *str;
+    char ch = map->inpch;
     int len;
     Side *side3;
 
@@ -1732,10 +1728,7 @@ Side **side2p;
 /* Given a pixel in a map, describe what's there. */
 
 static void
-move_look(side, map, sx, sy)
-Side *side;
-Map *map;
-int sx, sy;
+move_look(Side *side, Map *map, int sx, int sy)
 {
     int nx, ny;
     Unit *unit;
@@ -1918,15 +1911,16 @@ XtPointer call_data;
     }
 
     map->frombutton = TRUE;
+    side->ui->curmap = map;
     switch (which) {
       case UNIT_MOVE:
-	do_move_to(side, map);
+	do_move_to(side);
 	break;
       case UNIT_SHOOT:
-	do_fire(side, map);
+	do_fire(side);
 	break;
       case UNIT_BUILD:
-	do_build(side, map);
+	do_build(side);
 	break;
       default:
 	case_panic("not a valid command button", which);
@@ -1967,7 +1961,7 @@ XtPointer call_data;
 	map->vp->draw_people = !map->vp->draw_people;
 	break;
       case SHOW_ALL:
-	if (!all_see_all)
+	if (side->may_set_show_all)
 	  map->seeall = !map->seeall;
 	else
 	  beep(side);
@@ -2019,7 +2013,7 @@ XtPointer call_data;
     if (!find_side_and_map_via_control(w, &side, &map))
       return;
 
-    do_help(side, map);
+    do_help(side);
     map->prefixarg = -1;
 }
 
@@ -2032,7 +2026,7 @@ XtPointer call_data;
     int which = (int) client_data;
     Side *side;
     Map *map;
-    void (*fn) PARAMS ((Side *sidex, Map *mapx, int cancelledx));
+    void (*fn)(Side *sidex, Map *mapx, int cancelledx);
     
     if (!find_side_and_map_via_listform(w, &side, &map))
       return;
@@ -2167,7 +2161,7 @@ int sx, sy;
 {
     int ax, ay;
     Unit *unit2;
-    void (*fn) PARAMS ((Side *sidex, Map *mapx, int cancelledx));
+    void (*fn)(Side *sidex, Map *mapx, int cancelledx);
 
     if (map == NULL) {
 	beep(side);
@@ -2269,7 +2263,7 @@ int sx, sy;
 	    if (distance(unit->x, unit->y, x, y) > 1) {
 		DGprintf("Ordering %s to move to %d,%d\n",
 			 unit_desig(unit), x, y);
-		net_set_move_to_task(unit, x, y);
+		net_set_move_to_task(unit, x, y, 0);
 		return;
 	    }
 
@@ -2300,7 +2294,7 @@ int sx, sy;
 		    /* (not quite right, move should happen after other unit
 		       is actually inside, in case it fills dest) */
 		    net_prep_enter_action(other, other, unit);
-		    net_set_move_to_task(unit, x, y);
+		    net_set_move_to_task(unit, x, y, 0);
 		} else if (can_occupy_cell(unit, x, y)) {
 		    net_prep_move_action(unit, unit, x, y, 0);
 		} else {
@@ -2638,10 +2632,10 @@ Map *map;
 			      NULL);
     map->ctrlpanel_buttons[SHOW_ALL] =
       XtVaCreateManagedWidget("seeAll", toggleWidgetClass, map->ctrlpanel_form,
-			      XtNsensitive, !all_see_all,
+			      XtNsensitive, side->may_set_show_all,
 			      XtNlabel, "See All",
 			      NULL);
-    map->ctrlpanel_buttons[SHOW_LATLONG] =
+    map->ctrlpanel_buttons[SHOW_MERIDIANS] =
       XtVaCreateManagedWidget("Meridians", toggleWidgetClass, map->ctrlpanel_form,
 			      NULL);
     label =
@@ -2720,7 +2714,7 @@ XtPointer call_data;
     update_controls(side, map);
 }
 
-int get_toggle_state PARAMS ((Widget w));
+int get_toggle_state(Widget w);
 
 int
 get_toggle_state(w)
@@ -2756,13 +2750,13 @@ Map *map;
       anychanged = TRUE;
     map->vp->draw_names = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_FEATURE_NAMES]);
-    if (newval != map->drawfeaturenames)
+    if (newval != map->vp->draw_feature_names)
       anychanged = TRUE;
-    map->drawfeaturenames = newval;
+    map->vp->draw_feature_names = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_FEATURE_BOUNDARIES]);
-    if (newval != map->drawfeatureboundaries)
+    if (newval != map->vp->draw_feature_boundaries)
       anychanged = TRUE;
-    map->drawfeatureboundaries = newval;
+    map->vp->draw_feature_boundaries = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_PEOPLE]);
     if (newval != map->vp->draw_people)
       anychanged = TRUE;
@@ -2795,10 +2789,10 @@ Map *map;
     if (newval != map->seeall)
       anychanged = TRUE;
     map->seeall = newval;
-    newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_LATLONG]);
-    if (newval != map->vp->draw_latlong)
+    newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_MERIDIANS]);
+    if (newval != map->vp->draw_meridians)
       anychanged = TRUE;
-    map->vp->draw_latlong = newval;
+    map->vp->draw_meridians = newval;
     /* (shouldn't be affecting all maps without redrawing all...) */
     newval = get_toggle_state(map->ctrlpanel_buttons[COLR_TERR]);
     if (newval != side->ui->dflt_color_terr_images)
@@ -2862,7 +2856,7 @@ Map *map;
       XtPopdown(map->ctrlpanel_shell);
 }
 
-void set_control_state PARAMS ((Side *side, Map *map, int, int, int));
+void set_control_state(Side *side, Map *map, int, int, int);
 
 void
 update_controls(side, map)
@@ -2881,9 +2875,9 @@ Map *map;
     set_control_state(side, map, SHOW_NAMES,
 		      TRUE, map->vp->draw_names);
     set_control_state(side, map, SHOW_FEATURE_NAMES,
-		      TRUE, map->drawfeaturenames);
+		      TRUE, map->vp->draw_feature_names);
     set_control_state(side, map, SHOW_FEATURE_BOUNDARIES,
-		      TRUE, map->drawfeatureboundaries);
+		      TRUE, map->vp->draw_feature_boundaries);
     set_control_state(side, map, SHOW_PEOPLE,
 		      people_sides_defined(), map->vp->draw_people);
     set_control_state(side, map, SHOW_CONTROL,
@@ -2899,9 +2893,9 @@ Map *map;
     set_control_state(side, map, SHOW_COVER,
 		      !map->seeall, map->vp->draw_cover);
     set_control_state(side, map, SHOW_ALL,
-		      side->may_set_see_all, map->seeall);
-    set_control_state(side, map, SHOW_LATLONG,
-		      world.circumference > 0, map->vp->draw_latlong);
+		      side->may_set_show_all, map->seeall);
+    set_control_state(side, map, SHOW_MERIDIANS,
+		      world.circumference > 0, map->vp->draw_meridians);
     set_control_state(side, map, SHOW_MORE,
 		      TRUE, map->fullpanel);
     set_control_state(side, map, ZOOM_OUT,
@@ -3087,10 +3081,9 @@ int u;
 }
 
 void
-place_legends(side)
-Side *side;
+place_legends(Side *side)
 {
-    int nf = side->ui->numfeatures;
+    int nf = numfeatures;
 
     if (!features_defined() || nf <= 0)
       return;
